@@ -13,11 +13,14 @@ from os.path import (
 )
 import platform
 
-VERSION = "4.0.6"
+VERSION = "4.5.0"
+
+homePage = "https://github.com/ilius/pyglossary"
 
 
 TRACE = 5
 logging.addLevelName(TRACE, "TRACE")
+noColor = False
 
 
 class Formatter(logging.Formatter):
@@ -142,16 +145,19 @@ def format_exception(
 
 
 class StdLogHandler(logging.Handler):
+	colorsConfig = {
+		"CRITICAL": ("color.cmd.critical", 196),
+		"ERROR": ("color.cmd.error", 1),
+		"WARNING": ("color.cmd.warning", 208),
+	}
+	# 1: dark red (like 31m), 196: real red, 9: light red
+	# 15: white, 229: light yellow (#ffffaf), 226: real yellow (#ffff00)
+
 	def __init__(self, noColor: bool = False):
 		logging.Handler.__init__(self)
 		self.set_name("std")
 		self.noColor = noColor
-
-	@property
-	def startRed(self):
-		if self.noColor:
-			return ""
-		return "\x1b[31m"
+		self.config = {}
 
 	@property
 	def endFormat(self):
@@ -168,7 +174,7 @@ class StdLogHandler(logging.Handler):
 			_type, value, tback = record.exc_info
 			tback_text = format_exception(
 				exc_info=record.exc_info,
-				add_locals=(log.level <= logging.DEBUG),  # FIXME
+				add_locals=(log.level <= logging.DEBUG),
 				add_globals=False,
 			)
 
@@ -177,13 +183,24 @@ class StdLogHandler(logging.Handler):
 			msg += "\n"
 			msg += tback_text
 		###
-		if record.levelname in ("CRITICAL", "ERROR"):
-			if msg and not self.noColor:
-				msg = self.startRed + msg + self.endFormat
+		levelname = record.levelname
+
+		if levelname in ("CRITICAL", "ERROR"):
 			fp = sys.stderr
 		else:
 			fp = sys.stdout
+
+		if not self.noColor and levelname in self.colorsConfig:
+			key, default = self.colorsConfig[levelname]
+			colorCode = self.config.get(key, default)
+			startColor = f"\x1b[38;5;{colorCode}m"
+			msg = startColor + msg + self.endFormat
+
 		###
+		if fp is None:
+			print(f"fp=None, levelname={record.levelname}")
+			print(msg)
+			return
 		fp.write(msg + "\n")
 		fp.flush()
 
@@ -194,24 +211,99 @@ def checkCreateConfDir() -> None:
 			os.rename(confDir, confDir + ".bak")  # we do not import old config
 		os.mkdir(confDir)
 	if not exists(userPluginsDir):
-		os.mkdir(userPluginsDir)
+		try:
+			os.mkdir(userPluginsDir)
+		except Exception as e:
+			log.warning(f"failed to create user plugins directory: {e}")
 	if not isfile(confJsonFile):
 		with open(rootConfJsonFile) as srcF, open(confJsonFile, "w") as usrF:
 			usrF.write(srcF.read())
 
 
+
+def in_virtualenv():
+	if hasattr(sys, 'real_prefix'):
+		return True
+	if hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix:
+		return True
+	return False
+
+
+def getDataDir():
+	if in_virtualenv():
+		pass # TODO
+		# print(f"prefix={sys.prefix}, base_prefix={sys.base_prefix}")
+		# return join(
+		# 	dirname(dirname(dirname(rootDir))),
+		# 	os.getenv("VIRTUAL_ENV"), "share", "pyglossary",
+		# )
+
+	if not (
+		rootDir.endswith("dist-packages") or
+		rootDir.endswith("site-packages")
+	):
+		return rootDir
+
+	parent3 = dirname(dirname(dirname(rootDir)))
+	if os.sep == "/":
+		return join(parent3, "share", "pyglossary")
+
+	_dir = join(
+		parent3,
+		f"Python{sys.version_info.major}{sys.version_info.minor}",
+		"share",
+		"pyglossary",
+	)
+	if isdir(_dir):
+		return _dir
+
+	_dir = join(parent3, "Python3", "share", "pyglossary")
+	if isdir(_dir):
+		return _dir
+
+	_dir = join(parent3, "Python", "share", "pyglossary")
+	if isdir(_dir):
+		return _dir
+
+	_dir = join(sys.prefix, "share", "pyglossary")
+	if isdir(_dir):
+		return _dir
+
+	if os.getenv("CONDA_PREFIX"):
+		_dir = join(os.getenv("CONDA_PREFIX"), "share", "pyglossary")
+		if isdir(_dir):
+			return _dir
+
+	raise OSError("failed to detect dataDir")
+
+
+def windows_show_exception(*exc_info):
+	import ctypes
+	msg = format_exception(
+		exc_info=exc_info,
+		add_locals=(log.level <= logging.DEBUG),
+		add_globals=False,
+	)
+	log.critical(msg)
+	ctypes.windll.user32.MessageBoxW(0, msg, "PyGlossary Error", 0)
+
 # __________________________________________________________________________ #
 
+
 logging.setLoggerClass(MyLogger)
 log = logging.getLogger("pyglossary")
 
-sys.excepthook = lambda *exc_info: log.critical(
-	format_exception(
-		exc_info=exc_info,
-		add_locals=(log.level <= logging.DEBUG),  # FIXME
-		add_globals=False,
+if os.sep == "\\":
+	sys.excepthook = windows_show_exception
+else:
+	sys.excepthook = lambda *exc_info: log.critical(
+		format_exception(
+			exc_info=exc_info,
+			add_locals=(log.level <= logging.DEBUG),
+			add_globals=False,
+		)
 	)
-)
+
 
 sysName = platform.system().lower()
 # platform.system() is in	["Linux", "Windows", "Darwin", "FreeBSD"]
@@ -225,8 +317,8 @@ if os.getenv("WARNINGS"):
 	warnings.filterwarnings(os.getenv("WARNINGS"))
 
 
-if hasattr(sys, "frozen"):
-	# FIXME: was this for py2exe?
+if getattr(sys, "frozen", False):
+	# PyInstaller frozen executable
 	log.info(f"sys.frozen = {sys.frozen}")
 	rootDir = dirname(sys.executable)
 	uiDir = join(rootDir, "pyglossary", "ui")
@@ -235,21 +327,13 @@ else:
 	uiDir = join(_srcDir, "ui")
 	rootDir = dirname(_srcDir)
 
-dataDir = rootDir
-if dataDir.endswith("dist-packages") or dataDir.endswith("site-packages"):
-	parent3 = dirname(dirname(dirname(rootDir)))
-	if os.sep == "\\":
-		dataDir = join(parent3, "Python", "share", "pyglossary")
-	else:
-		dataDir = join(parent3, "share", "pyglossary")
-
+dataDir = getDataDir()
 appResDir = join(dataDir, "res")
 
 if os.sep == "/":  # Operating system is Unix-Like
 	homeDir = os.getenv("HOME")
 	user = os.getenv("USER")
-	tmpDir = "/tmp"
-	# os.name == "posix" # FIXME
+	tmpDir = os.getenv("TMPDIR", "/tmp")
 	if sysName == "darwin":  # MacOS X
 		_libDir = join(homeDir, "Library")
 		confDir = join(_libDir, "Preferences", "PyGlossary")
@@ -260,16 +344,26 @@ if os.sep == "/":  # Operating system is Unix-Like
 		# platform.release() == "10.3.0"
 		cacheDir = join(_libDir, "Caches", "PyGlossary")
 		pip = "pip3"
-	else:  # GNU/Linux, ...
+	else:  # GNU/Linux, Termux, FreeBSD, etc
+		# should switch to "$XDG_CONFIG_HOME/pyglossary" in version 5.0.0
+		# which generally means ~/.config/pyglossary
 		confDir = join(homeDir, ".pyglossary")
 		cacheDir = join(homeDir, ".cache", "pyglossary")
-		pip = "sudo pip3"
+		if "/com.termux/" in homeDir:
+			pip = "pip3"
+		else:
+			pip = "sudo pip3"
 elif os.sep == "\\":  # Operating system is Windows
 	homeDir = join(os.getenv("HOMEDRIVE"), os.getenv("HOMEPATH"))
 	user = os.getenv("USERNAME")
 	tmpDir = os.getenv("TEMP")
-	confDir = join(os.getenv("APPDATA"), "PyGlossary")
-	cacheDir = join(confDir, "Cache")  # FIXME: right directory?
+	_appData = os.getenv("APPDATA")
+	confDir = join(_appData, "PyGlossary")
+	_localAppData = os.getenv("LOCALAPPDATA")
+	if not _localAppData:
+		# Windows Vista or older
+		_localAppData = abspath(join(_appData, "..", "Local"))
+	cacheDir = join(_localAppData, "PyGlossary", "Cache")
 	pip = "pip3"
 else:
 	raise RuntimeError(
@@ -277,6 +371,7 @@ else:
 		f", unknown operating system!"
 	)
 
+pluginsDir = join(rootDir, "pyglossary", "plugins")
 confJsonFile = join(confDir, "config.json")
 rootConfJsonFile = join(dataDir, "config.json")
 userPluginsDir = join(confDir, "plugins")
