@@ -38,5 +38,25 @@ class MockLogHandler(logging.Handler):
 				continue
 			for record in self.recordsByLevel[level]:
 				count += 1
-				print(self.format(record))
+				print(repr(self.format(record)))
 		return count
+
+
+mockLog = None
+
+
+def getMockLogger():
+	global mockLog
+
+	if mockLog is not None:
+		return mockLog
+
+	log = logging.getLogger("pyglossary")
+
+	for handler in log.handlers:
+		log.removeHandler(handler)
+
+	mockLog = MockLogHandler()
+	mockLog.setLevel(logging.WARNING)
+	log.addHandler(mockLog)
+	return mockLog
