@@ -31,7 +31,7 @@ from datetime import datetime
 import shutil
 
 from pyglossary.text_utils import toStr, toBytes
-from pyglossary.os_utils import indir
+from pyglossary.os_utils import indir, rmtree
 
 import logging
 log = logging.getLogger("pyglossary")
@@ -79,7 +79,7 @@ class EbookWriter(object):
 	_group_by_prefix_length: int = 2
 	_include_index_page: bool = False
 	_compress: bool = True
-	_apply_css: str = ""  # path to css file, or ""
+	_css: str = ""  # path to css file, or ""
 	_cover_path: str = ""  # path to cover file, or ""
 
 	CSS_CONTENTS = ""
@@ -161,10 +161,9 @@ class EbookWriter(object):
 		if mode is None:
 			mode = zipfile.ZIP_DEFLATED
 		file_path = os.path.join(self._tmpDir, relative_path)
-		file_obj = self.myOpen(file_path, "wb")
 		contents = toBytes(contents)
-		file_obj.write(contents)
-		file_obj.close()
+		with self.myOpen(file_path, "wb") as file_obj:
+			file_obj.write(contents)
 		self.files.append({
 			"path": relative_path,
 			"mode": mode,
@@ -172,9 +171,8 @@ class EbookWriter(object):
 
 	def write_cover(self, cover_path):
 		basename = os.path.basename(cover_path)
-		cover_obj = self.myOpen(cover_path, "rb")
-		cover = cover_obj.read()
-		cover_obj.close()
+		with self.myOpen(cover_path, "rb") as cover_obj:
+			cover = cover_obj.read()
 		b = basename.lower()
 		mimetype = "image/jpeg"
 		if b.endswith(".png"):
@@ -188,9 +186,8 @@ class EbookWriter(object):
 		css = self.CSS_CONTENTS
 		if custom_css_path_absolute is not None:
 			try:
-				css_obj = self.myOpen(custom_css_path_absolute, "rb")
-				css = css_obj.read()
-				css_obj.close()
+				with self.myOpen(custom_css_path_absolute, "rb") as css_obj:
+					css = css_obj.read()
 			except Exception:
 				log.exception("")
 		self.add_file_manifest("OEBPS/style.css", "style.css", css, "text/css")
@@ -211,21 +208,10 @@ class EbookWriter(object):
 		return f"g{index:06d}.xhtml"
 
 	def get_prefix(self, word: str) -> str:
-		if not word:
-			return None
-		length = self._group_by_prefix_length
-		prefix = word[:length].lower()
-		if prefix[0] < "a":
-			return "SPECIAL"
-		return prefix
+		raise NotImplementedError
 
-	def sortKey(self, b_word: bytes) -> "Any":
-		# DO NOT change method name
-		word = b_word.decode("utf-8")
-		return (
-			self.get_prefix(word),
-			word,
-		)
+	def sortKey(self, words: "List[str]") -> "Any":
+		raise NotImplementedError
 
 	def write_groups(self):
 		# TODO: rtl=False option
@@ -271,6 +257,13 @@ class EbookWriter(object):
 			if entry is None:
 				break
 			if entry.isData():
+				if entry.getFileName() == "style.css":
+					self.add_file_manifest(
+						"OEBPS/style.css",
+						"style.css",
+						entry.data.decode("utf-8"),
+						"text/css",
+					)
 				continue
 
 			prefix = self.get_prefix(entry.s_word)
@@ -337,7 +330,7 @@ class EbookWriter(object):
 			sourceLang=self._glos.sourceLangName,
 			targetLang=self._glos.targetLangName,
 			title=self._glos.getInfo("name"),
-			creator=self._glos.getAuthor(),
+			creator=self._glos.author,
 			copyright=self._glos.getInfo("copyright"),
 			creationDate=creationDate,
 			cover=cover,
@@ -383,15 +376,15 @@ class EbookWriter(object):
 		filename = self._filename
 		# self._group_by_prefix_length
 		# self._include_index_page
-		apply_css = self._apply_css
+		css = self._css
 		cover_path = self._cover_path
 
 		with indir(self._tmpDir):
 			if cover_path:
 				cover_path = os.path.abspath(cover_path)
 
-			if apply_css:
-				apply_css = os.path.abspath(apply_css)
+			if css:
+				css = os.path.abspath(css)
 
 			os.makedirs("META-INF")
 			os.makedirs("OEBPS")
@@ -407,8 +400,8 @@ class EbookWriter(object):
 				except Exception:
 					log.exception("")
 
-			if apply_css:
-				self.write_css(apply_css)
+			if css:
+				self.write_css(css)
 
 			yield from self.write_groups()
 			group_labels = self._group_labels
@@ -433,10 +426,7 @@ class EbookWriter(object):
 					)
 				zipFp.close()
 				if not self._keep:
-					try:
-						shutil.rmtree(self._tmpDir)
-					except Exception:
-						log.exception(f"error removing {self._tmpDir}/")
+					rmtree(self._tmpDir)
 			else:
 				if self._keep:
 					shutil.copytree(self._tmpDir, filename)
