@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 import re
-from tempfile import mktemp
 import shutil
 import os
 from os.path import (
@@ -12,6 +11,9 @@ from os.path import (
 
 from .entry_base import BaseEntry, MultiStr, RawEntryType
 from .iter_utils import unique_everseen
+from .text_utils import (
+	joinByBar,
+)
 
 from pickle import dumps, loads
 from zlib import compress, decompress
@@ -35,21 +37,17 @@ class DataEntry(BaseEntry):
 	def __init__(
 		self,
 		fname: str,
-		data: bytes,
-		inTmp: bool = False,
+		data: bytes = b"",
+		tmpPath: "Optional[str]" = None,
 		byteProgress: "Optional[Tuple[int, int]]" = None,
 	) -> None:
 		assert isinstance(fname, str)
 		assert isinstance(data, bytes)
-		assert isinstance(inTmp, bool)
 
-		if inTmp:
-			tmpPath = mktemp(prefix=fname + "_")
+		if data and tmpPath:
 			with open(tmpPath, "wb") as toFile:
 				toFile.write(data)
-			data = ""
-		else:
-			tmpPath = None
+			data = b""
 
 		self._fname = fname
 		self._data = data  # bytes instance
@@ -62,8 +60,8 @@ class DataEntry(BaseEntry):
 	@property
 	def data(self) -> bytes:
 		if self._tmpPath:
-			with open(self._tmpPath, "rb") as fromFile:
-				return fromFile.read()
+			with open(self._tmpPath, "rb") as _file:
+				return _file.read()
 		else:
 			return self._data
 
@@ -86,6 +84,8 @@ class DataEntry(BaseEntry):
 			else:
 				with open(fpath, "wb") as toFile:
 					toFile.write(self._data)
+		except FileNotFoundError as e:
+			log.error(f"error in DataEntry.save: {e}")
 		except Exception:
 			log.exception(f"error while saving {fpath}")
 			return ""
@@ -148,24 +148,17 @@ class DataEntry(BaseEntry):
 		if glos.tmpDataDir:
 			b_fpath = self.save(glos.tmpDataDir).encode("utf-8")
 		tpl = (
-			self._fname.encode("utf-8"),
+			[self._fname],
 			b_fpath,
 			"b",
 		)
-		if glos._rawEntryCompress:
+		if glos.rawEntryCompress:
 			return compress(dumps(tpl), level=9)
 		return tpl
 
-	@classmethod
-	def fromFile(cls, glos, relPath, fullPath):
-		entry = DataEntry(relPath, b"")
-		entry._tmpPath = fullPath
-		return entry
-
 
 class Entry(BaseEntry):
-	sep = "|"
-	b_sep = b"|"
+	xdxfPattern = re.compile("^<k>[^<>]*</k>", re.S | re.I)
 	htmlPattern = re.compile(
 		".*(?:" + "|".join([
 			r"<font[ >]",
@@ -174,7 +167,7 @@ class Entry(BaseEntry):
 			r"<b[ >]",
 			r"<p[ >]",
 			r"<hr\s*/?\s*>",
-			r"<a href=",
+			r"<a ",  # or r"<a [^<>]*href="
 			r"<div[ >]",
 			r"<span[ >]",
 			r"<img[ >]",
@@ -199,51 +192,20 @@ class Entry(BaseEntry):
 	def isData(self) -> bool:
 		return False
 
-	def _join(self, parts: "List[str]") -> str:
-		return Entry.sep.join([
-			part.replace(Entry.sep, "\\" + Entry.sep)
-			for part in parts
-		])
-
-	@staticmethod
-	def defaultStringSortKey(word: str) -> "Any":
-		return Entry.defaultSortKey(word.encode("utf-8"))
-
-	@staticmethod
-	def defaultSortKey(b_word: bytes) -> "Any":
-		return b_word.lower()
-
-	@staticmethod
-	def getEntrySortKey(
-		key: "Optional[Callable[[bytes], Any]]" = None,
-	) -> "Callable[[BaseEntry], Any]":
-		if key is None:
-			key = Entry.defaultSortKey
-		return lambda entry: key(entry.l_word[0].encode("utf-8"))
-
 	@staticmethod
 	def getRawEntrySortKey(
 		glos: "GlossaryType",
-		key: "Optional[Callable[[bytes], Any]]" = None,
+		key: "Callable[[bytes], Any]",
 	) -> "Callable[[Tuple], Any]":
 		# here `x` is raw entity, meaning a tuple of form (word, defi) or
 		# (word, defi, defiFormat)
 		# so x[0] is word(s) in bytes, that can be a str (one word),
 		# or a list or tuple (one word with or more alternaties)
-		if key is None:
-			key = Entry.defaultSortKey
-
-		if glos.getConfig("enable_alts", True):
-			b_sep = Entry.b_sep
-			if glos._rawEntryCompress:
-				return lambda x: key(loads(decompress(x))[0].split(b_sep)[0])
-			else:
-				return lambda x: key(x[0].split(b_sep)[0])
+		if glos.rawEntryCompress:
+			return lambda x: key(loads(decompress(x))[0])
 		else:
-			if glos._rawEntryCompress:
-				return lambda x: key(loads(decompress(x))[0])
-			else:
-				return lambda x: key(x[0])
+			# x is rawEntry, so x[0] is list of words (entry.l_word)
+			return lambda x: key(x[0])
 
 	def __init__(
 		self,
@@ -298,7 +260,7 @@ class Entry(BaseEntry):
 		if isinstance(self._word, str):
 			return self._word
 		else:
-			return self._join(self._word)
+			return joinByBar(self._word)
 
 	@property
 	def l_word(self) -> "List[str]":
@@ -341,8 +303,12 @@ class Entry(BaseEntry):
 	def detectDefiFormat(self) -> None:
 		if self._defiFormat != "m":
 			return
+		if Entry.xdxfPattern.match(self.defi):
+			self._defiFormat = "x"
+			return
 		if Entry.htmlPattern.match(self.defi):
 			self._defiFormat = "h"
+			return
 
 	def byteProgress(self):
 		return self._byteProgress
@@ -433,7 +399,7 @@ class Entry(BaseEntry):
 		word = self.s_word
 		i = defi.find('<body')
 		if i == -1:
-			log.error(f"<body not found: word={word}")
+			log.warning(f"<body not found: word={word}")
 			return
 		defi = defi[i + 5:]
 		i = defi.find('>')
@@ -458,17 +424,17 @@ class Entry(BaseEntry):
 		"""
 		if self._defiFormat and self._defiFormat != glos.getDefaultDefiFormat():
 			tpl = (
-				self.b_word,
+				self.l_word,
 				self.b_defi,
 				self._defiFormat,
 			)
 		else:
 			tpl = (
-				self.b_word,
+				self.l_word,
 				self.b_defi,
 			)
 
-		if glos._rawEntryCompress:
+		if glos.rawEntryCompress:
 			return compress(dumps(tpl), level=9)
 
 		return tpl
@@ -489,18 +455,18 @@ class Entry(BaseEntry):
 		"""
 		if isinstance(rawEntry, bytes):
 			rawEntry = loads(decompress(rawEntry))
-		word = rawEntry[0].decode("utf-8")
+		word = rawEntry[0]
 		defi = rawEntry[1].decode("utf-8")
 		if len(rawEntry) > 2:
 			defiFormat = rawEntry[2]
 			if defiFormat == "b":
-				return DataEntry.fromFile(glos, word, defi)
+				fname = word
+				if isinstance(fname, list):
+					fname = fname[0]
+				return DataEntry(fname, tmpPath=defi)
 		else:
 			defiFormat = defaultDefiFormat
 
-		if glos.getConfig("enable_alts", True):
-			word = word.split(cls.sep)
-
 		return cls(
 			word,
 			defi,
