@@ -8,7 +8,6 @@ from .text_utils import (
 )
 
 from .entry_base import BaseEntry
-from .glossary import Glossary
 
 
 log = logging.getLogger("pyglossary")
@@ -27,7 +26,7 @@ class EntryFilter(object):
 		"""
 		pass
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		"""
 			returns an Entry object, or None to skip
 				may return the same `entry`,
@@ -37,11 +36,11 @@ class EntryFilter(object):
 		return entry
 
 
-class StripEntryFilter(EntryFilter):
+class StripWhitespaces(EntryFilter):
 	name = "strip"
-	desc = "Strip Whitespaces"
+	desc = "Strip whitespaces in word(s) and definition"
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		entry.strip()
 		entry.replace("\r", "")
 		return entry
@@ -49,9 +48,9 @@ class StripEntryFilter(EntryFilter):
 
 class NonEmptyWordFilter(EntryFilter):
 	name = "non_empty_word"
-	desc = "Non-empty Words"
+	desc = "Skip entries with empty word"
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		if not entry.s_word:
 			return
 		return entry
@@ -59,9 +58,9 @@ class NonEmptyWordFilter(EntryFilter):
 
 class NonEmptyDefiFilter(EntryFilter):
 	name = "non_empty_defi"
-	desc = "Non-empty Definition"
+	desc = "Skip entries with empty definition"
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		if not entry.defi:
 			return
 		return entry
@@ -69,36 +68,57 @@ class NonEmptyDefiFilter(EntryFilter):
 
 class RemoveEmptyAndDuplicateAltWords(EntryFilter):
 	name = "remove_empty_dup_alt_words"
-	desc = "Remove Empty and Duplicate Alternate Words"
+	desc = "Remove empty and duplicate alternate words"
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		entry.removeEmptyAndDuplicateAltWords()
 		if not entry.l_word:
 			return
 		return entry
 
 
-class FixUnicodeFilter(EntryFilter):
-	name = "fix_unicode"
-	desc = "Fix Unicode"
+class FixUnicode(EntryFilter):
+	name = "utf8_check"
+	desc = "Fix Unicode in word(s) and definition"
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		entry.editFuncWord(fixUtf8)
 		entry.editFuncDefi(fixUtf8)
 		return entry
 
 
-class LowerWordFilter(EntryFilter):
-	name = "lower_word"
-	desc = "Lowercase Words"
+class LowerWord(EntryFilter):
+	name = "lower"
+	desc = "Lowercase word(s)"
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def __init__(self, glos: "GlossaryType"):
+		EntryFilter.__init__(self, glos)
+		self._re_word_ref = re.compile('href=["\'](bword://[^"\']+)["\']')
+
+	def lowerWordRefs(self, defi):
+		return self._re_word_ref.sub(
+			lambda m: m.group(0).lower(),
+			defi,
+		)
+
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		entry.editFuncWord(str.lower)
+		entry.editFuncDefi(self.lowerWordRefs)
+		return entry
+
+
+class RTLDefi(EntryFilter):
+	name = "rtl"
+	desc = "Make definition right-to-left"
+
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
+		entry.editFuncDefi(lambda defi: f'<div dir="rtl">{defi}</div>')
 		return entry
 
 
 class RemoveHtmlTagsAll(EntryFilter):
 	name = "remove_html_all"
+	desc = "Remove all HTML tags from definition"
 
 	def __init__(self, glos: "GlossaryType"):
 		self._p_pattern = re.compile(
@@ -110,7 +130,7 @@ class RemoveHtmlTagsAll(EntryFilter):
 			re.IGNORECASE,
 		)
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		from bs4 import BeautifulSoup
 
 		def fixStr(st: str) -> str:
@@ -126,15 +146,17 @@ class RemoveHtmlTagsAll(EntryFilter):
 
 class RemoveHtmlTags(EntryFilter):
 	name = "remove_html"
+	desc = "Remove specific HTML tags from definition"
 
-	def __init__(self, glos: "GlossaryType", tags: "List[str]"):
+	def __init__(self, glos: "GlossaryType", tagsStr: str):
 		import re
+		tags = tagsStr.split(",")
 		self.glos = glos
 		self.tags = tags
 		tagsRE = "|".join(self.tags)
-		self.pattern = re.compile(f"<.?({tagsRE})[^>]*>")
+		self.pattern = re.compile(f"</?({tagsRE})( [^>]*)?>")
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		def fixStr(st: str) -> str:
 			return self.pattern.sub("", st)
 
@@ -142,11 +164,12 @@ class RemoveHtmlTags(EntryFilter):
 		return entry
 
 
-# FIXME: this may not be safe as it lowercases everything between < and >
-# including class name, element ids/names, scripts etc. how can we fix that?
+# FIXME: It's is not safe to lowercases everything between < and >
+# including class name, element ids/names, scripts, <a href="bword://...">
+# etc. How can we fix that?
 class NormalizeHtml(EntryFilter):
 	name = "normalize_html"
-	desc = "Normalize HTML Tags"
+	desc = "Normalize HTML tags in definition (WIP)"
 
 	def __init__(self, glos: "GlossaryType"):
 		log.info("Normalizing HTML tags")
@@ -172,24 +195,24 @@ class NormalizeHtml(EntryFilter):
 		st = self._pattern.sub(self._subLower, st)
 		return st
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		entry.editFuncDefi(self._fixDefi)
 		return entry
 
 
-class SkipDataEntryFilter(EntryFilter):
+class SkipDataEntry(EntryFilter):
 	name = "skip_resources"
-	desc = "Skip Resources"
+	desc = "Skip resources / data files"
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		if entry.isData():
 			return
 		return entry
 
 
-class LangEntryFilter(EntryFilter):
+class LanguageCleanup(EntryFilter):
 	name = "lang"
-	desc = "Language-dependent Filters"
+	desc = "Language-specific cleanup/fixes"
 
 	def __init__(self, glos: "GlossaryType"):
 		EntryFilter.__init__(self, glos)
@@ -214,15 +237,22 @@ class LangEntryFilter(EntryFilter):
 		# for GoldenDict ^^ FIXME
 		return entry
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		if self._run_func:
 			entry = self._run_func(entry)
 		return entry
 
 
-class CleanEntryFilter(EntryFilter):  # FIXME
-	name = "clean"
-	desc = "Clean"
+class TextListSymbolCleanup(EntryFilter):
+	"""
+	Symbols like ♦ (diamond) ● (black circle) or * (star) are used in some
+	plaintext or even html glossaries to represent items of a list
+	(like <li> in proper html).
+	This EntryFilter cleans up spaces/newlines issues around them.
+	"""
+
+	name = "text_list_symbol_cleanup"
+	desc = "Text List Symbol Cleanup"
 
 	winNewlinePattern = re.compile("[\r\n]+")
 	spacesNewlinePattern = re.compile(" *\n *")
@@ -233,21 +263,6 @@ class CleanEntryFilter(EntryFilter):  # FIXME
 		st = self.winNewlinePattern.sub("\n", st)
 		st = self.spacesNewlinePattern.sub("\n", st)
 
-		"""
-		This code may correct snippets like:
-		- First sentence .Second sentence. -> First sentence. Second sentence.
-		- First clause ,second clause. -> First clause, second clause.
-		But there are cases when this code have undesirable effects
-		( "<" represented as "&lt;" in HTML markup):
-		- <Adj.> -> < Adj. >
-		- <fig.> -> < fig. >
-		"""
-		"""
-		for j in range(3):
-			for ch in ",.;":
-				st = replacePostSpaceChar(st, ch)
-		"""
-
 		st = self.blocksNewlinePattern.sub("♦", st)
 		if st.endswith("<p"):
 			st = st[:-2]
@@ -257,12 +272,43 @@ class CleanEntryFilter(EntryFilter):  # FIXME
 
 		return st
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		entry.editFuncDefi(self.cleanDefi)
 		return entry
 
 
-class ProgressBarEntryFilter(EntryFilter):
+class PreventDuplicateWords(EntryFilter):
+	name = "prevent_duplicate_words"
+	desc = "Prevent duplicate words"
+
+	def __init__(self, glos: "GlossaryType"):
+		EntryFilter.__init__(self, glos)
+		self._wordSet = set()
+
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
+		if entry.isData():
+			return entry
+
+		wordSet = self._wordSet
+		word = entry.s_word
+
+		if word not in wordSet:
+			wordSet.add(word)
+			return entry
+
+		n = 2
+		while f"{word} ({n})" in wordSet:
+			n += 1
+		word = f"{word} ({n})"
+
+		wordSet.add(word)
+		entry._word = word
+		# use entry.editFuncWord?
+
+		return entry
+
+
+class ShowProgressBar(EntryFilter):
 	name = "progressbar"
 	desc = "Progress Bar"
 
@@ -271,8 +317,12 @@ class ProgressBarEntryFilter(EntryFilter):
 		self._wordCount = -1
 		self._wordCountThreshold = 0
 		self._lastPos = 0
+		self._index = 0
+
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
+		index = self._index
+		self._index = index + 1
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
 		if entry is not None:
 			bp = entry.byteProgress()
 			if bp:
@@ -283,7 +333,10 @@ class ProgressBarEntryFilter(EntryFilter):
 
 		if self._wordCount == -1:
 			self._wordCount = len(self.glos)
-			self._wordCountThreshold = self.glos._calcProgressThreshold(self._wordCount)
+			self._wordCountThreshold = max(1, min(
+				500,
+				self._wordCount // 200,
+			))
 
 		if self._wordCount > 1:
 			if index % self._wordCountThreshold == 0:
@@ -292,7 +345,7 @@ class ProgressBarEntryFilter(EntryFilter):
 		return entry
 
 
-class MaxMemoryUsageEntryFilter(EntryFilter):
+class ShowMaxMemoryUsage(EntryFilter):
 	name = "max_memory_usage"
 	desc = "Show Max Memory Usage"
 
@@ -300,7 +353,7 @@ class MaxMemoryUsageEntryFilter(EntryFilter):
 		EntryFilter.__init__(self, glos)
 		self._max_mem_usage = 0
 
-	def run(self, entry: BaseEntry, index: int) -> "Optional[BaseEntry]":
+	def run(self, entry: BaseEntry) -> "Optional[BaseEntry]":
 		import os
 		import psutil
 		usage = psutil.Process(os.getpid()).memory_info().rss // 1024
