@@ -6,6 +6,11 @@ from .langs import Lang
 
 
 class GlossaryType(object):
+	"""
+	an abstract type class for Glossary class in plugins. it only
+	contains methods and properties that might be used in plugins
+	"""
+
 	def setDefaultDefiFormat(self, defiFormat: str) -> None:
 		raise NotImplementedError
 
@@ -30,7 +35,16 @@ class GlossaryType(object):
 	def getExtraInfos(self, excludeKeys: "List[str]") -> "OrderedDict":
 		raise NotImplementedError
 
-	def getAuthor(self) -> str:
+	@property
+	def author(self) -> str:
+		raise NotImplementedError
+
+	@property
+	def alts(self) -> bool:
+		raise NotImplementedError
+
+	@property
+	def filename(self):
 		raise NotImplementedError
 
 	@property
@@ -57,6 +71,21 @@ class GlossaryType(object):
 	def targetLangName(self, langName: str) -> None:
 		raise NotImplementedError
 
+	def titleElement(
+		self,
+		hf: "lxml.etree.htmlfile",
+		sample: str = "",
+	) -> "lxml.etree._FileWriterElement":
+		raise NotImplementedError
+
+	def wordTitleStr(
+		self,
+		word: str,
+		sample: str = "",
+		_class: str = "",
+	) -> str:
+		raise NotImplementedError
+
 	def getConfig(self, name: str, default: "Optional[str]") -> "Optional[str]":
 		raise NotImplementedError
 
@@ -68,29 +97,3 @@ class GlossaryType(object):
 
 	def newDataEntry(self, fname: str, data: bytes) -> DataEntry:
 		raise NotImplementedError
-
-	def writeTxt(
-		self,
-		entryFmt: str = "",  # contain {word} and {defi}
-		filename: str = "",
-		fileObj: "Optional[file]" = None,
-		writeInfo: bool = True,
-		wordEscapeFunc: "Optional[Callable]" = None,
-		defiEscapeFunc: "Optional[Callable]" = None,
-		ext: str = ".txt",
-		head: str = "",
-		tail: str = "",
-		outInfoKeysAliasDict: "Optional[Dict[str, str]]" = None,
-		encoding: str = "utf-8",
-		newline: str = "\n",
-		resources: bool = True,
-	) -> "Generator[None, BaseEntry, None]":
-		raise NotImplementedError
-
-	def writeTabfile(
-		self,
-		filename: str = "",
-		fileObj: "Optional[file]" = None,
-		**kwargs,
-	) -> "Generator[None, BaseEntry, None]":
-		raise NotImplementedError
