@@ -59,7 +59,6 @@ monthNameAb = (
 )
 
 
-
 epoch = 1721426
 
 options = ()
@@ -85,14 +84,14 @@ def to_jd(year: int, month: int, day: int) -> int:
 		tm = -2
 
 	return (
-		epoch - 1
-		+ 365 * (year - 1)
-		+ (year - 1) // 4
-		- (year - 1) // 100
-		+ (year - 1) // 400
-		+ (367 * month - 362) // 12
-		+ tm
-		+ day
+		epoch - 1 +
+		365 * (year - 1) +
+		(year - 1) // 4 +
+		-((year - 1) // 100) +
+		(year - 1) // 400 +
+		(367 * month - 362) // 12 +
+		tm +
+		day
 	)
 
 
@@ -109,11 +108,11 @@ def jd_to(jd: int) -> "Tuple[int, int, int]":
 	quad, dquad = divmod(dcent, 1461)
 	yindex = dquad // 365  # divmod(dquad, 365)[0]
 	year = (
-		qc * 400
-		+ cent * 100
-		+ quad * 4
-		+ yindex
-		+ (cent != 4 and yindex != 4)
+		qc * 400 +
+		cent * 100 +
+		quad * 4 +
+		yindex +
+		(cent != 4 and yindex != 4)
 	)
 	yearday = jd - to_jd(year, 1, 1)
 
