@@ -7,16 +7,36 @@ import logging
 log = logging.getLogger("pyglossary")
 
 
+def optionFromDict(data):
+	className = data.pop("class")
+	if className == "Option":
+		data["typ"] = data.pop("type")
+		optClass = Option
+	else:
+		data.pop("type")
+		optClass = Option.classes[className]
+	return optClass(**data)
+
+
 class Option(object):
+	classes = {}
+
+	@classmethod
+	def register(cls, optClass):
+		cls.classes[optClass.__name__] = optClass
+		return optClass
+
 	def __init__(
 		self,
 		typ: str,
 		customValue: bool = False,
 		values: "Optional[List[str]]" = None,
+		allowNone: bool = False,
 		comment: str = "",
+		multiline: bool = False,
 		disabled: bool = False,
-		cmd: bool = False,
-		cmdFlag: str = "",
+		hasFlag: bool = False,
+		customFlag: str = "",
 		falseComment: str = "",
 	) -> None:
 		if values is None:
@@ -24,13 +44,47 @@ class Option(object):
 			customValue = True
 		self.typ = typ
 		self.values = values
+		self.allowNone = allowNone
 		self.customValue = customValue
 		self.comment = comment
+		self.multiline = multiline
 		self.disabled = disabled
-		self.cmd = cmd
-		self.cmdFlag = cmdFlag
+		self.hasFlag = hasFlag
+		self.customFlag = customFlag
 		self.falseComment = falseComment
 
+	@property
+	def typeDesc(self):
+		return self.typ
+
+	@property
+	def longComment(self):
+		comment = self.typeDesc
+		if self.comment:
+			if comment:
+				comment += ", "
+			comment += self.comment
+		return comment
+
+	def toDict(self):
+		data = {
+			"class": self.__class__.__name__,
+			"type": self.typ,
+			"customValue": self.customValue,
+		}
+		if self.values:
+			data["values"] = self.values
+		if self.comment:
+			data["comment"] = self.comment
+		if self.disabled:
+			data["disabled"] = True
+		if self.hasFlag:
+			data["hasFlag"] = True
+			data["customFlag"] = self.customFlag
+		if self.falseComment:
+			data["falseComment"] = self.falseComment
+		return data
+
 	def evaluate(self, raw: str) -> "Tuple[Any, bool]":
 		"returns (value, isValid)"
 		if raw == "None":
@@ -47,7 +101,7 @@ class Option(object):
 				return False
 			return value in self.values
 		if value is None:
-			return self.typ in ("dict", "list")
+			return self.allowNone
 		valueType = type(value).__name__
 		return self.typ == valueType
 
@@ -64,17 +118,30 @@ class Option(object):
 		return None
 
 
+@Option.register
 class BoolOption(Option):
-	def __init__(self, **kwargs):
+	def __init__(self, allowNone=False, **kwargs):
+		values = [False, True]
+		if allowNone:
+			values.append(None)
 		Option.__init__(
 			self,
-			"bool",
+			typ="bool",
 			customValue=False,
-			values=[False, True],
+			values=values,
+			allowNone=allowNone,
 			**kwargs,
 		)
 
+	def toDict(self):
+		data = Option.toDict(self)
+		del data["customValue"]
+		del data["values"]
+		return data
+
 	def evaluate(self, raw: "Union[str, bool]") -> "Tuple[Optional[bool], bool]":
+		if raw is None or raw.lower() == "none":
+			return None, True
 		if isinstance(raw, bool):
 			return raw, True
 		if raw.lower() in ("yes", "true", "1"):
@@ -84,9 +151,14 @@ class BoolOption(Option):
 		return None, False  # not valid
 
 
+@Option.register
 class StrOption(Option):
 	def __init__(self, **kwargs):
-		Option.__init__(self, "str", **kwargs)
+		Option.__init__(
+			self,
+			typ="str",
+			**kwargs
+		)
 
 	def validate(self, value):
 		if not self.customValue:
@@ -103,9 +175,14 @@ class StrOption(Option):
 		return None
 
 
+@Option.register
 class IntOption(Option):
 	def __init__(self, **kwargs):
-		Option.__init__(self, "int", **kwargs)
+		Option.__init__(
+			self,
+			typ="int",
+			**kwargs
+		)
 
 	def evaluate(self, raw: "Union[str, int]") -> "Tuple[Optional[int], bool]":
 		"returns (value, isValid)"
@@ -113,13 +190,78 @@ class IntOption(Option):
 			value = int(raw)
 		except ValueError:
 			return None, False
-		else:
-			return value, True
+		return value, True
 
 
+@Option.register
+class FileSizeOption(IntOption):
+	factors = {
+		"KiB": 1024,
+		"kib": 1024,
+		"Ki": 1024,
+		"ki": 1024,
+
+		"MiB": 1048576,
+		"mib": 1048576,
+		"Mi": 1048576,
+		"mi": 1048576,
+
+		"GiB": 1073741824,
+		"gib": 1073741824,
+		"Gi": 1073741824,
+		"gi": 1073741824,
+
+		"kB": 1000,
+		"kb": 1000,
+		"KB": 1000,
+		"k": 1000,
+		"K": 1000,
+
+		"MB": 1000000,
+		"mb": 1000000,
+		"mB": 1000000,
+		"M": 1000000,
+		"m": 1000000,
+
+		"GB": 1000000000,
+		"gb": 1000000000,
+		"gB": 1000000000,
+		"G": 1000000000,
+		"g": 1000000000,
+	}
+	validPattern = "^([0-9.]+)([kKmMgG]i?[bB]?)$"
+
+	@property
+	def typeDesc(self):
+		return ""
+
+	def evaluate(self, raw: "Union[str, int]") -> "Tuple[Optional[int], bool]":
+		if not raw:
+			return 0
+		factor = 1
+		m = re.match(self.validPattern, raw)
+		if m is not None:
+			raw, unit = m.groups()
+			factor = self.factors.get(unit)
+			if factor is None:
+				return None, False
+		try:
+			value = float(raw)
+		except ValueError:
+			return None, False
+		if value < 0:
+			return None, False
+		return int(value * factor), True
+
+
+@Option.register
 class FloatOption(Option):
 	def __init__(self, **kwargs):
-		Option.__init__(self, "float", **kwargs)
+		Option.__init__(
+			self,
+			typ="float",
+			**kwargs
+		)
 
 	def evaluate(
 		self,
@@ -134,15 +276,23 @@ class FloatOption(Option):
 			return value, True
 
 
+@Option.register
 class DictOption(Option):
 	def __init__(self, **kwargs):
 		Option.__init__(
 			self,
-			"dict",
+			typ="dict",
 			customValue=True,
+			allowNone=True,
+			multiline=True,
 			**kwargs,
 		)
 
+	def toDict(self):
+		data = Option.toDict(self)
+		del data["customValue"]
+		return data
+
 	def evaluate(self, raw: "Union[str, dict]") -> "Tuple[Optional[Dict], bool]":
 		import ast
 		if isinstance(raw, dict):
@@ -158,15 +308,23 @@ class DictOption(Option):
 		return value, True  # valid
 
 
+@Option.register
 class ListOption(Option):
 	def __init__(self, **kwargs):
 		Option.__init__(
 			self,
-			"list",
+			typ="list",
 			customValue=True,
+			allowNone=True,
+			multiline=True,
 			**kwargs,
 		)
 
+	def toDict(self):
+		data = Option.toDict(self)
+		del data["customValue"]
+		return data
+
 	def evaluate(self, raw: str) -> "Tuple[Optional[List], bool]":
 		import ast
 		if raw == "":
@@ -180,10 +338,17 @@ class ListOption(Option):
 		return value, True  # valid
 
 
+@Option.register
 class EncodingOption(Option):
 	re_category = re.compile("^[a-z]+")
 
-	def __init__(self, customValue=True, values=None, **kwargs):
+	def __init__(
+		self,
+		customValue=True,
+		values=None,
+		comment=None,
+		**kwargs
+	):
 		if values is None:
 			values = [
 				"utf-8",
@@ -213,14 +378,22 @@ class EncodingOption(Option):
 				"latin5",
 				"latin6",
 			]
+		if comment is None:
+			comment = "Encoding/charset"
 		Option.__init__(
 			self,
-			"str",
+			typ="str",
 			customValue=customValue,
 			values=values,
+			comment=comment,
 			**kwargs
 		)
 
+	def toDict(self):
+		data = Option.toDict(self)
+		del data["values"]
+		return data
+
 	def groupValues(self) -> "Optional[Dict[str, Any]]":
 		from collections import OrderedDict
 		groups = OrderedDict()  # type: Dict[str, List[str]]
@@ -242,24 +415,46 @@ class EncodingOption(Option):
 		return groups
 
 
+@Option.register
 class NewlineOption(Option):
-	def __init__(self, customValue=True, values=None, **kwargs):
+	def __init__(
+		self,
+		customValue=True,
+		values=None,
+		comment=None,
+		**kwargs
+	):
 		if values is None:
 			values = [
 				"\r\n",
 				"\n",
 				"\r",
 			]
+		if comment is None:
+			comment = "Newline string"
 		Option.__init__(
 			self,
-			"str",
+			typ="str",
 			customValue=customValue,
 			values=values,
+			multiline=True,
+			comment=comment,
 			**kwargs
 		)
 
 
+@Option.register
 class HtmlColorOption(Option):
+	def toDict(self):
+		data = Option.toDict(self)
+		del data["customValue"]
+		return data
+
 	def __init__(self, **kwargs):
-		Option.__init__(self, "str", customValue=True, **kwargs)
-		# FIXME: use a specific type?
+		Option.__init__(
+			self,
+			typ="str",
+			customValue=True,
+			**kwargs
+		)
+		# TODO: use a specific type?
