@@ -1,6 +1,7 @@
 import os
 import shutil
 import logging
+from pyglossary import core
 
 log = logging.getLogger("pyglossary")
 
@@ -42,6 +43,7 @@ def runDictzip(filename: str) -> None:
 	import subprocess
 	dictzipCmd = shutil.which("dictzip")
 	if not dictzipCmd:
+		log.warning("dictzip command was not found. Make sure it's in your $PATH")
 		return False
 	(out, err) = subprocess.Popen(
 		[dictzipCmd, filename],
@@ -56,6 +58,36 @@ def runDictzip(filename: str) -> None:
 		log.error(f"dictzip error: {out}")
 
 
+def _rmtreeError(func, direc, exc_info):
+	exc_type, exc_val, exc_tb = exc_info
+	log.error(exc_val)
+
+
+def rmtree(direc):
+	import shutil
+	from os.path import isdir
+	try:
+		for i in range(2):
+			if isdir(direc):
+				shutil.rmtree(
+					direc,
+					onerror=_rmtreeError,
+				)
+	except Exception:
+		log.exception(f"error removing directory: {direc}")
+
+
+def showMemoryUsage():
+	if log.level > core.TRACE:
+		return
+	try:
+		import psutil
+	except ModuleNotFoundError:
+		return
+	usage = psutil.Process(os.getpid()).memory_info().rss // 1024
+	log.trace(f"Memory Usage: {usage} kB")
+
+
 def my_url_show(link: str) -> None:
 	import subprocess
 	for path in (
