@@ -106,6 +106,17 @@ class MDict(object):
 			print("Try Brutal Force on Encrypted Key Blocks")
 			self._key_list = self._read_keys_brutal()
 
+	def __repr__(self):
+		return (
+			f"MDict({self._fname!r}, "
+			f"encoding={self._encoding!r}, "
+			f"passcode={self._passcode})"
+		)
+
+	@property
+	def filename(self):
+		return self._fname
+
 	def __len__(self):
 		return self._num_entries
 
@@ -125,7 +136,7 @@ class MDict(object):
 		"""
 		extract attributes from <Dict attr="value" ... >
 		"""
-		taglist = re.findall(b'(\w+)="(.*?)"', header, re.DOTALL)
+		taglist = re.findall(rb'(\w+)="(.*?)"', header, re.DOTALL)
 		tagdict = {}
 		for key, value in taglist:
 			tagdict[key] = _unescape_entities(value)
@@ -512,8 +523,8 @@ class MDX(MDict):
 
 	def _substitute_stylesheet(self, txt):
 		# substitute stylesheet definition
-		txt_list = re.split('`\d+`', txt)
-		txt_tag = re.findall('`\d+`', txt)
+		txt_list = re.split(r'`\d+`', txt)
+		txt_tag = re.findall(r'`\d+`', txt)
 		txt_styled = txt_list[0]
 		for j, p in enumerate(txt_list[1:]):
 			key = txt_tag[j][1:-1]
