@@ -170,17 +170,6 @@ def sortkey(strength, maxlength=None):
 		return lambda x: c.getSortKey(x)[:maxlength]
 
 
-def sortkey_length(strength, word):
-	c = Collator.createInstance(Locale(''))
-	c.setStrength(strength)
-	c.setAttribute(
-		UCollAttribute.ALTERNATE_HANDLING,
-		UCollAttributeValue.SHIFTED,
-	)
-	coll_key = c.getSortKey(word)
-	return len(coll_key) - 1  # subtract 1 for ending \x00 byte
-
-
 class MultiFileReader(io.BufferedIOBase):
 
 	def __init__(self, *args):
@@ -660,10 +649,6 @@ def open(file_or_filenames):
 	return Slob(file_or_filenames)
 
 
-def create(*args, **kwargs):
-	return Writer(*args, **kwargs)
-
-
 class BinMemWriter:
 
 	def __init__(self):
@@ -843,32 +828,6 @@ class Store(ItemList):
 		return (content_type, content)
 
 
-def find(word, slobs, match_prefix=True):
-	seen = set()
-	if isinstance(slobs, Slob):
-		slobs = [slobs]
-
-	variants = []
-
-	for strength in (QUATERNARY, TERTIARY, SECONDARY, PRIMARY):
-		variants.append((strength, None))
-
-	if match_prefix:
-		for strength in (QUATERNARY, TERTIARY, SECONDARY, PRIMARY):
-			variants.append((strength, sortkey_length(strength, word)))
-
-	for strength, maxlength in variants:
-		for slob in slobs:
-			d = slob.as_dict(strength=strength, maxlength=maxlength)
-			for item in d[word]:
-				dedup_key = (slob.id, item.id, item.fragment)
-				if dedup_key in seen:
-					continue
-				else:
-					seen.add(dedup_key)
-					yield slob, item
-
-
 WriterEvent = namedtuple('WriterEvent', 'name data')
 
 
@@ -1083,10 +1042,10 @@ class Writer(object):
 							self.f_refs.name) as f_ref_list:
 			ref_list = RefList(f_ref_list, self.encoding, count=self.ref_count)
 			ref_dict = ref_list.as_dict()
-			with open(self.aliases_path) as r:
+			with Slob(self.aliases_path) as r:
 				aliases = r.as_dict()
 				path = os.path.join(self.tmpdir.name, 'resolved-aliases')
-				with create(
+				with Writer(
 					path,
 					workdir=self.tmpdir.name,
 					max_redirects=0,
@@ -1133,7 +1092,7 @@ class Writer(object):
 								)
 								alias_writer.add(pickle.dumps(ref), key)
 
-		with open(path) as resolved_aliases_reader:
+		with Slob(path) as resolved_aliases_reader:
 			previous_key = None
 			for item in resolved_aliases_reader:
 				ref = pickle.loads(item.content)
