@@ -2,7 +2,7 @@
 # appledict/__init__.py
 # Output to Apple Dictionary xml sources for Dictionary Development Kit.
 #
-# Copyright © 2016-2020 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
+# Copyright © 2016-2021 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 # Copyright © 2016 Ratijas <ratijas.t@me.com>
 # Copyright © 2012-2015 Xiaoqiang Wang <xiaoqiangwang AT gmail DOT com>
 #
@@ -27,37 +27,55 @@ import pkgutil
 import shutil
 
 from pyglossary.plugins.formats_common import *
-from pyglossary.xdxf_transform import xdxf_to_html_transformer
 from ._dict import *
+from ._content import prepare_content
 
 sys.setrecursionlimit(10000)
 
 enable = True
+lname = "appledict"
 format = "AppleDict"
 description = "AppleDict Source"
 extensions = (".apple",)
+extensionCreate = ".apple/"
+kind = "directory"
+wiki = ""
+website = (
+	"https://support.apple.com/en-gu/guide/dictionary/welcome/mac",
+	"Dictionary User Guide for Mac",
+)
 # FIXME: rename indexes arg/option to indexes_lang?
 optionsProp = {
-	"cleanHTML": BoolOption(comment="use BeautifulSoup parser"),
-	"css": StrOption(comment="custom .css file path"),
-	"xsl": StrOption(comment="custom XSL transformations file path"),
-	"defaultPrefs": DictOption(
-		comment='default prefs in python dict format',
+	"clean_html": BoolOption(comment="use BeautifulSoup parser"),
+	"css": StrOption(
+		comment="custom .css file path",
+	),
+	"xsl": StrOption(
+		comment="custom XSL transformations file path",
+	),
+	"default_prefs": DictOption(
+		comment="default prefs in python dict format",
 		# example: {"key": "value", "version": "1"}
 	),
-	"prefsHTML": StrOption(comment="preferences XHTML file path"),
-	"frontBackMatter": StrOption(comment="XML file path with top-level tag"),
+	"prefs_html": StrOption(
+		comment="preferences XHTML file path",
+	),
+	"front_back_matter": StrOption(
+		comment="XML file path with top-level tag",
+	),
 	"jing": BoolOption(comment="run Jing check on generated XML"),
-	"indexes": StrOption(customValue=False, values=["", "ru", "zh"]),
+	"indexes": StrOption(
+		customValue=False,
+		values=["", "ru", "zh"],
+		comment="Additional indexes to dictionary entries",
+	),
 }
 
-tools = [
-	{
-		"name": "Dictionary Development Kit",
-		"web": "https://github.com/SebastianSzturo/Dictionary-Development-Kit",
-		"platforms": ["Mac"],
-		"license": "Unknown",
-	},
+extraDocs = [
+	(
+		"Also see:",
+		"See [doc/apple.md](./doc/apple.md) for additional AppleDict instructions.",
+	),
 ]
 
 
@@ -88,7 +106,7 @@ def abspath_or_None(path):
 def write_header(
 	glos: "GlossaryType",
 	toFile: "TextIO",
-	frontBackMatter: "Optional[str]",
+	front_back_matter: "Optional[str]",
 ) -> None:
 	# write header
 	toFile.write(
@@ -97,38 +115,42 @@ def write_header(
 		'xmlns:d="http://www.apple.com/DTDs/DictionaryService-1.0.rng">\n'
 	)
 
-	if frontBackMatter:
-		with open(frontBackMatter, "r") as front_back_matter:
+	if front_back_matter:
+		with open(
+			front_back_matter,
+			mode="r",
+			encoding="utf-8",
+		) as front_back_matter:
 			toFile.write(front_back_matter.read())
 
 
-def format_default_prefs(defaultPrefs):
+def format_default_prefs(default_prefs):
 	"""
-	:type defaultPrefs: dict or None
+	:type default_prefs: dict or None
 
 	as by 14th of Jan 2016, it is highly recommended that prefs should contain
 	{"version": "1"}, otherwise Dictionary.app does not keep user changes
 	between restarts.
 	"""
-	if not defaultPrefs:
+	if not default_prefs:
 		return ""
-	if not isinstance(defaultPrefs, dict):
-		raise TypeError(f"defaultPrefs not a dictionary: {defaultPrefs!r}")
-	if str(defaultPrefs.get("version", None)) != "1":
+	if not isinstance(default_prefs, dict):
+		raise TypeError(f"default_prefs not a dictionary: {default_prefs!r}")
+	if str(default_prefs.get("version", None)) != "1":
 		log.error(
 			"default prefs does not contain {'version': '1'}.  prefs "
 			"will not be persistent between Dictionary.app restarts."
 		)
 	return "\n".join(
 		f"\t\t<key>{key}</key>\n\t\t<string>{value}</string>"
-		for key, value in sorted(defaultPrefs.items())
+		for key, value in sorted(default_prefs.items())
 	).strip()
 
 
 def write_css(fname, css_file):
-	with open(fname, "wb") as toFile:
+	with open(fname, mode="wb") as toFile:
 		if css_file:
-			with open(css_file, "rb") as fromFile:
+			with open(css_file, mode="rb") as fromFile:
 				toFile.write(fromFile.read())
 		else:
 			toFile.write(pkgutil.get_data(
@@ -142,22 +164,22 @@ write glossary to Apple dictionary .xml and supporting files.
 
 :param dirname: directory path, must not have extension
 
-:param cleanHTML: pass True to use BeautifulSoup parser.
+:param clean_html: pass True to use BeautifulSoup parser.
 
 :param css: path to custom .css file
 
 :param xsl: path to custom XSL transformations file.
 
-:param defaultPrefs: Default prefs in python dictionary literal format,
-i.e. {"key1": "value1", "key2": "value2", ...}.  All keys and values 
+:param default_prefs: Default prefs in python dictionary literal format,
+i.e. {"key1": "value1", "key2": "value2", ...}.  All keys and values
 must be quoted strings; not allowed characters (e.g. single/double
 quotes,equal sign "=", semicolon) must be escaped as hex code
 according to python string literal rules.
 
-:param prefsHTML: path to XHTML file with user interface for
+:param prefs_html: path to XHTML file with user interface for
 dictionary's preferences. refer to Apple's documentation for details.
 
-:param frontBackMatter: path to XML file with top-level tag
+:param front_back_matter: path to XML file with top-level tag
 <d:entry id="front_back_matter" d:title="Your Front/Back Matter Title">
 	your front/back matter entry content
 </d:entry>
@@ -170,6 +192,7 @@ how to perform flexible search.  we can help it by manually providing
 additional indexes to dictionary entries.
 """
 
+
 class Writer(object):
 	depends = {
 		"lxml": "lxml",
@@ -177,12 +200,12 @@ class Writer(object):
 		"html5lib": "html5lib",
 	}
 
-	_cleanHTML: bool = True
+	_clean_html: bool = True
 	_css: str = ""
 	_xsl: str = ""
-	_defaultPrefs: "Optional[Dict]" = None
-	_prefsHTML: str = ""
-	_frontBackMatter: str = ""
+	_default_prefs: "Optional[Dict]" = None
+	_prefs_html: str = ""
+	_front_back_matter: str = ""
 	_jing: bool = False
 	_indexes: str = ""  # FIXME: rename to indexes_lang?
 
@@ -200,25 +223,26 @@ class Writer(object):
 
 	def write(self) -> "Generator[None, BaseEntry, None]":
 		global BeautifulSoup
+		from pyglossary.xdxf_transform import XdxfTransformer
 
 		glos = self._glos
-		cleanHTML = self._cleanHTML
+		clean_html = self._clean_html
 		css = self._css
 		xsl = self._xsl
-		defaultPrefs = self._defaultPrefs
-		prefsHTML = self._prefsHTML
-		frontBackMatter = self._frontBackMatter
+		default_prefs = self._default_prefs
+		prefs_html = self._prefs_html
+		front_back_matter = self._front_back_matter
 		jing = self._jing
 		indexes = self._indexes
 
-		xdxf_to_html = xdxf_to_html_transformer()
+		xdxf_to_html = XdxfTransformer(encoding="utf-8")
 
-		if cleanHTML:
+		if clean_html:
 			if BeautifulSoup is None:
 				loadBeautifulSoup()
 			if BeautifulSoup is None:
 				log.warning(
-					"cleanHTML option passed but BeautifulSoup not found. "
+					"clean_html option passed but BeautifulSoup not found. "
 					f"to fix this run "
 					f"`{pip} install lxml beautifulsoup4 html5lib`"
 				)
@@ -231,8 +255,8 @@ class Writer(object):
 		# before chdir (outside indir block)
 		css = abspath_or_None(css)
 		xsl = abspath_or_None(xsl)
-		prefsHTML = abspath_or_None(prefsHTML)
-		frontBackMatter = abspath_or_None(frontBackMatter)
+		prefs_html = abspath_or_None(prefs_html)
+		front_back_matter = abspath_or_None(front_back_matter)
 
 		generate_id = id_generator()
 		generate_indexes = indexes_generator(indexes)
@@ -241,8 +265,8 @@ class Writer(object):
 		if not isdir(myResDir):
 			os.mkdir(myResDir)
 
-		with open(filePathBase + ".xml", "w", encoding="utf-8") as toFile:
-			write_header(glos, toFile, frontBackMatter)
+		with open(filePathBase + ".xml", mode="w", encoding="utf-8") as toFile:
+			write_header(glos, toFile, front_back_matter)
 			while True:
 				entry = yield
 				if entry is None:
@@ -270,7 +294,7 @@ class Writer(object):
 
 				content_title = long_title
 				if entry.defiFormat == "x":
-					defi = xdxf_to_html(defi)
+					defi = xdxf_to_html.transformByInnerString(defi)
 					content_title = None
 				content = prepare_content(content_title, defi, BeautifulSoup)
 
@@ -286,12 +310,12 @@ class Writer(object):
 		if xsl:
 			shutil.copy(xsl, myResDir)
 
-		if prefsHTML:
-			shutil.copy(prefsHTML, myResDir)
+		if prefs_html:
+			shutil.copy(prefs_html, myResDir)
 
 		write_css(filePathBase + ".css", css)
 
-		with open(join(dirname, "Makefile"), "w") as toFile:
+		with open(join(dirname, "Makefile"), mode="w", encoding="utf-8") as toFile:
 			toFile.write(
 				toStr(pkgutil.get_data(
 					__name__,
@@ -309,11 +333,11 @@ class Writer(object):
 
 		# if DCSDictionaryXSL provided but DCSDictionaryDefaultPrefs <dict/> not
 		# present in Info.plist, Dictionary.app will crash.
-		with open(filePathBase + ".plist", "w", encoding="utf-8") as toFile:
+		with open(filePathBase + ".plist", mode="w", encoding="utf-8") as toFile:
 			frontMatterReferenceID = (
 				"<key>DCSDictionaryFrontMatterReferenceID</key>\n"
 				"\t<string>front_back_matter</string>"
-				if frontBackMatter
+				if front_back_matter
 				else ""
 			)
 			toFile.write(
@@ -326,10 +350,10 @@ class Writer(object):
 					CFBundleDisplayName=glos.getInfo("name"),
 					CFBundleName=fileNameBase,
 					DCSDictionaryCopyright=copyright,
-					DCSDictionaryManufacturerName=glos.getAuthor(),
+					DCSDictionaryManufacturerName=glos.author,
 					DCSDictionaryXSL=basename(xsl) if xsl else "",
-					DCSDictionaryDefaultPrefs=format_default_prefs(defaultPrefs),
-					DCSDictionaryPrefsHTML=basename(prefsHTML) if prefsHTML else "",
+					DCSDictionaryDefaultPrefs=format_default_prefs(default_prefs),
+					DCSDictionaryPrefsHTML=basename(prefs_html) if prefs_html else "",
 					DCSDictionaryFrontMatterReferenceID=frontMatterReferenceID,
 				)
 			)
