@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # appledict/indexes/__init__.py
 #
-# Copyright (C) 2016 Ratijas <ratijas.t@me.com>
+# Copyright © 2016 Ratijas <ratijas.t@me.com>
 #
 # This program is a free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -48,9 +48,4 @@ use
 here = os.path.dirname(os.path.abspath(__file__))
 
 for _, module, _ in pkgutil.iter_modules([here]):
-	try:
-		__import__("%s.%s" % (__name__, module))
-	except ImportError:
-		log.exception(
-			"error while importing indexes plugin %s" % module
-		)
+	__import__(f"{__name__}.{module}")
