@@ -5,6 +5,6 @@ be checked.  otherwise, you need to import this module and call
 `run` function with the filename as its only argument.
 """
 
-__all__ = ['run', 'JingTestError']
+__all__ = ["run", "JingTestError"]
 
 from .main import run, JingTestError
