@@ -14,26 +14,29 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 
 from struct import unpack
 from zlib import decompress
 
 enable = True
+lname = "appledict_bin"
 format = "AppleDictBin"
 description = "AppleDict Binary"
 extensions = (".dictionary", ".data",)
+extensionCreate = ""
 singleFile = True
-tools = [
-	{
-		"name": "Apple Dictionary",
-		"web": "https://support.apple.com/en-gu/guide/dictionary/welcome/mac",
-		"platforms": ["Mac"],
-		"license": "Proprietary",
-	},
-]
+kind = "binary"
+wiki = ""
+website = (
+	"https://support.apple.com/en-gu/guide/dictionary/welcome/mac",
+	"Dictionary User Guide for Mac",
+)
 optionsProp = {
-	"html": BoolOption(),
+	"html": BoolOption(comment="Entries are HTML"),
+	"html_full": BoolOption(
+		comment="Turn every entry's definition into an HTML document",
+	),
 }
 
 
@@ -42,7 +45,8 @@ class Reader(object):
 		"lxml": "lxml",
 	}
 
-	_html: bool = False
+	_html: bool = True
+	_html_full: bool = False
 
 	def __init__(self, glos):
 		self._glos = glos
@@ -52,6 +56,12 @@ class Reader(object):
 		self._buf = ""
 		self._defiFormat = "m"
 
+		try:
+			from lxml import etree
+		except ModuleNotFoundError as e:
+			e.msg += f", run `{pip} install lxml` to install"
+			raise e
+
 	def open(self, filename):
 		self._defiFormat = "h" if self._html else "m"
 		parts = filename.split(os.sep)
@@ -61,8 +71,15 @@ class Reader(object):
 				filename = join(filename, "Body.data")
 				if len(parts) > 2:
 					dbname = parts[-2]
-			else:
+			elif isfile(join(filename, "Contents/Body.data")):
+				filename = join(filename, "Contents/Body.data")
+			elif isfile(join(filename, "Contents/Resources/Body.data")):
 				filename = join(filename, "Contents/Resources/Body.data")
+			else:
+				raise IOError(
+					"could not find Body.data file, "
+					"please select Body.data file instead of directory"
+				)
 		elif dbname == "Body.data" and len(parts) > 1:
 			dbname = parts[-2]
 			if len(parts) > 2:
@@ -71,6 +88,9 @@ class Reader(object):
 				elif dbname == "Resources" and len(parts) > 3:
 					dbname = parts[-4]
 
+		if not isfile(filename):
+			raise IOError(f"no such file: {filename}")
+
 		if dbname.endswith(".dictionary"):
 			dbname = dbname[:-len(".dictionary")]
 		self._glos.setInfo("name", dbname)
@@ -108,15 +128,41 @@ class Reader(object):
 			raise e
 		return chunkSize, plus
 
+	def _getDefi(self, entryElem: "Element") -> str:
+		from lxml import etree
+
+		if not self._html:
+			# FIXME: this produces duplicate text for Idioms.dictionary, see #301
+			return "".join([
+				self.decode(etree.tostring(
+					child,
+					encoding="utf-8",
+				))
+				for child in entryElem.iterdescendants()
+			])
+
+
+		defi = self.decode(etree.tostring(
+			entryElem,
+			encoding="utf-8",
+		))
+
+		if self._html_full:
+			defi = (
+				f'<!DOCTYPE html><html><head>'
+				f'<link rel="stylesheet" href="DefaultStyle.css">'
+				f'</head><body>{defi}</body></html>'
+			)
+
+		return defi
+
+
 	def _readEntry(self, pos: int) -> "Tuple[BaseEntry, int]":
 		"""
 			returns (entry, pos)
 		"""
-		try:
-			from lxml import etree
-		except ModuleNotFoundError as e:
-			e.msg += f", run `{pip} install lxml` to install"
-			raise e
+		from lxml import etree
+
 		chunkSize, plus = self.getChunkSize(pos)
 		pos += plus
 		if chunkSize == 0:
@@ -143,13 +189,9 @@ class Reader(object):
 		if not entryElems:
 			return None, pos
 		word = entryElems[0].xpath("./@d:title", namespaces=entryRoot.nsmap)[0]
-		if self._html:
-			defi = self.decode(etree.tostring(entryElems[0]))
-		else:
-			defi = "".join([
-				self.decode(etree.tostring(child))
-				for child in entryElems[0].iterdescendants()
-			])
+
+		defi = self._getDefi(entryElems[0])
+
 		pos += chunkSize
 		if self._limit <= 0:
 			raise ValueError(f"self._limit = {self._limit}")
@@ -160,13 +202,25 @@ class Reader(object):
 		), pos
 
 	def __iter__(self):
-		file = self._file
+		from os.path import dirname
+
+		if self._file is None:
+			raise RuntimeError("iterating over a reader while it's not open")
+		glos = self._glos
+
+		cssFilename = join(dirname(self._filename), "DefaultStyle.css")
+		if isfile(cssFilename):
+			with open(cssFilename, mode="rb") as cssFile:
+				cssBytes = cssFile.read()
+			yield glos.newDataEntry("style.css", cssBytes)
+
+		_file = self._file
 		limit = self._limit
 		while True:
-			self._absPos = file.tell()
+			self._absPos = _file.tell()
 			if self._absPos >= limit:
 				break
-			bufSizeB = file.read(4)  # type: bytes
+			bufSizeB = _file.read(4)  # type: bytes
 			# alternative for buf, bufSize is calculated
 			# ~ flag = f.tell()
 			# ~ bufSize = 0
@@ -184,7 +238,7 @@ class Reader(object):
 			# ~			bufSize = bufSize+1
 
 			bufSize, = unpack("i", bufSizeB)  # type: int
-			self._buf = decompress(file.read(bufSize)[8:])
+			self._buf = decompress(_file.read(bufSize)[8:])
 
 			pos = 0
 			while pos < len(self._buf):
