@@ -1,9 +1,14 @@
 # -*- coding: utf-8 -*-
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 
 enable = False
+lname = "babylon_bdc"
 format = "BabylonBdc"
 description = "Babylon (bdc)"
 extensions = (".bdc",)
+extensionCreate = ""
 singleFile = True
+kind = "binary"
+wiki = ""
+website = None
 optionsProp = {}
