@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2008-2020 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
+# Copyright © 2008-2021 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 #
 # This program is a free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,35 +16,20 @@
 # with this program. Or on Debian systems, from /usr/share/common-licenses/GPL
 # If not, see <http://www.gnu.org/licenses/gpl.txt>.
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from .bgl_reader import BglReader as Reader
 from .bgl_reader import optionsProp
 
 enable = True
+lname = "babylon_bgl"
 format = "BabylonBgl"
-description = "Babylon (bgl)"
+description = "Babylon (.BGL)"
 extensions = (".bgl",)
+extensionCreate = ""
 singleFile = True
-tools = [
-	{
-		"name": "Babylon Translator",
-		"web": "https://www.babylon-software.com/",
-		"platforms": ["Windows"],
-		"license": "Proprietary",
-	},
-	{
-		"name": "GoldenDict",
-		"web": "http://goldendict.org/",
-		"platforms": ["Linux", "Windows"],
-		"license": "GPL",
-	},
-	{
-		"name": "GoldenDict Mobile",
-		"web": "http://goldendict.mobi/",
-		"platforms": ["Android"],
-		"license": "Unknown",
-	},
-]
+kind = "binary"
+wiki = ""
+website = None
 # progressbar = DEFAULT_YES
 
 # FIXME: document type of read/write options
