@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2008-2020 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
+# Copyright © 2008-2021 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 # Copyright © 2011-2012 kubtek <kubtek@gmail.com>
 # This file is part of PyGlossary project, http://github.com/ilius/pyglossary
 # Thanks to Raul Fernandes <rgfbr@yahoo.com.br> and Karl Grill for reverse
@@ -32,13 +32,9 @@ try:
 		f"pyglossary.plugin_lib.py{vmajor}{vminor}.gzip_no_crc",
 		fromlist="GzipFile",
 	).GzipFile
-except ImportError:
+except ImportError as e:
 	from gzip import GzipFile
-	log.exception("")
-	log.warning(
-		"If things didn\'t work well with BGL files, "
-		"install Python 3.5 to 3.7 and try again"
-	)
+	log.warning(str(e))
 
 from pyglossary.text_utils import (
 	uintFromBytes,
@@ -68,34 +64,69 @@ from .bgl_text import (
 file = io.BufferedReader
 
 debugReadOptions = {
-	"searchCharSamples",  # bool
-	"collectMetadata2",  # bool
-	"writeGz",  # bool
-	"charSamplesPath",  # str, file path
-	"msgLogPath",  # str, file path
-	"rawDumpPath",  # str, file path
-	"unpackedGzipPath",  # str, file path
+	"search_char_samples",  # bool
+	"collect_metadata2",  # bool
+	"write_gz",  # bool
+	"char_samples_path",  # str, file path
+	"msg_log_path",  # str, file path
+	"raw_dump_path",  # str, file path
+	"unpacked_gzip_path",  # str, file path
 }
 
 optionsProp = {
-	"defaultEncodingOverwrite": EncodingOption(),
-	"sourceEncodingOverwrite": EncodingOption(),
-	"targetEncodingOverwrite": EncodingOption(),
-	"partOfSpeechColor": HtmlColorOption(),
+	"default_encoding_overwrite": EncodingOption(
+		comment="Default encoding (overwrite)",
+	),
+	"source_encoding_overwrite": EncodingOption(
+		comment="Source encoding (overwrite)",
+	),
+	"target_encoding_overwrite": EncodingOption(
+		comment="Target encoding (overwrite)",
+	),
+	"part_of_speech_color": HtmlColorOption(
+		comment="Color for Part of Speech",
+	),
 
-	"noControlSequenceInDefi": BoolOption(),
-	"strictStringConvertion": BoolOption(),
-	"processHtmlInKey": BoolOption(),
-	"keyRStripChars": StrOption(),
+	"no_control_sequence_in_defi": BoolOption(
+		comment="No control sequence in definitions",
+	),
+	"strict_string_convertion": BoolOption(
+		comment="Strict string convertion",
+	),
+	"process_html_in_key": BoolOption(
+		comment="Process HTML in (entry or info) key",
+	),
+	"key_rstrip_chars": StrOption(
+		multiline=True,
+		comment="Characters to strip from right-side of keys",
+	),
 
 	# debug read options:
-	"searchCharSamples": BoolOption(),
-	"collectMetadata2": BoolOption(),
-	"writeGz": BoolOption(),
-	"charSamplesPath": StrOption(), # file path
-	"msgLogPath": StrOption(), # file path
-	"rawDumpPath": StrOption(), # file path
-	"unpackedGzipPath": StrOption(), # file path
+	"search_char_samples": BoolOption(
+		comment="(debug) Search character samples",
+	),
+	"collect_metadata2": BoolOption(
+		comment="(debug) Collect second pass metadata from definitions",
+	),
+	"write_gz": BoolOption(
+		comment="(debug) Create a file named *-data.gz",
+	),
+	"char_samples_path": StrOption(
+		# file path
+		comment="(debug) File path for character samples",
+	),
+	"msg_log_path": StrOption(
+		# file path
+		comment="(debug) File path for message log",
+	),
+	"raw_dump_path": StrOption(
+		# file path
+		comment="(debug) File path for writing raw blocks",
+	),
+	"unpacked_gzip_path": StrOption(
+		# file path
+		comment="(debug) Path to create unzipped file",
+	),
 }
 
 
@@ -244,12 +275,12 @@ class DefinitionFields(object):
 
 
 class BglReader(object):
-	_defaultEncodingOverwrite = ""
-	_sourceEncodingOverwrite = ""
-	_targetEncodingOverwrite = ""
-	_partOfSpeechColor = "007000"
-	_noControlSequenceInDefi = False
-	_strictStringConvertion = False
+	_default_encoding_overwrite = ""
+	_source_encoding_overwrite = ""
+	_target_encoding_overwrite = ""
+	_part_of_speech_color = "007000"
+	_no_control_sequence_in_defi = False
+	_strict_string_convertion = False
 	# process keys and alternates as HTML
 	# Babylon does not interpret keys and alternates as HTML text,
 	# however you may encounter many keys containing character references
@@ -259,10 +290,10 @@ class BglReader(object):
 	# explicitly by user. Namely this option does the following:
 	# - resolve character references
 	# - strip HTML tags
-	_processHtmlInKey = False
+	_process_html_in_key = False
 	# a string of characters that will be stripped from the end of the
 	# key (and alternate), see str.rstrip function
-	_keyRStripChars = ""
+	_key_rstrip_chars = ""
 
 	##########################################################################
 	"""
@@ -467,7 +498,8 @@ class BglReader(object):
 		###
 		for key, value in self.info.items():
 			if value == "":
-				continue # TODO: a bool flag to add empty value infos?
+				continue
+				# TODO: a bool flag to add empty value infos?
 			# leave "creationTime" and "lastUpdated" as is
 			if key in {
 				"utf8Encoding",
@@ -624,8 +656,8 @@ class BglReader(object):
 		pos += Len
 		b_data = block.data[pos:]
 		# if b_name in (b"C2EEF3F6.html", b"8EAF66FD.bmp"):
-		#	log.debug(f"Skipping non-useful file {b_name!r}")
-		#	return
+		# 	log.debug(f"Skipping non-useful file {b_name!r}")
+		# 	return
 		u_name = b_name.decode(self.sourceEncoding)
 		return self._glos.newDataEntry(
 			u_name,
@@ -685,15 +717,15 @@ class BglReader(object):
 		"""
 		utf8Encoding = self.info.get("utf8Encoding", False)
 
-		if self._defaultEncodingOverwrite:
-			self.defaultEncoding = self._defaultEncodingOverwrite
+		if self._default_encoding_overwrite:
+			self.defaultEncoding = self._default_encoding_overwrite
 		elif self.defaultCharset:
 			self.defaultEncoding = self.defaultCharset
 		else:
 			self.defaultEncoding = "cp1252"
 
-		if self._sourceEncodingOverwrite:
-			self.sourceEncoding = self._sourceEncodingOverwrite
+		if self._source_encoding_overwrite:
+			self.sourceEncoding = self._source_encoding_overwrite
 		elif utf8Encoding:
 			self.sourceEncoding = "utf-8"
 		elif self.sourceCharset:
@@ -703,8 +735,8 @@ class BglReader(object):
 		else:
 			self.sourceEncoding = self.defaultEncoding
 
-		if self._targetEncodingOverwrite:
-			self.targetEncoding = self._targetEncodingOverwrite
+		if self._target_encoding_overwrite:
+			self.targetEncoding = self._target_encoding_overwrite
 		elif utf8Encoding:
 			self.targetEncoding = "utf-8"
 		elif self.targetCharset:
@@ -723,7 +755,7 @@ class BglReader(object):
 
 	def __iter__(self):
 		if not self.file:
-			raise StopIteration
+			raise RuntimeError("iterating over a reader while it's not open")
 
 		for fname, iconData in self.iconDataList:
 			yield self._glos.newDataEntry(fname, iconData)
@@ -1024,7 +1056,7 @@ class BglReader(object):
 					self.charReferencesStat(b_text2, encoding)
 					if encoding == "cp1252":
 						b_text2 = replaceAsciiCharRefs(b_text2, encoding)
-					if self._strictStringConvertion:
+					if self._strict_string_convertion:
 						try:
 							u_text2 = b_text2.decode(encoding)
 						except UnicodeError:
@@ -1095,7 +1127,7 @@ class BglReader(object):
 				f"number of dollar indexes = {strip_count}",
 			)
 		# convert to unicode
-		if self._strictStringConvertion:
+		if self._strict_string_convertion:
 			try:
 				u_word_main = b_word_main.decode(self.sourceEncoding)
 			except UnicodeError:
@@ -1109,18 +1141,18 @@ class BglReader(object):
 		else:
 			u_word_main = b_word_main.decode(self.sourceEncoding, "ignore")
 
-		if self._processHtmlInKey:
+		if self._process_html_in_key:
 			# u_word_main_orig = u_word_main
 			u_word_main = stripHtmlTags(u_word_main)
 			u_word_main = replaceHtmlEntriesInKeys(u_word_main)
-#			if(re.match(".*[&<>].*", u_word_main_orig)):
-#				log.debug("original text: " + u_word_main_orig + "\n" \
-#						  + "new      text: " + u_word_main + "\n")
+			# if(re.match(".*[&<>].*", u_word_main_orig)):
+			# 	log.debug("original text: " + u_word_main_orig + "\n" \
+			# 			  + "new      text: " + u_word_main + "\n")
 		u_word_main = removeControlChars(u_word_main)
 		u_word_main = removeNewlines(u_word_main)
 		u_word_main = u_word_main.lstrip()
-		if self._keyRStripChars:
-			u_word_main = u_word_main.rstrip(self._keyRStripChars)
+		if self._key_rstrip_chars:
+			u_word_main = u_word_main.rstrip(self._key_rstrip_chars)
 		return u_word_main
 
 	def processAlternativeKey(self, b_word, b_key):
@@ -1130,7 +1162,7 @@ class BglReader(object):
 		"""
 		b_word_main, strip_count = stripDollarIndexes(b_word)
 		# convert to unicode
-		if self._strictStringConvertion:
+		if self._strict_string_convertion:
 			try:
 				u_word_main = b_word_main.decode(self.sourceEncoding)
 			except UnicodeError:
@@ -1148,17 +1180,17 @@ class BglReader(object):
 			u_word_main,
 		)
 
-		if self._processHtmlInKey:
+		if self._process_html_in_key:
 			# u_word_main_orig = u_word_main
 			u_word_main = stripHtmlTags(u_word_main)
 			u_word_main = replaceHtmlEntriesInKeys(u_word_main)
-#			if(re.match(".*[&<>].*", u_word_main_orig)):
-#				log.debug("original text: " + u_word_main_orig + "\n" \
-#						+ "new      text: " + u_word_main + "\n")
+			# if(re.match(".*[&<>].*", u_word_main_orig)):
+			# 	log.debug("original text: " + u_word_main_orig + "\n" \
+			# 			+ "new      text: " + u_word_main + "\n")
 		u_word_main = removeControlChars(u_word_main)
 		u_word_main = removeNewlines(u_word_main)
 		u_word_main = u_word_main.lstrip()
-		u_word_main = u_word_main.rstrip(self._keyRStripChars)
+		u_word_main = u_word_main.rstrip(self._key_rstrip_chars)
 		return u_word_main
 
 	def processDefi(self, b_defi, b_key):
@@ -1262,7 +1294,7 @@ class BglReader(object):
 		if fields.partOfSpeech or fields.u_title:
 			if fields.partOfSpeech:
 				pos = xml_escape(fields.partOfSpeech)
-				posColor = self._partOfSpeechColor
+				posColor = self._part_of_speech_color
 				u_defi_format += f'<font color="#{posColor}">{pos}</font>'
 			if fields.u_title:
 				if u_defi_format:
@@ -1298,7 +1330,7 @@ class BglReader(object):
 		by space, we assume this is part of the article and continue search.
 		Unfortunately this does no help in many cases...
 		"""
-		if self._noControlSequenceInDefi:
+		if self._no_control_sequence_in_defi:
 			return -1
 		index = -1
 		while True:
@@ -1431,8 +1463,8 @@ class BglReader(object):
 				i += 1
 				if Len == 0:
 					# log.debug(
-					#	f"collecting definition fields, b_defi = {b_defi!r}\n"
-					#	f"b_key = {b_key!r}:\nblank entry title"
+					# 	f"collecting definition fields, b_defi = {b_defi!r}\n"
+					# 	f"b_key = {b_key!r}:\nblank entry title"
 					# )
 					continue
 				if i + Len > len(b_defi):
