@@ -1,28 +1,37 @@
 import re
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from . import conv
 
 enable = True
+lname = "cc_cedict"
 format = "CC-CEDICT"
 description = "CC-CEDICT"
 extensions = (".u8",)
+extensionCreate = ""
 singleFile = True
+kind = "text"
+wiki = "https://en.wikipedia.org/wiki/CEDICT"
+website = (
+	"https://cc-cedict.org/editor/editor.php",
+	"CC-CEDICT Editor",
+)
 optionsProp = {
 	"encoding": EncodingOption(),
+	"traditional_title": BoolOption(
+		comment="Use traditional Chinese for entry titles/keys",
+	),
 }
 
-# https://en.wikipedia.org/wiki/CEDICT
-# https://cc-cedict.org/editor/editor.php
-
 entry_count_reg = re.compile(r"#! entries=(\d+)")
 
 
 class Reader:
 	depends = {
-		"jinja2": "jinja2",
+		"lxml": "lxml",
 	}
 
 	_encoding: str = "utf-8"
+	_traditional_title: bool = False
 
 	def __init__(self, glos):
 		self._glos = glos
@@ -76,6 +85,9 @@ class Reader:
 			if parts is None:
 				log.warning("bad line: %s", line)
 				continue
-			names, article = conv.make_entry(*parts)
+			names, article = conv.make_entry(
+				*parts,
+				traditional_title=self._traditional_title,
+			)
 			entry = self._glos.newEntry(names, article, defiFormat="h")
 			yield entry
