@@ -1,19 +1,25 @@
 # -*- coding: utf-8 -*-
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from pyglossary.text_reader import TextGlossaryReader
 from io import BytesIO
 from os.path import dirname
 
 enable = True
+lname = "cc_kedict"
 format = "cc-kedict"
 description = "cc-kedict"
 extensions = ()
+extensionCreate = ""
 singleFile = True
+kind = "text"
+wiki = ""
+website = (
+	"https://github.com/mhagiwara/cc-kedict",
+	"@mhagiwara/cc-kedict",
+)
 optionsProp = {
 }
 
-# https://github.com/mhagiwara/cc-kedict
-
 
 class YamlReader(TextGlossaryReader):
 	tagStyle = (
@@ -230,7 +236,7 @@ class YamlReader(TextGlossaryReader):
 
 		f = BytesIO()
 
-		with ET.htmlfile(f) as hf:
+		with ET.htmlfile(f, encoding="utf-8") as hf:
 			with hf.element("div"):
 				self._processEntry(hf, edict)
 
