@@ -1,4 +1,4 @@
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from hashlib import sha1
 from os.path import dirname
 from os import makedirs, listdir
@@ -11,10 +11,15 @@ from pyglossary.compression import (
 )
 
 enable = True
+lname = "crawler_dir"
 format = "CrawlerDir"
 description = "Crawler Directory"
 extensions = (".crawler",)
+extensionCreate = ".crawler/"
 singleFile = True
+kind = "directory"
+wiki = ""
+website = None
 optionsProp = {
 	"compression": StrOption(
 		values=["", "gz", "bz2", "lzma"],
@@ -77,7 +82,7 @@ class Writer(object):
 			if not isdir(parentDir):
 				makedirs(parentDir)
 			if isfile(fpath):
-				log.warn(f"file exists: {fpath}")
+				log.warning(f"file exists: {fpath}")
 				fpath += f"-{sha1(entry.b_defi).hexdigest()[:4]}"
 			with c_open(fpath, "wt", encoding="utf-8") as _file:
 				_file.write(
