@@ -1,27 +1,24 @@
 # -*- coding: utf-8 -*-
 import re
 
-from tabfile import Reader as TabfileReader
-from formats_common import *
+from pyglossary.plugins.tabfile import Reader as TabfileReader
+from pyglossary.plugins.formats_common import *
 
+lname = "dicformids"
 enable = True
 format = "Dicformids"
 description = "DictionaryForMIDs"
 extensions = (".mids",)
+extensionCreate = ".mids/"
 sortOnWrite = ALWAYS
-
-tools = [
-	{
-		"name": "DictionaryForMIDs",
-		"web": "http://dictionarymid.sourceforge.net/",
-		# https://sourceforge.net/projects/dictionarymid/
-		"platforms": ["Android", "Web", "Windows", "Linux", "Mac"],
-		# PC version is Java-based
-		"license": "GPL",
-		# android last commit:	2015/02/09
-		# android last release:	2015/02/09 - version 1.0.1
-	},
-]
+sortKeyName = "dicformids"
+sortEncoding = "utf-8"
+kind = "directory"
+wiki = ""
+website = (
+	"http://dictionarymid.sourceforge.net/",
+	"DictionaryForMIDs - SourceForge",
+)
 
 optionsProp = {}
 
@@ -146,12 +143,6 @@ class Writer(object):
 		word = word.lower()
 		return word
 
-	def sortKey(self, b_word: bytes) -> "Any":
-		# DO NOT change method name
-		# FIXME: confirm
-		word = b_word.decode("utf-8")
-		return self.normateWord(word)
-
 	def writeProbs(self):
 		glos = self._glos
 		with open(join(
@@ -160,7 +151,7 @@ class Writer(object):
 		), "w") as fileObj:
 			fileObj.write(PROP_TEMPLATE.format(
 				name=glos.getInfo("name"),
-				author=glos.getAuthor(),
+				author=glos.author,
 				indexFileMaxSize=self.indexFileMaxSize,
 				wordCount=self.wordCount,
 				directoryPostfix=self.directoryPostfix,
@@ -262,6 +253,3 @@ class Writer(object):
 		langSearchListFp.close()
 
 		self.writeProbs()
-
-	# def close(self):
-	#	pass
