@@ -1,23 +1,21 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 import html
+from operator import itemgetter
 
 enable = True
+lname = "dict_cc"
 format = 'Dictcc'
 description = 'Dict.cc (SQLite3)'
 extensions = ()
-readOptions = []
-writeOptions = []
-
-tools = [
-	{
-		"name": "dict.cc dictionary",
-		"web": "https://play.google.com/store/apps/details?id=cc.dict.dictcc",
-		"platforms": ["Android"],
-		"license": "Proprietary",
-	},
-]
+extensionCreate = ".db"
+kind = "binary"
+wiki = "https://en.wikipedia.org/wiki/Dict.cc"
+website = (
+	"https://play.google.com/store/apps/details?id=cc.dict.dictcc",
+	"dict.cc dictionary - Google Play",
+)
 
 
 class Reader(object):
@@ -36,7 +34,6 @@ class Reader(object):
 		self._con = connect(filename)
 		self._cur = self._con.cursor()
 		self._glos.setDefaultDefiFormat("h")
-		self._glos.setInfo("definition_has_headwords", "True")
 
 	def __len__(self):
 		self._cur.execute(
@@ -92,7 +89,7 @@ class Reader(object):
 			f"select {column1}, {column2}, entry_type from main_ft"
 			f" order by {column1}"
 		)
-		for row in self._cur:
+		for row in self._cur.fetchall():
 			term1 = row[0]
 			term2 = row[1]
 			try:
@@ -105,16 +102,41 @@ class Reader(object):
 				log.error(f"html.unescape({term2!r}) -> {e}")
 			yield term1, term2, row[2]
 
+	def parseGender(self, headword):
+		# {m}	masc	masculine	German: maskulin
+		# {f}	fem 	feminine	German: feminin
+		# {n}	neut	neutral		German: neutral
+		# { }	????
+		i = headword.find(" {")
+		if i <= 0:
+			return None, headword
+		if len(headword) < i + 4:
+			return None, headword
+		if headword[i + 3] != "}":
+			return None, headword
+		g = headword[i + 2]
+		gender = None
+		if g == "m":
+			gender = "masculine"
+		elif g == "f":
+			gender = "feminine"
+		elif g == "n":
+			gender = "neutral"
+		else:
+			log.warning(f"invalid gender {g!r}")
+			return None, headword
+		headword = headword[:i] + headword[i + 4:]
+		return gender, headword
+
 	def _iterOneDirection(self, column1, column2):
 		from itertools import groupby
 		from lxml import etree as ET
 		from io import BytesIO
-		from pyglossary.html_utils import unescape_unicode
 
 		glos = self._glos
 		for headword, groupsOrig in groupby(
 			self.iterRows(column1, column2),
-			key=lambda row: row[0],
+			key=itemgetter(0),
 		):
 			headword = html.unescape(headword)
 			groups = [
@@ -122,22 +144,19 @@ class Reader(object):
 				for _, term2, entry_type in groupsOrig
 			]
 			f = BytesIO()
-			with ET.htmlfile(f) as hf:
+			gender, headword = self.parseGender(headword)
+			with ET.htmlfile(f, encoding="utf-8") as hf:
 				with hf.element("div"):
-					with glos.titleElement(hf, headword):
-						try:
-							hf.write(headword)
-						except Exception as e:
-							log.error(f"error in writing {headword!r}, {e}")
-							hf.write(repr(headword))
-					if len(groups) == 1:
+					if gender:
+						with hf.element("i"):
+							hf.write(gender)
 						hf.write(ET.Element("br"))
 					self.makeList(
 						hf,
 						groups,
 						self.writeSense,
 					)
-			defi = unescape_unicode(f.getvalue().decode("utf-8"))
+			defi = f.getvalue().decode("utf-8")
 			yield self._glos.newEntry(headword, defi, defiFormat="h")
 
 	def __iter__(self):
