@@ -1,23 +1,20 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 import html
 
 enable = True
+lname = "dict_cc_split"
 format = 'Dictcc_split'
 description = 'Dict.cc (SQLite3) - Split'
 extensions = ()
-readOptions = []
-writeOptions = []
-
-tools = [
-	{
-		"name": "dict.cc dictionary",
-		"web": "https://play.google.com/store/apps/details?id=cc.dict.dictcc",
-		"platforms": ["Android"],
-		"license": "Proprietary",
-	},
-]
+extensionCreate = ".db"
+kind = "binary"
+wiki = "https://en.wikipedia.org/wiki/Dict.cc"
+website = (
+	"https://play.google.com/store/apps/details?id=cc.dict.dictcc",
+	"dict.cc dictionary - Google Play",
+)
 
 
 class Reader(object):
@@ -36,7 +33,6 @@ class Reader(object):
 		self._con = connect(filename)
 		self._cur = self._con.cursor()
 		self._glos.setDefaultDefiFormat("m")
-		self._glos.setInfo("definition_has_headwords", "False")
 
 	def __len__(self):
 		self._cur.execute("select count(*) * 2 from main_ft")
@@ -47,7 +43,7 @@ class Reader(object):
 			f"select {column1}, {column2}, entry_type from main_ft"
 			f" order by {column1}"
 		)
-		for row in self._cur:
+		for row in self._cur.fetchall():
 			term1 = row[0]
 			term2 = row[1]
 			try:
@@ -59,7 +55,6 @@ class Reader(object):
 			except Exception as e:
 				log.error(f"html.unescape({term2!r}) -> {e}")
 			yield term1, term2, row[2]
-		
 
 	def _iterOneDirection(self, column1, column2):
 		for word, defi, entry_type in self.iterRows(column1, column2):
