@@ -1,59 +1,42 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+import re
+from pyglossary.plugins.formats_common import *
 from pyglossary.file_utils import fileCountLines
 from pyglossary.plugin_lib.dictdlib import DictDB
 
 enable = True
+lname = "dict_org"
 format = "DictOrg"
 description = "DICT.org file format (.index)"
 extensions = (".index",)
 optionsProp = {
-	"dictzip": BoolOption(),
-	"install": BoolOption(),
+	"dictzip": BoolOption(comment="Compress .dict file to .dict.dz"),
+	"install": BoolOption(comment="Install dictionary to /usr/share/dictd/"),
 }
 sortOnWrite = DEFAULT_NO
-
-# https://en.wikipedia.org/wiki/DICT#DICT_file_format
-tools = [
-	{
-		"name": "Dictd",
-		"web": "https://directory.fsf.org/wiki/Dictd",
-		"platforms": ["Linux"],
-		"license": "GPL",
-	},
-	{
-		"name": "GNOME Dictionary",
-		"web": "https://wiki.gnome.org/Apps/Dictionary",
-		"platforms": ["Linux"],
-		"license": "GPL",
-	},
-	{
-		"name": "Xfce4 Dictionary",
-		"web": "https://docs.xfce.org/apps/xfce4-dict/start",
-		"platforms": ["linux"],
-		"license": "GPL",
-	},
-	{
-		"name": "Ding",
-		"desc": "Graphical dictionary lookup program for Unix (Tk)",
-		"web": "https://www-user.tu-chemnitz.de/~fri/ding/",
-		"platforms": ["linux"],
-		"license": "GPL",
-		"copyright": "Copyright (c) 1999 - 2016 Frank Richter",
-	},
-]
+kind = "directory"
+wiki = "https://en.wikipedia.org/wiki/DICT#DICT_file_format"
+website = (
+	"http://dict.org/bin/Dict",
+	"The DICT Development Group",
+)
 
 
 def installToDictd(filename: str, dictzip: bool, title: str = "") -> None:
 	"""
 	filename is without extension (neither .index or .dict or .dict.dz)
 	"""
-	import shutil, subprocess
+	import shutil
+	import subprocess
 	targetDir = "/usr/share/dictd/"
 	if filename.startswith(targetDir):
 		return
 
+	if not isdir(targetDir):
+		log.warning(f"Directory {targetDir!r} does not exist, skipping install")
+		return
+
 	log.info(f"Installing {filename!r} to DICTD server directory: {targetDir}")
 
 	if dictzip and os.path.isfile(filename + ".dict.dz"):
@@ -84,6 +67,14 @@ class Reader(object):
 		self._filename = ""
 		self._dictdb = None  # type: Optional[DictDB]
 
+		# regular expression patterns used to prettify definition text
+		self._re_newline_in_braces = re.compile(
+			r'\{(?P<left>.*?)\n(?P<right>.*?)?\}',
+		)
+		self._re_words_in_braces = re.compile(
+			r'\{(?P<word>.+?)\}',
+		)
+
 	def open(self, filename: str) -> None:
 		import gzip
 		if filename.endswith(".index"):
@@ -98,6 +89,22 @@ class Reader(object):
 			# self._dictdb.finish()
 			self._dictdb = None
 
+	def prettifyDefinitionText(self, defi: str) -> str:
+		# Handle words in {}
+		# First, we remove any \n in {} pairs
+		defi = self._re_newline_in_braces.sub(r'{\g<left>\g<right>}', defi)
+
+		# Then, replace any {words} into <a href="bword://words">words</a>,
+		# so it can be rendered as link correctly
+		defi = self._re_words_in_braces.sub(
+			r'<a href="bword://\g<word>">\g<word></a>',
+			defi,
+		)
+
+		# Use <br /> so it can be rendered as newline correctly
+		defi = defi.replace("\n", "<br />")
+		return defi
+
 	def __len__(self) -> int:
 		if self._dictdb is None:
 			return 0
@@ -105,13 +112,13 @@ class Reader(object):
 
 	def __iter__(self) -> "Iterator[BaseEntry]":
 		if self._dictdb is None:
-			log.error("reader is not open, can not iterate")
-			raise StopIteration
+			raise RuntimeError("iterating over a reader while it's not open")
 		dictdb = self._dictdb
 		for word in dictdb.getdeflist():
-			b_defi = b"\n<hr>\n".join(dictdb.getdef(word))
+			b_defi = b"\n\n<hr>\n\n".join(dictdb.getdef(word))
 			try:
-				defi = b_defi.decode("utf_8")
+				defi = b_defi.decode("utf_8", 'ignore')
+				defi = self.prettifyDefinitionText(defi)
 			except Exception as e:
 				log.error(f"b_defi = {b_defi}")
 				raise e
@@ -157,4 +164,3 @@ class Writer(object):
 				# does dictd support resources? and how? FIXME
 				continue
 			dictdb.addentry(entry.b_defi, entry.l_word)
-
