@@ -1,25 +1,24 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 
 enable = True
+lname = "dict_org_source"
 format = "DictOrgSource"
 description = "DICT.org dictfmt source file"
 extensions = (".dtxt",)
+extensionCreate = ".dtxt"
 singleFile = True
+kind = "text"
+wiki = "https://en.wikipedia.org/wiki/DICT"
+website = (
+	"https://github.com/cheusov/dictd",
+	"@cheusov/dictd",
+)
 optionsProp = {
-	"remove_html_all": BoolOption(),
+	"remove_html_all": BoolOption(comment="Remove all HTML tags"),
 }
 
-tools = [
-	{
-		"name": "dictfmt",
-		"web": "https://linux.die.net/man/1/dictfmt",
-		"platforms": ["Linux"],
-		"license": "GPL",
-	},
-]
-
 
 class Writer(object):
 	_remove_html_all: bool = True
@@ -39,7 +38,9 @@ class Writer(object):
 		# supported by GtkTextView
 
 	def write(self) -> "Generator[None, BaseEntry, None]":
-		yield from self._glos.writeTxt(
+		from pyglossary.text_writer import writeTxt
+		yield from writeTxt(
+			self._glos,
 			entryFmt=":{word}:{defi}\n",
 			filename=self._filename,
 			defiEscapeFunc=replaceStringTable([
