@@ -1,24 +1,23 @@
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from pyglossary.text_reader import TextGlossaryReader
 
 enable = True
+lname = "dictunformat"
 format = "Dictunformat"
 description = "dictunformat output file"
 extensions = (".dictunformat",)
+extensionCreate = ".dictunformat"
 singleFile = True
+kind = "text"
+wiki = "https://directory.fsf.org/wiki/Dictd"
+website = (
+	"https://github.com/cheusov/dictd/blob/master/dictunformat.1.in",
+	"dictd/dictunformat.1.in - @cheusov/dictd",
+)
 optionsProp = {
 	"encoding": EncodingOption(),
 }
 
-tools = [
-	{
-		"name": "dictunformat",
-		"web": "https://linux.die.net/man/1/dictunformat",
-		"platforms": ["Linux"],
-		"license": "GPL",
-	},
-]
-
 
 def unescapeDefi(defi: str) -> str:
 	return defi
@@ -73,7 +72,7 @@ class Reader(TextGlossaryReader):
 				if not word:
 					continue
 				if not defiLines:
-					log.warn(f"no definition/value for {word!r}")
+					log.warning(f"no definition/value for {word!r}")
 				defi = unescapeDefi("\n".join(defiLines))
 				return word, defi
 
