@@ -1,24 +1,20 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 import html
 
 enable = True
+lname = "digitalnk"
 format = 'DigitalNK'
 description = 'DigitalNK (SQLite3, N-Korean)'
 extensions = ()
-readOptions = []
-writeOptions = []
-
-tools = [
-	{
-		"name": "Dic.rs",
-		"web": "https://github.com/digitalprk/dicrs",
-		"platforms": ["Linux"],
-		"license": " BSD-2-Clause",
-	},
-]
-
+extensionCreate = ".db"
+kind = "binary"
+wiki = ""
+website = (
+	"https://github.com/digitalprk/dicrs",
+	"@digitalprk/dicrs",
+)
 
 class Reader(object):
 	def __init__(self, glos):
@@ -46,7 +42,12 @@ class Reader(object):
 			"select word, definition from dictionary"
 			" order by word"
 		)
-		for row in self._cur:
+		# iteration over self._cur stops after one entry
+		# and self._cur.fetchone() returns None
+		# no idea why!
+		# https://github.com/ilius/pyglossary/issues/282
+		# for row in self._cur:
+		for row in self._cur.fetchall():
 			word = html.unescape(row[0])
 			definition = row[1]
 			yield self._glos.newEntry(word, definition, defiFormat="m")
