@@ -19,10 +19,12 @@
 # GNU General Public License for more details.
 
 import re
+import html
 import html.entities
 from xml.sax.saxutils import escape, quoteattr
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
+from pyglossary.text_reader import TextFilePosWrapper
 
 from . import layer
 from . import tag
@@ -31,33 +33,24 @@ from .main import (
 )
 
 enable = True
+lname = "dsl"
 format = "ABBYYLingvoDSL"
-description = "ABBYY Lingvo DSL (dsl)"
+description = "ABBYY Lingvo DSL (.dsl)"
 extensions = (".dsl",)
+extensionCreate = ".dsl"
 singleFile = True
+kind = "text"
+wiki = "https://ru.wikipedia.org/wiki/ABBYY_Lingvo"
+website = (
+	"https://www.lingvo.ru/",
+	"www.lingvo.ru",
+)
 optionsProp = {
 	"encoding": EncodingOption(),
-	"audio": BoolOption(),
-	"onlyFixMarkUp": BoolOption(),
+	"audio": BoolOption(comment="Enable audio objects"),
+	"only_fix_markup": BoolOption(comment="Only fix markup, without tag conversion"),
 }
 
-tools = [
-	{
-		"name": "ABBYY Lingvo",
-		"web": "https://www.lingvo.ru/",
-		# https://ru.wikipedia.org/wiki/ABBYY_Lingvo
-		"platforms": [
-			"Windows",
-			"Mac",
-			"Android",
-			"iOS",
-			"Windows Mobile",
-			"Symbian",
-		],
-		"license": "Proprietary",
-	},
-]
-
 # ABBYY is a Russian company
 # https://ru.wikipedia.org/wiki/ABBYY_Lingvo
 # http://lingvo.helpmax.net/en/troubleshooting/dsl-compiler/compiling-a-dictionary/
@@ -79,6 +72,7 @@ __all__ = ["read"]
 
 htmlEntityPattern = re.compile(r"&#?\w+;")
 
+
 def unescape(text):
 	def fixup(m):
 		text = m.group(0)
@@ -119,24 +113,6 @@ def ref_sub(x):
 # order matters, a lot.
 shortcuts = [
 	# canonical: m > * > ex > i > c
-	(
-		"[i][c](.*?)[/c][/i]",
-		"<i style=\"color:green\">\\g<1></i>"
-	),
-	(
-		"[m(\\d)][ex](.*?)[/ex][/m]",
-		"<div class=\"ex\" "
-		"style=\"margin-left:\\g<1>em;color:steelblue\">\\g<2></div>"
-	),
-	(
-		"[m(\\d)][*][ex](.*?)[/ex][/*][/m]",
-		"<div class=\"sec ex\" "
-		"style=\"margin-left:\\g<1>em;color:steelblue\">\\g<2></div>"
-	),
-	(
-		"[*][ex](.*?)[/ex][/*]",
-		"<span class=\"sec ex\" style=\"color:steelblue\">\\g<1></span>"
-	),
 	(
 		"[m1](?:-{2,})[/m]",
 		"<hr/>"
@@ -181,15 +157,11 @@ def apply_shortcuts(line):
 
 def _clean_tags(line, audio):
 	r"""
-	WARNING! shortcuts may apply:
-		[m2][*][ex]{}[/ex][/*][/m]
-		=>
-		<div class="sec ex" style="margin-left:2em;color:steelblue">{}</div>
 	[m{}] => <div style="margin-left:{}em">
 	[*]   => <span class="sec">
-	[ex]  => <span class="ex" style="color:steelblue">
-	[c]   => <span style="color:green">
-	[p]   => <i class="p" style="color:green">
+	[ex]  => <span class="ex"><font color="steelblue">
+	[c]   => <font color="green">
+	[p]   => <i class="p"><font color="green">
 
 	[']   => <u>
 	[b]   => <b>
@@ -207,7 +179,7 @@ def _clean_tags(line, audio):
 			</object>
 	[s] =>  <img align="top" src="{}" alt="{}" />
 
-	[t] => <!-- T --><span style="font-family:'Helvetica'">
+	[t] => <font face="Helvetica" class="dsl_t">
 
 	{{...}}   \
 	[trn]      |
@@ -235,12 +207,16 @@ def _clean_tags(line, audio):
 	line = re_lang_open.sub("", line).replace("[/lang]", "")
 	# remove com tags
 	line = line.replace("[com]", "").replace("[/com]", "")
+
+	# escape html special characters like '<' and '>'
+	line = html.escape(html.unescape(line))
+
 	# remove t tags
 	line = line.replace(
 		"[t]",
-		"<!-- T --><span style=\"font-family:'Helvetica'\">"
+		"<font face=\"Helvetica\" class=\"dsl_t\">"
 	)
-	line = line.replace("[/t]", "</span><!-- T -->")
+	line = line.replace("[/t]", "</font>")
 
 	line = _parse(line)
 
@@ -269,21 +245,21 @@ def _clean_tags(line, audio):
 	line = line.replace("[sub]", "<sub>").replace("[/sub]", "</sub>")
 
 	# color
-	line = line.replace("[c]", "<span style=\"color:green\">")
-	line = re_c_open_color.sub("<span style=\"color:\\g<1>\">", line)
-	line = line.replace("[/c]", "</span>")
+	line = line.replace("[c]", "<font color=\"green\">")
+	line = re_c_open_color.sub("<font color=\"\\g<1>\">", line)
+	line = line.replace("[/c]", "</font>")
 
 	# example zone
-	line = line.replace("[ex]", "<span class=\"ex\" style=\"color:steelblue\">")
-	line = line.replace("[/ex]", "</span>")
+	line = line.replace("[ex]", "<span class=\"ex\"><font color=\"steelblue\">")
+	line = line.replace("[/ex]", "</font></span>")
 
 	# secondary zone
 	line = line.replace("[*]", "<span class=\"sec\">")\
 		.replace("[/*]", "</span>")
 
 	# abbrev. label
-	line = line.replace("[p]", "<i class=\"p\" style=\"color:green\">")
-	line = line.replace("[/p]", "</i>")
+	line = line.replace("[p]", "<i class=\"p\"><font color=\"green\">")
+	line = line.replace("[/p]", "</font></i>")
 
 	# cross reference
 	line = line.replace("[ref]", "<<").replace("[/ref]", ">>")
@@ -316,11 +292,11 @@ def unwrap_quotes(s):
 
 
 class Reader(object):
-	compressions = stdCompressions
+	compressions = stdCompressions + ("dz",)
 
 	_encoding: str = ""
 	_audio: bool = False
-	_onlyFixMarkUp: bool = False
+	_only_fix_markup: bool = False
 
 	re_tags_open = re.compile(r"(?<!\\)\[(c |[cuib]\])")
 	re_tags_close = re.compile(r"\[/[cuib]\]")
@@ -329,6 +305,7 @@ class Reader(object):
 		self._glos = glos
 		self.clean_tags = _clean_tags
 		self._file = None
+		self._fileSize = 0
 		self._bufferLine = ""
 
 	def close(self):
@@ -348,7 +325,7 @@ class Reader(object):
 		filename: str,
 	) -> None:
 		self._filename = filename
-		if self._onlyFixMarkUp:
+		if self._only_fix_markup:
 			self.clean_tags = self._clean_tags_only_markup
 		else:
 			self.clean_tags = _clean_tags
@@ -356,11 +333,21 @@ class Reader(object):
 		encoding = self._encoding
 		if not encoding:
 			encoding = self.detectEncoding()
-		self._file = compressionOpen(filename, mode="rt", encoding=encoding)
+		cfile = compressionOpen(
+			filename,
+			dz=True,
+			mode="rt",
+			encoding=encoding,
+		)
+		cfile.seek(0, 2)
+		self._fileSize = cfile.tell()
+		cfile.seek(0)
+		self._file = TextFilePosWrapper(cfile, encoding)
 
 		# read header
 		for line in self._file:
-			line = line.rstrip()
+			line = line.rstrip().lstrip('\ufeff')
+			# \ufeff -> https://github.com/ilius/pyglossary/issues/306
 			if not line:
 				continue
 			if not line.startswith("#"):
@@ -370,7 +357,12 @@ class Reader(object):
 
 	def detectEncoding(self):
 		for testEncoding in ("utf-8", "utf-16"):
-			with compressionOpen(self._filename, mode="rt", encoding=testEncoding) as fileObj:
+			with compressionOpen(
+				self._filename,
+				dz=True,
+				mode="rt",
+				encoding=testEncoding,
+			) as fileObj:
 				try:
 					for i in range(10):
 						fileObj.readline()
@@ -390,11 +382,11 @@ class Reader(object):
 
 	def processHeaderLine(self, line):
 		if line.startswith("#NAME"):
-			self.setInfo("name", line[6:])
+			self.setInfo("name", unwrap_quotes(line[6:].strip()))
 		elif line.startswith("#INDEX_LANGUAGE"):
-			self._glos.sourceLangName = unwrap_quotes(line[16:])
+			self._glos.sourceLangName = unwrap_quotes(line[16:].strip())
 		elif line.startswith("#CONTENTS_LANGUAGE"):
-			self._glos.targetLangName = unwrap_quotes(line[19:])
+			self._glos.targetLangName = unwrap_quotes(line[19:].strip())
 
 	def _iterLines(self) -> "Iterator[str]":
 		if self._bufferLine:
@@ -423,7 +415,7 @@ class Reader(object):
 				line_type = "text"
 				line = unfinished_line + line.lstrip()
 
-				# some ill formated source may have tags spanned into
+				# some ill formatted source may have tags spanned into
 				# multiple lines
 				# try to match opening and closing tags
 				tags_open = re_tags_open.findall(line)
@@ -449,11 +441,12 @@ class Reader(object):
 			# append previous entry
 			if line_type == "text":
 				if unfinished_line:
-					# line may be skipped if ill formated
+					# line may be skipped if ill formatted
 					current_text.append(self.clean_tags(unfinished_line, self._audio))
 				yield self._glos.newEntry(
 					[current_key] + current_key_alters,
 					"\n".join(current_text),
+					byteProgress=(self._file.tell(), self._fileSize),
 				)
 
 			# start new entry
