@@ -19,65 +19,44 @@
 # SOFTWARE.
 
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from pyglossary.ebook_base import *
 
 enable = True
+lname = "epub2"
 format = "Epub2"
 description = "EPUB-2 E-Book"
 extensions = (".epub",)
+extensionCreate = ".epub"
 sortOnWrite = ALWAYS
+sortKeyName = "ebook"
+kind = "package"
+wiki = "https://en.wikipedia.org/wiki/EPUB"
+website = None
 
-# https://en.wikipedia.org/wiki/EPUB
 # EPUB-3: https://www.w3.org/community/epub3/
-tools = [
-	{
-		"name": "calibre",
-		"web": "https://calibre-ebook.com/",
-		"platforms": ["Linux", "Windows", "Mac"],
-		"license": "GPL",
-	},
-	{
-		"name": "Okular",
-		"web": "https://okular.kde.org/",
-		"platforms": ["Linux", "Windows", "Mac"],
-		"license": "GPL",
-	},
-	{
-		"name": " Book Reader ",
-		"web": "https://f-droid.org/en/packages/com.github.axet.bookreader/",
-		"platforms": ["Android"],
-		"license": "GPL",
-	},
-	{
-		"name": "Kobo eReader",
-		"web": "https://www.kobo.com",
-		"platforms": ["Kobo eReader"],
-		"license": "Proprietary",
-	},
-	{
-		"name": "Icecream Ebook Reader",
-		"web": "https://icecreamapps.com/Ebook-Reader/",
-		"platforms": ["Windows"],
-		"license": "Proprietary",
-	},
-	{
-		"name": "Aldiko",
-		"web": "https://www.demarque.com/aldiko",
-		"platforms": ["Android", "iOS"],
-		"license": "Proprietary",
-	},
-]
 
 optionsProp = {
-	"group_by_prefix_length": IntOption(),
+	"group_by_prefix_length": IntOption(
+		comment="Prefix length for grouping",
+	),
 	# "group_by_prefix_merge_min_size": IntOption(),
 	# "group_by_prefix_merge_across_first": BoolOption(),
-	"compress": BoolOption(),
-	"keep": BoolOption(),
-	"include_index_page": BoolOption(),
-	"apply_css": StrOption(comment="path to css file"),
-	"cover_path": StrOption(comment="path to cover file"),
+	"compress": BoolOption(
+		comment="Enable compression",
+	),
+	"keep": BoolOption(
+		comment="Keep temp files",
+	),
+	"include_index_page": BoolOption(
+		comment="Include index page",
+	),
+	"css": StrOption(
+		comment="Path to css file",
+	),
+	"cover_path": StrOption(
+		comment="Path to cover file",
+	),
 }
 
 
@@ -224,6 +203,25 @@ p.groupDefinition {
 		)
 		glos.setInfo("uuid", str(uuid.uuid4()).replace("-", ""))
 
+	@classmethod
+	def cls_get_prefix(cls, options: "Dict[str, Any]", word: str) -> str:
+		if not word:
+			return None
+		length = options.get("group_by_prefix_length", cls._group_by_prefix_length)
+		prefix = word[:length].lower()
+		if prefix[0] < "a":
+			return "SPECIAL"
+		return prefix
+
+	def get_prefix(self, word: str) -> str:
+		if not word:
+			return None
+		length = self._group_by_prefix_length
+		prefix = word[:length].lower()
+		if prefix[0] < "a":
+			return "SPECIAL"
+		return prefix
+
 	def write_ncx(self, group_labels):
 		"""
 			write_ncx
