@@ -2,7 +2,7 @@
 # The MIT License (MIT)
 
 # Copyright © 2012-2016 Alberto Pettarin (alberto@albertopettarin.it)
-# Copyright © 2020 Saeed Rasooli <saeed.gnu@gmail.com>
+# Copyright © 2022 Saeed Rasooli <saeed.gnu@gmail.com>
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -22,28 +22,31 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from itertools import groupby
 from pathlib import Path
 import unicodedata
 import re
 from pickle import dumps, loads
 from gzip import compress, decompress
+from operator import itemgetter
 
 enable = True
+lname = "kobo"
 format = "Kobo"
 description = "Kobo E-Reader Dictionary"
 extensions = (".kobo",)
+extensionCreate = ".kobo.zip"
+kind = "package"
 sortOnWrite = NEVER
+wiki = "https://en.wikipedia.org/wiki/Kobo_eReader"
+website = (
+	"https://www.kobo.com",
+	"www.kobo.com",
+)
+
+# https://help.kobo.com/hc/en-us/articles/360017640093-Add-new-dictionaries-to-your-Kobo-eReader
 
-tools = [
-	{
-		"name": "Kobo eReader",
-		"web": "https://www.kobo.com",
-		"platforms": ["Kobo eReader"],
-		"license": "Proprietary",
-	},
-]
 
 optionsProp = {
 }
@@ -149,9 +152,9 @@ class Writer:
 			nonlocal htmlContents
 			group_fname = fixFilename(lastPrefix)
 			htmlContents += "</html>"
-			log.debug(
+			log.trace(
 				f"writeGroup: {lastPrefix!r}, "
-				"{group_fname!r}, count={groupCounter}"
+				f"{group_fname!r}, count={groupCounter}"
 			)
 			with gzip.open(group_fname + ".html", mode="wb") as gzipFile:
 				gzipFile.write(htmlContents.encode("utf-8"))
@@ -194,7 +197,7 @@ class Writer:
 			del entry
 
 		log.info(f"Kobo: sorting entries...")
-		data.sort(key=lambda x: x[0])
+		data.sort(key=itemgetter(0))
 
 		log.info(f"Kobo: writing entries...")
 
@@ -219,7 +222,7 @@ class Writer:
 			writeGroup(lastPrefix)
 
 		if dataEntryCount > 0:
-			log.warn(
+			log.warning(
 				f"ignored {dataEntryCount} files (data entries)"
 				" and replaced '<img ...' tags in definitions with placeholders"
 			)
