@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # The MIT License (MIT)
 
-# Copyright © 2020 Saeed Rasooli <saeed.gnu@gmail.com>
+# Copyright © 2020-2021 Saeed Rasooli <saeed.gnu@gmail.com>
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,30 +21,29 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from pyglossary.text_reader import TextGlossaryReader
 from pyglossary.image_utils import extractInlineHtmlImages
 
 enable = True
+lname = "kobo_dictfile"
 format = "Dictfile"
-description = "Kobo E-Reader Dictfile"
+description = "Kobo E-Reader Dictfile (.df)"
 extensions = (".df",)
+extensionCreate = ".df"
+kind = "text"
+wiki = ""
+website = (
+	"https://pgaskin.net/dictutil/dictgen/#dictfile-format",
+	"dictgen - dictutil",
+)
 # https://github.com/pgaskin/dictutil
 
 optionsProp = {
 	"encoding": EncodingOption(),
-	"extract_inline_images": BoolOption(),
+	"extract_inline_images": BoolOption(comment="Extract inline images"),
 }
 
-tools = [
-	{
-		"name": "dictgen",
-		"web": "https://pgaskin.net/dictutil/dictgen/",
-		"platforms": ["Linux", "Windows", "Mac"],
-		"license": "MIT",
-	},
-]
-
 
 def fixWord(word: str) -> str:
 	return word.replace("\n", " ")
@@ -63,6 +62,9 @@ class Reader(TextGlossaryReader):
 
 	_extract_inline_images = True
 
+	def __init__(self, glos: "GlossaryType"):
+		TextGlossaryReader.__init__(self, glos, hasInfo=False)
+
 	def open(self, filename: str) -> None:
 		try:
 			import mistune
@@ -78,17 +80,17 @@ class Reader(TextGlossaryReader):
 	def fixInfoWord(self, word):
 		raise NotImplementedError
 
-	def fixDefi(self, defi: str) -> str:
+	def fixDefi(self, defi: str, html: bool) -> str:
 		import mistune
 		defi = defi.replace("\n @", "\n@")\
 			.replace("\n :", "\n:")\
-			.replace("\n &", "\n&")
+			.replace("\n &", "\n&")\
+			.replace("</p><br />", "</p>")\
+			.replace("</p><br/>", "</p>")\
+			.replace("</p></br>", "</p>")
 		defi = defi.lstrip()
-		if defi.startswith("<html>"):
-			defi = defi[len("<html>"):].lstrip()
-			i = defi.find("</html>")
-			if i >= 0:
-				defi = defi[:i]
+		if html:
+			pass
 		else:
 			defi = mistune.html(defi)
 		if self._extract_inline_images:
@@ -106,6 +108,7 @@ class Reader(TextGlossaryReader):
 			raise StopIteration
 		words = []
 		defiLines = []
+		html = False
 
 		while True:
 			line = self.readline()
@@ -115,16 +118,24 @@ class Reader(TextGlossaryReader):
 			if line.startswith("@"):
 				if words:
 					self._bufferLine = line
-					return words, self.fixDefi("\n".join(defiLines))
+					return words, self.fixDefi("\n".join(defiLines), html=html)
 				words = [line[1:]]
 				continue
+			if line.startswith(": "):
+				defiLines.append(line[2:])
+				continue
+			if line.startswith("::"):
+				continue
 			if line.startswith("&"):
 				words.append(line[1:])
 				continue
+			if line.startswith("<html>"):
+				line = line[6:]
+				html = True
 			defiLines.append(line)
 
 		if words:
-			return words, self.fixDefi("\n".join(defiLines))
+			return words, self.fixDefi("\n".join(defiLines), html=html)
 
 		raise StopIteration
 
