@@ -19,7 +19,7 @@
 # If not, see <http://www.gnu.org/licenses/gpl.txt>.
 
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from pyglossary.text_utils import (
 	escapeNTB,
 	unescapeNTB,
@@ -27,13 +27,18 @@ from pyglossary.text_utils import (
 )
 
 enable = True
+lname = "edlin"
 format = "Edlin"
 # Editable Linked List of Entries
 description = "EDLIN"
 extensions = (".edlin",)
+extensionCreate = ".edlin/"
+kind = "directory"
+wiki = ""
+website = None
 optionsProp = {
 	"encoding": EncodingOption(),
-	"havePrevLink": BoolOption(),
+	"prev_link": BoolOption(comment="Enable link to previous entry"),
 }
 
 
@@ -54,7 +59,7 @@ class Reader(object):
 
 	def _clear(self) -> None:
 		self._filename = ""
-		self._havePrevLink = True
+		self._prev_link = True
 		self._wordCount = None
 		self._rootPath = None
 		self._resDir = ""
@@ -76,7 +81,7 @@ class Reader(object):
 		with open(infoFname, "r", encoding=self._encoding) as infoFp:
 			info = jsonToOrderedData(infoFp.read())
 		self._wordCount = info.pop("wordCount")
-		self._havePrevLink = info.pop("havePrevLink")
+		self._prev_link = info.pop("prev_link")
 		self._rootPath = info.pop("root")
 		for key, value in info.items():
 			self._glos.setInfo(key, value)
@@ -96,8 +101,7 @@ class Reader(object):
 
 	def __iter__(self) -> "Iterator[BaseEntry]":
 		if not self._rootPath:
-			log.error("iterating over a reader which is not open")
-			raise StopIteration
+			raise RuntimeError("iterating over a reader while it's not open")
 
 		wordCount = 0
 		nextPath = self._rootPath
@@ -110,17 +114,17 @@ class Reader(object):
 				join(self._filename, nextPath),
 				"r",
 				encoding=self._encoding,
-			) as fromFile:
-				header = fromFile.readline().rstrip()
-				if self._havePrevLink:
+			) as _file:
+				header = _file.readline().rstrip()
+				if self._prev_link:
 					_prevPath, nextPath = header.split(" ")
 				else:
 					nextPath = header
-				word = fromFile.readline()
+				word = _file.readline()
 				if not word:
 					yield None  # update progressbar
 					continue
-				defi = fromFile.read()
+				defi = _file.read()
 				if not defi:
 					log.warning(
 						f"Edlin Reader: no definition for word {word!r}"
@@ -131,7 +135,7 @@ class Reader(object):
 				word = word.rstrip()
 				defi = defi.rstrip()
 
-			if self._glos.getConfig("enable_alts", True):
+			if self._glos.alts:
 				word = splitByBarUnescapeNTB(word)
 				if len(word) == 1:
 					word = word[0]
@@ -150,16 +154,16 @@ class Reader(object):
 
 		resDir = self._resDir
 		for fname in self._resFileNames:
-			with open(join(resDir, fname), "rb") as fromFile:
+			with open(join(resDir, fname), "rb") as _file:
 				yield self._glos.newDataEntry(
 					fname,
-					fromFile.read(),
+					_file.read(),
 				)
 
 
 class Writer(object):
 	_encoding: str = "utf-8"
-	_havePrevLink: bool = True
+	_prev_link: bool = True
 
 	def __init__(self, glos: GlossaryType):
 		self._glos = glos
@@ -203,7 +207,13 @@ class Writer(object):
 				return tmp_hash
 			index += 1
 
-	def saveEntry(self, thisEntry: BaseEntry, thisHash: str, prevHash: str, nextHash: str) -> None:
+	def saveEntry(
+		self,
+		thisEntry: BaseEntry,
+		thisHash: str,
+		prevHash: str,
+		nextHash: str,
+	) -> None:
 		dpath = join(self._filename, thisHash[:2])
 		makeDir(dpath)
 		with open(
@@ -212,7 +222,7 @@ class Writer(object):
 			encoding=self._encoding,
 		) as toFile:
 			nextPath = self.hashToPath(nextHash) if nextHash else "END"
-			if self._havePrevLink:
+			if self._prev_link:
 				prevPath = self.hashToPath(prevHash) if prevHash else "START"
 				header = prevPath + " " + nextPath
 			else:
@@ -223,7 +233,6 @@ class Writer(object):
 				thisEntry.defi,
 			]))
 
-
 	def write(self) -> "Generator[None, BaseEntry, None]":
 		from collections import OrderedDict as odict
 		from pyglossary.json_utils import dataToPrettyJson
@@ -260,17 +269,16 @@ class Writer(object):
 			info = odict()
 			info["name"] = self._glos.getInfo("name")
 			info["root"] = self.hashToPath(rootHash)
-			info["havePrevLink"] = self._havePrevLink
+			info["prev_link"] = self._prev_link
 			info["wordCount"] = count
 			# info["modified"] =
 
 			for key, value in self._glos.getExtraInfos((
 				"name",
 				"root",
-				"havePrevLink",
+				"prev_link",
 				"wordCount",
 			)).items():
 				info[key] = value
 
 			toFile.write(dataToPrettyJson(info))
-
