@@ -1,5 +1,3 @@
-from formats_common import *
-
 import sys
 import os
 from os.path import (
@@ -17,7 +15,7 @@ log = logging.getLogger("pyglossary")
 
 from pprint import pformat
 
-from paths import rootDir
+from pyglossary.core import rootDir
 sys.path.insert(0, rootDir)
 
 from pyglossary.flags import *
@@ -43,14 +41,17 @@ from pyglossary.entry_base import BaseEntry
 from pyglossary.glossary_type import GlossaryType
 
 enable = False
+lname = ""
 format = "Unknown"
 description = "Unknown"
 extensions = ()  # type: Tuple[str, ...]
+extensionCreate = ""  # type: str
 singleFile = False
+kind = ""
+wiki = ""
+website = None
 
 # key is option/argument name, value is instance of Option
 optionsProp = {}  # type: Dict[str, Option]
 
 sortOnWrite = DEFAULT_NO  # type: YesNoAlwaysNever
-
-tools = []  # type: List[Dict[str, Any]]
