@@ -1,31 +1,24 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 
 enable = True
+lname = "gettext_po"
 format = "GettextPo"
-description = "Gettext Source (po)"
+description = "Gettext Source (.po)"
 extensions = (".po",)
+extensionCreate = ".po"
 singleFile = True
+kind = "text"
+wiki = "https://en.wikipedia.org/wiki/Gettext"
+website = (
+	"https://www.gnu.org/software/gettext",
+	"gettext - GNU Project",
+)
 optionsProp = {
-	"resources": BoolOption(),
+	"resources": BoolOption(comment="Enable resources / data files"),
 }
 
-tools = [
-	{
-		"name": "gettext",
-		"web": "https://www.gnu.org/software/gettext/",
-		"platforms": ["Linux", "Windows"],
-		"license": "GPL",
-	},
-	{
-		"name": "poEdit",
-		"web": "https://github.com/vslavik/poedit",
-		"platforms": ["Linux", "Windows", "Mac"],
-		"license": "MIT / Shareware",
-	},
-]
-
 
 class Reader(object):
 	depends = {
