@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from pyglossary.text_utils import (
 	escapeNTB,
 	unescapeNTB,
@@ -10,19 +10,43 @@ import os
 import json
 
 enable = True
+lname = "html_dir"
 format = "HtmlDir"
 description = "HTML Directory"
 extensions = (".hdir",)
+extensionCreate = ".hdir/"
 singleFile = False
+kind = "directory"
+wiki = ""
+website = None
 optionsProp = {
 	"encoding": EncodingOption(),
-	"resources": BoolOption(),
-	"max_file_size": IntOption(),
-	"filename_format": StrOption(),
-	"escape_defi": BoolOption(),
-	"dark": BoolOption(),
+	"resources": BoolOption(
+		comment="Enable resources / data files",
+	),
+	"max_file_size": IntOption(
+		comment="Maximum file size in bytes",
+	),
+	"filename_format": StrOption(
+		comment="Filename format, default: {n:05d}.html",
+	),
+	"escape_defi": BoolOption(
+		comment="Escape definitions",
+	),
+	"dark": BoolOption(
+		comment="Use dark style",
+	),
+	"css": StrOption(
+		comment="Path to css file",
+	),
+	"word_title": BoolOption(
+		comment="Add headwords title to begining of definition",
+	),
 }
 
+nbsp = "\xa0"
+# nbsp = "&nbsp;"
+
 darkStyle = """
 body {{
 	background-color: #373737;
@@ -52,6 +76,8 @@ class Writer(object):
 	_filename_format: str = "{n:05d}.html"
 	_escape_defi: bool = False
 	_dark: bool = True
+	_css: str = ""
+	_word_title: bool = True
 
 	def __init__(self, glos: GlossaryType):
 		self._glos = glos
@@ -71,6 +97,12 @@ class Writer(object):
 			os.mkdir(filename)
 		if not isdir(resDir):
 			os.mkdir(resDir)
+		if self._css:
+			self.copyCSS(self._css)
+
+	def copyCSS(self, cssPath):
+		import shutil
+		shutil.copy(self._css, join(self._filename, "style.css"))
 
 	def finish(self):
 		pass
@@ -273,7 +305,7 @@ class Writer(object):
 			if not entry_url_fmt:
 				return ""
 			url = entry_url_fmt.format(word=html.escape(entry.l_word[0]))
-			return f'&nbsp;<a class="no_ul" href="{url}">&#127759;</a>'
+			return f'{nbsp}<a class="no_ul" href="{url}">&#127759;</a>'
 
 		# from math import log2, ceil
 		# maxPosHexLen = int(ceil(log2(max_file_size) / 4))
@@ -294,12 +326,17 @@ class Writer(object):
 		if self._dark:
 			style = darkStyle
 
+		if self._css:
+			cssLink = '<link rel="stylesheet" href="style.css" />'
+		else:
+			cssLink = ""
+
 		header = (
 			'<!DOCTYPE html>\n'
 			'<html><head>'
 			f'<title>{{pageTitle}}</title>'
 			f'<meta charset="{encoding}">'
-			f'<style type="text/css">{style}{{customStyle}}</style>'
+			f'<style type="text/css">{style}{{customStyle}}</style>{cssLink}'
 			'</meta></head><body>\n'
 		)
 
@@ -316,9 +353,9 @@ class Writer(object):
 			links.append(f'<a href="./{self.getNextFilename()}">&#9654;</a>')
 			links.append(f'<a href="./info.html">ℹ️</a></div>')
 			return (
-				'<div style="text-align: center; font-size: 2.5em;">' +
-				'&nbsp;&nbsp;&nbsp;'.join(links) +
-				'</div>'
+				'<nav style="text-align: center; font-size: 2.5em;">' +
+				f'{nbsp}{nbsp}{nbsp}'.join(links) +
+				'</nav>'
 			)
 
 		tailSize = len(self._tail.encode(encoding))
@@ -369,7 +406,7 @@ class Writer(object):
 
 		self.writeInfo(filename, header)
 
-		defiHasHeadwords = glos.getInfo("definition_has_headwords") == "True"
+		_word_title = self._word_title
 
 		resDir = self._resDir
 		entryIndex = -1
@@ -405,19 +442,26 @@ class Writer(object):
 
 			entryId = f"entry{entryIndex}"
 
-			if defiHasHeadwords:
-				headwords = f'Entry {entryIndex}'
-			else:
+			if _word_title:
 				words = [
 					html.escape(word)
 					for word in entry.l_word
 				]
-				headwords = f'<b class="headword">{wordSep.join(words)}</b>'
+				title = glos.wordTitleStr(
+					wordSep.join(words),
+					sample=entry.l_word[0],
+					_class="headword",
+				)
+
+			if not title:
+				title = f'Entry {entryIndex}'
+
 			# entry_link_sym = "&#182;"
 			entry_link_sym = "&#128279;"
 			text = (
-				f'<div id="{entryId}">{headwords}&nbsp;&nbsp;'
-				f'<a class="no_ul" class="entry_link" href="#{entryId}">{entry_link_sym}</a>'
+				f'<div id="{entryId}">{title}{nbsp}{nbsp}'
+				f'<a class="no_ul" class="entry_link" href="#{entryId}">'
+				f'{entry_link_sym}</a>'
 				f'{getEntryWebLink(entry)}'
 				f"<br>\n{defi}"
 				'</div>\n'
