@@ -1,21 +1,28 @@
 # -*- coding: utf-8 -*-
-from formats_common import *
-from pyglossary.xml_utils import xml_escape
-from pyglossary.html_utils import unescape_unicode
+from pyglossary.plugins.formats_common import *
 from io import BytesIO
 import re
 import html
 
 enable = True
+lname = "jmdict"
 format = "JMDict"
 description = "JMDict"
 extensions = ()
+extensionCreate = ""
 singleFile = True
+kind = "text"
+wiki = "https://en.wikipedia.org/wiki/JMdict"
+website = (
+	"https://www.edrdg.org/jmdict/j_jmdict.html",
+	"The JMDict Project",
+)
 optionsProp = {
 }
 
 
 class Reader(object):
+	compressions = stdCompressions
 	depends = {
 		"lxml": "lxml",
 	}
@@ -30,7 +37,8 @@ class Reader(object):
 	)
 
 	re_inf_mapping = {
-		"gikun (meaning as reading) or jukujikun (special kanji reading)": "gikun/jukujikun",
+		"gikun (meaning as reading) or jukujikun (special kanji reading)":
+			"gikun/jukujikun",
 		"out-dated or obsolete kana usage": "obsolete",  # outdated/obsolete
 		"word containing irregular kana usage": "irregular",
 	}
@@ -67,6 +75,16 @@ class Reader(object):
 		def br():
 			return ET.Element("br")
 
+		for elem in sense.findall("pos"):
+			if not elem.text:
+				continue
+			desc = elem.text
+			if desc == "unclassified":
+				continue
+			with hf.element("i"):
+				hf.write(f"{desc.capitalize()}")
+			hf.write(br())
+
 		glossList = [
 			elem.text.strip()
 			for elem in sense.findall("gloss")
@@ -79,16 +97,6 @@ class Reader(object):
 				hf.write(gloss)
 			hf.write(br())
 
-		for elem in sense.findall("pos"):
-			if not elem.text:
-				continue
-			desc = elem.text
-			if desc == "unclassified":
-				continue
-			with hf.element("i"):
-				hf.write(f"{desc.capitalize()}")
-			hf.write(br())
-
 		relatedWords = []
 		for elem in sense.findall("xref"):
 			if not elem.text:
@@ -154,7 +162,7 @@ class Reader(object):
 		def br():
 			return ET.Element("br")
 
-		with ET.htmlfile(f) as hf:
+		with ET.htmlfile(f, encoding="utf-8") as hf:
 			kebList = []  # type: List[str]
 			rebList = []  # type: List[str]
 			with hf.element("div"):
@@ -187,7 +195,7 @@ class Reader(object):
 				# this is for making internal links valid
 				# this makes too many alternates!
 				# but we don't seem to have a choice
-				# execpt for scanning and indexing all words once
+				# except for scanning and indexing all words once
 				# and then starting over and fixing/optimizing links
 				for keb in kebList:
 					for reb, _ in rebList:
@@ -223,9 +231,13 @@ class Reader(object):
 				)
 
 		defi = f.getvalue().decode("utf-8")
-		defi = unescape_unicode(defi)
 		byteProgress = (self._file.tell(), self._fileSize)
-		return self._glos.newEntry(keywords, defi, defiFormat="h", byteProgress=byteProgress)
+		return self._glos.newEntry(
+			keywords,
+			defi,
+			defiFormat="h",
+			byteProgress=byteProgress,
+		)
 
 	def tostring(self, elem: "lxml.etree.Element") -> str:
 		from lxml import etree as ET
@@ -282,14 +294,14 @@ class Reader(object):
 		# also good: f"https://sakuradict.com/search?q={{word}}"
 
 		header = ""
-		with open(filename, mode="rt", encoding="utf-8") as _file:
+		with compressionOpen(filename, mode="rt", encoding="utf-8") as _file:
 			for line in _file:
 				if "<JMdict>" in line:
 					break
 				header += line
 		self.setMetadata(header)
 
-		self._file = open(filename, mode="rb")
+		self._file = compressionOpen(filename, mode="rb")
 
 	def __iter__(self) -> "Iterator[BaseEntry]":
 		from lxml import etree as ET
