@@ -1,32 +1,29 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from pyglossary.text_reader import TextGlossaryReader
+from pyglossary.text_utils import splitByBar
 from pyglossary.file_utils import fileCountLines
 
 enable = True
+lname = "lingoes_ldf"
 format = "LingoesLDF"
-description = "Lingoes Source (LDF)"
+description = "Lingoes Source (.ldf)"
 extensions = (".ldf",)
+extensionCreate = ".ldf"
 singleFile = True
+kind = "text"
+wiki = "https://en.wikipedia.org/wiki/Lingoes"
+website = (
+	"http://www.lingoes.net/en/dictionary/dict_format.php",
+	"Lingoes.net",
+)
 optionsProp = {
 	"newline": NewlineOption(),
-	"resources": BoolOption(),
+	"resources": BoolOption(comment="Enable resources / data files"),
 	"encoding": EncodingOption(),
 }
 
-tools = [
-	{
-		"name": "Lingoes Dictionary Creator",
-		"web": "http://www.lingoes.net/en/dictionary/dict_format.php",
-		"platforms": [],
-		"comment": """Lingoes Dictionary Creator is developing now.
-Please send your finished dictionary source file to kevin-yau@msn.com
-Lingoes will compile it into .ld2 for you.
-You will can do it yourself after the creator release."""
-	},
-]
-
 
 class Reader(TextGlossaryReader):
 	compressions = stdCompressions
@@ -57,7 +54,7 @@ class Reader(TextGlossaryReader):
 			raise StopIteration
 		entryLines = []
 		while True:
-			line = self._file.readline()
+			line = self.readline()
 			if not line:
 				raise StopIteration
 			line = line.rstrip("\n\r")  # FIXME
@@ -83,7 +80,7 @@ class Reader(TextGlossaryReader):
 			defi = "\n".join(entryLines[1:])
 			defi = defi.replace("<br/>", "\n")  # FIXME
 
-			word = [p.strip() for p in word.split("|")]
+			word = splitByBar(word)
 
 			return word, defi
 
@@ -102,7 +99,7 @@ class Writer(object):
 		return self._glos.getInfo(key).replace("\n", "<br>")
 
 	def getAuthor(self):
-		return self._glos.getAuthor().replace("\n", "<br>")
+		return self._glos.author.replace("\n", "<br>")
 
 	def finish(self):
 		self._filename = None
@@ -111,6 +108,7 @@ class Writer(object):
 		self._filename = filename
 
 	def write(self) -> "Generator[None, BaseEntry, None]":
+		from pyglossary.text_writer import writeTxt
 		newline = self._newline
 		resources = self._resources
 		head = (
@@ -121,9 +119,10 @@ class Writer(object):
 			f"###Website: {self.getInfo('website')}\n"
 			f"###Copyright: {self.getInfo('copyright')}\n"
 		)
-		yield from self._glos.writeTxt(
+		yield from writeTxt(
+			self._glos,
 			entryFmt="{word}\n{defi}\n\n",
-			filename=self._filename ,
+			filename=self._filename,
 			writeInfo=False,
 			defiEscapeFunc=replaceStringTable([
 				("\n", "<br/>"),
