@@ -2,31 +2,30 @@
 # Source Glossary for "Sdictionary" (http://sdict.org)
 # It has extension ".sdct"
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 
 enable = True
+lname = "sdict_source"
 format = "SdictSource"
-description = "Sdictionary Source (sdct)"
+description = "Sdictionary Source (.sdct)"
 extensions = (".sdct",)
+extensionCreate = ".sdct"
 singleFile = True
+kind = "text"
+wiki = ""
+website = (
+	"http://swaj.net/sdict/",
+	"Sdictionary Project",
+)
 optionsProp = {
-	"writeInfo": BoolOption(),
+	"enable_info": BoolOption(comment="Enable glossary info / metedata"),
 	"newline": NewlineOption(),
-	"resources": BoolOption(),
+	"resources": BoolOption(comment="Enable resources / data files"),
 }
 
-tools = [
-	{
-		"name": "PTkSdict",
-		"web": "http://swaj.net/sdict/create-dicts.html",
-		"platforms": ["Linux", "Windows", "Mac"],
-		"license": "GPL",
-	},
-]
-
 
 class Writer(object):
-	_writeInfo: bool = True
+	_enable_info: bool = True
 	_newline: bool = "\n"
 	_resources: bool = True
 
@@ -44,9 +43,10 @@ class Writer(object):
 		self._filename = filename
 
 	def write(self) -> "Generator[None, BaseEntry, None]":
+		from pyglossary.text_writer import writeTxt
 		glos = self._glos
 		head = ""
-		if self._writeInfo:
+		if self._enable_info:
 			head = (
 				"<header>\n"
 				f"title = {self.getInfo('name')}\n"
@@ -56,7 +56,8 @@ class Writer(object):
 				f"a_lang = {glos.targetLangName}\n"
 				"</header>\n#\n#\n#\n"
 			)
-		yield from glos.writeTxt(
+		yield from writeTxt(
+			glos,
 			entryFmt="{word}___{defi}\n",
 			filename=self._filename,
 			writeInfo=False,
