@@ -11,81 +11,69 @@ import re
 import gzip
 from time import time as now
 from collections import Counter
+from operator import itemgetter
 
 from pyglossary.text_utils import (
 	uint32ToBytes,
 	uint32FromBytes,
 )
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 
 enable = True
+lname = "stardict"
 format = "Stardict"
-description = "StarDict (ifo)"
+description = "StarDict (.ifo)"
 extensions = (".ifo",)
+extensionCreate = "-stardict/"
+
+sortOnWrite = ALWAYS
+sortKeyName = "stardict"
+sortEncoding = "utf-8"
+
+kind = "directory"
+wiki = "https://en.wikipedia.org/wiki/StarDict"
+website = (
+	"http://huzheng.org/stardict/",
+	"huzheng.org/stardict",
+)
 optionsProp = {
-	"stardict_client": BoolOption(),
-	"dictzip": BoolOption(),
+	"stardict_client": BoolOption(
+		comment="Modify html entries for StarDict 3.0",
+	),
+	"dictzip": BoolOption(
+		comment="Compress .dict file to .dict.dz",
+	),
 	"sametypesequence": StrOption(
 		values=["", "h", "m", "x", None],
+		comment="Definition format: h=html, m=plaintext, x=xdxf",
+	),
+	"merge_syns": BoolOption(
+		comment="Write alternates to .idx instead of .syn",
+	),
+	"xdxf_to_html": BoolOption(
+		comment="Convert XDXF entries to HTML",
+	),
+	"unicode_errors": StrOption(
+		values=[
+			"strict",  # raise a UnicodeDecodeError exception
+			"ignore",  # just leave the character out
+			"replace",  # use U+FFFD, REPLACEMENT CHARACTER
+			"backslashreplace",  # insert a \xNN escape sequence
+		],
+		comment="What to do with Unicode decoding errors",
+	),
+	"audio_goldendict": BoolOption(
+		comment="Convert audio links for GoldenDict (desktop)"
+	),
+	"audio_icon": BoolOption(
+		comment="Add glossary's audio icon",
 	),
-	"merge_syns": BoolOption(),
-	"xdxf_to_html": BoolOption(),
 }
-sortOnWrite = ALWAYS
-# sortKey is defined in Writer class
 
 if os.getenv("PYGLOSSARY_STARDICT_NO_FORCE_SORT") == "1":
 	sortOnWrite = DEFAULT_YES
 
-tools = [
-	{
-		"name": "StarDict",
-		"web": "http://www.huzheng.org/stardict/",
-		"platforms": ["Linux", "Windows", "Mac"],
-		"license": "GPL",
-	},
-	{
-		"name": "GoldenDict",
-		"web": "http://goldendict.org/",
-		"platforms": ["Linux", "Windows"],
-		"license": "GPL",
-	},
-	{
-		"name": "GoldenDict Mobile",
-		"web": "http://goldendict.mobi/",
-		"platforms": ["Android"],
-		"license": "Unknown",
-	},
-	{
-		"name": "Twinkle Star Dictionary",
-		"web": "https://play.google.com/store/apps/details?id=com.qtier.dict",
-		"platforms": ["Android"],
-		"license": "Unknown",
-		# last release: 2015/10/19
-		# could not find the source code, license or website
-	},
-	{
-		"name": "WordMateX",
-		"web": "https://apkcombo.com/wordmatex/org.d1scw0rld.wordmatex/",
-		"platforms": ["Android"],
-		"license": "Proprietary",
-		# last release: 2020/01/01, version 2.1.1
-		# Google Play says "not compatible with your devices", no letting me
-		# download and install, so I downloaded apk from apkcombo.com
-		# This is the only Android app (not just for StarDict format) I found
-		# that supports auto-RTL
-	},
-	{
-		"name": "QDict",
-		"web": "https://github.com/namndev/QDict",
-		"platforms": ["Android"],
-		"license": "Apache 2.0",
-		# last release: 2017/04/16 (keeps crashing on my device, unusable)
-		# last commit: 2020/06/24
-	},
-]
-
 infoKeys = (
 	"bookname",
 	"author",
@@ -119,15 +107,14 @@ def verifySameTypeSequence(s: str) -> bool:
 
 class Reader(object):
 	_xdxf_to_html = True
+	_unicode_errors = "strict"
 
 	def __init__(self, glos: GlossaryType):
-		from pyglossary.xdxf_transform import xdxf_to_html_transformer
+
 		self._glos = glos
 		self.clear()
 
-		self._xdxf_tr = None
-		if self._xdxf_to_html:
-			self._xdxf_tr = xdxf_to_html_transformer()
+		self._xdxfTr = None
 
 		"""
 		indexData format
@@ -146,6 +133,15 @@ class Reader(object):
 			a dict { entryIndex -> altList }
 		"""
 
+	def xdxf_setup(self):
+		from pyglossary.xdxf_transform import XdxfTransformer
+		self._xdxfTr = XdxfTransformer(encoding="utf-8")
+
+	def xdxf_transform(self, text: str):
+		if self._xdxfTr is None:
+			self.xdxf_setup()
+		return self._xdxfTr.transformByInnerString(text)
+
 	def close(self) -> None:
 		if self._dictFile:
 			self._dictFile.close()
@@ -245,14 +241,14 @@ class Reader(object):
 		synDict = self._synDict
 		sametypesequence = self._sametypesequence
 		dictFile = self._dictFile
+		unicode_errors = self._unicode_errors
 
 		if not dictFile:
-			log.error(f"{self} is not open, can not iterate")
-			raise StopIteration
+			raise RuntimeError("iterating over a reader while it's not open")
 
 		if not indexData:
 			log.warning("indexData is empty")
-			raise StopIteration
+			return
 
 		for entryIndex, (b_word, defiOffset, defiSize) in enumerate(indexData):
 			if not b_word:
@@ -286,7 +282,7 @@ class Reader(object):
 			defis = []
 			defiFormats = []
 			for b_partDefi, defiFormatCode in defisData:
-				partDefi = b_partDefi.decode("utf-8")
+				partDefi = b_partDefi.decode("utf-8", errors=unicode_errors)
 				partDefiFormat = {
 					"m": "m",
 					"t": "m",
@@ -295,8 +291,8 @@ class Reader(object):
 					"h": "h",
 					"x": "x",
 				}.get(chr(defiFormatCode), "")
-				if partDefiFormat == "x" and self._xdxf_tr:
-					partDefi = self._xdxf_tr(partDefi)
+				if partDefiFormat == "x" and self._xdxf_to_html:
+					partDefi = self.xdxf_transform(partDefi)
 					partDefiFormat = "h"
 				defis.append(partDefi)
 				defiFormats.append(partDefiFormat)
@@ -308,7 +304,7 @@ class Reader(object):
 			if not defiFormat:
 				log.warning(f"Definition format {defiFormat!r} is not supported")
 
-			word = b_word.decode("utf-8")
+			word = b_word.decode("utf-8", errors=unicode_errors)
 			try:
 				alts = synDict[entryIndex]
 			except KeyError:  # synDict is dict
@@ -318,7 +314,7 @@ class Reader(object):
 
 			defiSep = "\n<hr>\n"
 			# if defiFormat == "x"
-			#	defiSep = FIXME
+			# 	defiSep = FIXME
 			defi = defiSep.join(defis)
 
 			# FIXME:
@@ -328,10 +324,10 @@ class Reader(object):
 		if isdir(self._resDir):
 			for fname in os.listdir(self._resDir):
 				fpath = join(self._resDir, fname)
-				with open(fpath, "rb") as fromFile:
+				with open(fpath, "rb") as _file:
 					yield self._glos.newDataEntry(
 						fname,
-						fromFile.read(),
+						_file.read(),
 					)
 
 	def readSynFile(self) -> "Dict[int, List[str]]":
@@ -340,6 +336,8 @@ class Reader(object):
 		"""
 		if not isfile(self._filename + ".syn"):
 			return {}
+		unicode_errors = self._unicode_errors
+
 		with open(self._filename + ".syn", "rb") as synFile:
 			synBytes = synFile.read()
 		synBytesLen = len(synBytes)
@@ -365,7 +363,8 @@ class Reader(object):
 				)
 				continue
 
-			s_alt = b_alt.decode("utf-8")  # s_alt is str
+			s_alt = b_alt.decode("utf-8", errors=unicode_errors)
+			# s_alt is str
 			try:
 				synDict[entryIndex].append(s_alt)
 			except KeyError:
@@ -458,12 +457,12 @@ class Reader(object):
 		return res
 
 	# def readResources(self):
-	#	if not isdir(self._resDir):
-	#		resInfoPath = join(baseDirPath, "res.rifo")
-	#		if isfile(resInfoPath):
-	#			log.warning(
-	#				"StarDict resource database is not supported. Skipping"
-	#			)
+	# 	if not isdir(self._resDir):
+	# 		resInfoPath = join(baseDirPath, "res.rifo")
+	# 		if isfile(resInfoPath):
+	# 			log.warning(
+	# 				"StarDict resource database is not supported. Skipping"
+	# 			)
 
 
 class Writer(object):
@@ -471,6 +470,8 @@ class Writer(object):
 	_sametypesequence: str = ""  # type: Literal["", "h", "m", "x", None]
 	_stardict_client: bool = False
 	_merge_syns: bool = False
+	_audio_goldendict: bool = False
+	_audio_icon: bool = True
 
 	def __init__(self, glos: GlossaryType):
 		self._glos = glos
@@ -486,9 +487,11 @@ class Writer(object):
 			"<br[ /]*>",
 			re.IGNORECASE,
 		)
+		self._re_audio_link = re.compile(
+			'<a (type="sound" )?([^<>]*? )?href="sound://([^<>"]+)"( .*?)?>(.*?)</a>'
+		)
 
-	def sortKey(self, b_word: bytes) -> "Tuple[bytes, bytes]":
-		assert isinstance(b_word, bytes)
+	def byteSortKey(self, b_word: bytes) -> "Tuple[bytes, bytes]":
 		return (
 			b_word.lower(),
 			b_word,
@@ -501,7 +504,7 @@ class Writer(object):
 		self._targetLang = None
 
 	def open(self, filename: str) -> None:
-		log.info(f"open: filename = {filename}")
+		log.debug(f"open: filename = {filename}")
 		fileBasePath = filename
 		##
 		if splitext(filename)[1].lower() == ".ifo":
@@ -528,7 +531,7 @@ class Writer(object):
 			log.debug(f"Using write option sametypesequence={self._sametypesequence}")
 		elif self._sametypesequence is not None:
 			stat = self._glos.collectDefiFormat(100)
-			log.info(f"defiFormat stat: {stat}")
+			log.debug(f"defiFormat stat: {stat}")
 			if stat:
 				if stat["m"] > 0.97:
 					log.info(f"Auto-selecting sametypesequence=m")
@@ -560,6 +563,18 @@ class Writer(object):
 			defi = defi.replace("</p>", "<br>")
 			defi = self._br_pattern.sub("<br>", defi)
 
+		if self._audio_goldendict:
+			if self._audio_icon:
+				defi = self._re_audio_link.sub(
+					r'<audio src="\3">\5</audio>',
+					defi,
+				)
+			else:
+				defi = self._re_audio_link.sub(
+					r'<audio src="\3"></audio>',
+					defi,
+				)
+
 		# FIXME:
 		# defi = defi.replace(' src="./', ' src="./res/')
 		return defi
@@ -709,7 +724,7 @@ class Writer(object):
 		t0 = now()
 
 		altIndexList.sort(
-			key=lambda x: self.sortKey(x[0])
+			key=lambda x: self.byteSortKey(x[0])
 		)
 		# 28 seconds with old sort key (converted from custom cmp)
 		# 0.63 seconds with my new sort key
@@ -860,7 +875,7 @@ class Writer(object):
 		log.info(f"Sorting {len(indexList)} items...")
 		t0 = now()
 
-		indexList.sort(key=lambda x: self.sortKey(x[0]))
+		indexList.sort(key=lambda x: self.byteSortKey(x[0]))
 		log.info(
 			f"Sorting {len(indexList)} {filename} took {now()-t0:.2f} seconds",
 		)
