@@ -1,11 +1,16 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 
 enable = False
+lname = "testformat"
 format = "Test"
 description = "Test Format File(.test)"
 extensions = (".test", ".tst")
+extensionCreate = ".test"
+kind = "text"
+wiki = ""
+website = None
 
 # key is option/argument name, value is instance of Option
 optionsProp = {}
@@ -30,7 +35,7 @@ class Reader(object):
 		# and set self._wordCount if you can
 		# read-options should be keyword arguments in this method
 		self._wordCount = 100
-		log.info(f"reading from format {format} using plugin")
+		# log.info(f"some useful message")
 		# here read info from file and set to Glossary object
 		self._glos.setInfo("name", "Test")
 		desc = "Test glossary craeted by a PyGlossary plugin"
@@ -71,7 +76,7 @@ class Writer(object):
 	def write(self) -> "Generator[None, BaseEntry, None]":
 		glos = self._glos
 		filename = self._filename
-		log.info(f"writing to format {format} using plugin")
+		# log.info(f"some useful message")
 		while True:
 			entry = yield
 			if entry is None:
@@ -83,7 +88,7 @@ class Writer(object):
 		# here read info from Glossaey object
 		name = glos.getInfo("name")
 		desc = glos.getInfo("description")
-		author = glos.getAuthor()
+		author = glos.author
 		copyright = glos.getInfo("copyright")
 		# if an info key doesn't exist, getInfo returns empty string
 		# now write info to the output file (depending on your output format)
