@@ -4,12 +4,20 @@ from time import time as now
 import re
 import html
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 
 enable = True
+lname = "wiktionary_dump"
 format = "WiktionaryDump"
 description = "Wiktionary Dump (.xml)"
-extensions = (".xml",)
+extensions = ()
+extensionCreate = ""
+kind = "text"
+wiki = "https://en.wiktionary.org/wiki/Wiktionary:Main_Page"
+website = (
+	"https://dumps.wikimedia.org/mirrors.html",
+	"dumps.wikimedia.org",
+)
 optionsProp = {
 	"encoding": EncodingOption(),
 }
@@ -98,11 +106,7 @@ class Reader(object):
 	def __iter__(self) -> "Iterator[BaseEntry]":
 		from lxml import etree as ET
 		if not self._filename:
-			log.error(
-				"WikipediaDump: trying to iterate over reader"
-				" while it's not open"
-			)
-			raise StopIteration
+			raise RuntimeError("iterating over a reader while it's not open")
 		while True:
 			page = self._readPage()
 			if page is None:
@@ -116,6 +120,7 @@ class Reader(object):
 	def compilePatterns(self):
 		self._re_comment = re.compile(
 			"<!--.*?-->",
+			re.MULTILINE | re.DOTALL,
 		)
 		self._re_internal_link = re.compile(
 			r"\[\[(.+?)\]\]",
@@ -165,10 +170,10 @@ class Reader(object):
 			r"^([^<\s].+?[^>\s])$",
 			re.MULTILINE,
 		)
-		#self._re_emptyCircledLines = re.compile(
-		#	r"^\s*⚫︎\s*$",
-		#	re.MULTILINE | re.UNICODE,
-		#)
+		# self._re_emptyCircledLines = re.compile(
+		# 	r"^\s*⚫︎\s*$",
+		# 	re.MULTILINE | re.UNICODE,
+		# )
 
 	def fixText(self, text: str) -> str:
 		text = self._re_comment.sub("", text)
