@@ -1,16 +1,25 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 from pyglossary.entry import Entry
+from pyglossary.sort_keys import namedSortKeyByName
 from os import listdir
 from os.path import isfile
 from json import load
 
 enable = True
+lname = "wordset"
 format = "Wordset"
 description = "Wordset.org JSON directory"
 extensions = ()
+extensionCreate = "-wordset/"
 singleFile = False
+kind = "directory"
+wiki = ""
+website = (
+	"https://github.com/wordset/wordset-dictionary",
+	"@wordset/wordset-dictionary",
+)
 optionsProp = {
 	"encoding": EncodingOption(),
 }
@@ -32,15 +41,14 @@ class Reader(object):
 			"</p>"
 		)
 		"""
-        {
-            "id": "492099d426",
-            "def": "without musical accompaniment",
-            "example": "they performed a cappella",
-            "speech_part": "adverb"
-        },
+		{
+			"id": "492099d426",
+			"def": "without musical accompaniment",
+			"example": "they performed a cappella",
+			"speech_part": "adverb"
+		},
 		"""
 
-
 	def close(self) -> None:
 		self._clear()
 
@@ -65,20 +73,20 @@ class Reader(object):
 
 	def __iter__(self) -> "Iterator[BaseEntry]":
 		if not self._filename:
-			log.error("iterating over a reader which is not open")
-			raise StopIteration
+			raise RuntimeError("iterating over a reader while it's not open")
 
 		direc = self._filename
 		encoding = self._encoding
 		glos = self._glos
 		for fname in sorted(listdir(direc), key=self.fileNameSortKey):
 			fpath = join(direc, fname)
-			if not (fname.endswith(".json") and isfile(fpath)) :
+			if not (fname.endswith(".json") and isfile(fpath)):
 				continue
 			with open(fpath, encoding=encoding) as fileObj:
 				data = load(fileObj)
 				words = list(data.keys())
-				words.sort(key=Entry.defaultStringSortKey)
+				sortKey = namedSortKeyByName["headword_lower"].normal("utf-8")
+				words.sort(key=sortKey)
 				for word in words:
 					entryDict = data[word]
 					defi = "".join(
@@ -92,4 +100,3 @@ class Reader(object):
 					)
 					yield glos.newEntry(word, defi, defiFormat="h")
 			log.info(f"finished reading {fname}")
-
