@@ -22,42 +22,26 @@
 
 from os import path
 
-from formats_common import *
-from pyglossary.xdxf_transform import xdxf_to_html_transformer
+from pyglossary.plugins.formats_common import *
+from pyglossary.xdxf_transform import *
 
 enable = True
+lname = "xdxf"
 format = "Xdxf"
-description = "XDXF"
+description = "XDXF (.xdxf)"
 extensions = (".xdxf",)
+extensionCreate = ".xdxf"
 singleFile = True
+kind = "text"
+wiki = "https://en.wikipedia.org/wiki/XDXF"
+website = (
+	"https://github.com/soshial/xdxf_makedict/blob/master/format_standard/xdxf_description.md",
+	"xdxf_description.md - @soshial/xdxf_makedict",
+)
 optionsProp = {
-	"html": BoolOption(),
+	"html": BoolOption(comment="Entries are HTML"),
 }
 
-# https://en.wikipedia.org/wiki/XDXF
-
-tools = [
-	{
-		"name": "GoldenDict",
-		"web": "http://goldendict.org/",
-		"platforms": ["Linux", "Windows"],
-		"license": "GPL",
-	},
-	{
-		"name": "QTranslate",
-		"web": "https://quest-app.appspot.com/",
-		"platforms": ["Windows"],
-		"license": "Proprietary",
-	},
-	{
-		"name": "Alpus",
-		"web": "https://alpusapp.com/",
-		"platforms": ["Windows", "Mac", "Linux", "Android"],
-		# Java-based
-		"license": "Freeware",
-	},
-]
-
 
 """
 new format
@@ -107,35 +91,42 @@ class Reader(object):
 		self._filename = ""
 		self._file = None
 		self._encoding = "utf-8"
-		self._xdxf_to_html = None
+		self._htmlTr = None
 		self._re_span_k = re.compile(
 			'<span class="k">[^<>]*</span>(<br/>)?',
 		)
 
-	def open(self, filename: str, html: bool = True):
+	def open(self, filename: str):
 		# <!DOCTYPE xdxf SYSTEM "http://xdxf.sourceforge.net/xdxf_lousy.dtd">
 		from lxml import etree as ET
 		self._filename = filename
 		if self._html:
-			self._xdxf_to_html = xdxf_to_html_transformer()
+			self._htmlTr = XdxfTransformer(encoding=self._encoding)
+			self._glos.setDefaultDefiFormat("h")
+		else:
+			self._glos.setDefaultDefiFormat("x")
+
+		_file = open(self._filename, mode="rb")
 		context = ET.iterparse(
-			filename,
+			_file,
 			events=("end",),
 		)
 		for action, elem in context:
 			if elem.tag in ("meta_info", "ar", "k", "abr", "dtrn"):
 				break
 			# every other tag before </meta_info> or </ar> is considered info
+			if elem.tag in ("abbr_def",):
+				continue
 			if not elem.text:
-				log.warn(f"empty tag <{elem.tag}>")
+				log.warning(f"empty tag <{elem.tag}>")
 				continue
 			key = self.infoKeyMap.get(elem.tag, elem.tag)
 			self._glos.setInfo(key, elem.text)
 
+		_file.close()
 		del context
 		self._fileSize = os.path.getsize(filename)
 		self._file = open(self._filename, mode="rb")
-		self._glos.setDefaultDefiFormat("x")
 		self._glos.setInfo("input_file_size", f"{self._fileSize}")
 
 	def __len__(self):
@@ -153,15 +144,15 @@ class Reader(object):
 		for action, article in context:
 			article.tail = None
 			words = [toStr(w) for w in self.titles(article)]
-			defi = tostring(article, encoding=self._encoding)
-			# <ar>...</ar>
-			defi = defi[4:-5].decode(self._encoding).strip()
-			defiFormat = "x"
-			if self._xdxf_to_html:
-				defi = self._xdxf_to_html(defi)
+			if self._htmlTr:
+				defi = self._htmlTr.transform(article)
 				defiFormat = "h"
 				if len(words) == 1:
 					defi = self._re_span_k.sub("", defi)
+			else:
+				defi = tostring(article, encoding=self._encoding)
+				defi = defi[4:-5].decode(self._encoding).strip()
+				defiFormat = "x"
 
 			# log.info(f"defi={defi}, words={words}")
 			yield self._glos.newEntry(
@@ -203,6 +194,14 @@ class Reader(object):
 		if desc:
 			self._glos.setInfo("description", desc)
 
+	def tostring(self, elem: "lxml.etree.Element") -> str:
+		from lxml import etree as ET
+		return ET.tostring(
+			elem,
+			method="html",
+			pretty_print=True,
+		).decode("utf-8").strip()
+
 	def titles(self, article):
 		"""
 
@@ -212,6 +211,10 @@ class Reader(object):
 		from itertools import combinations
 		titles = []
 		for title_element in article.findall("k"):
+			if title_element.text is None:
+				# TODO: look for <opt> tag?
+				log.warning(f"empty title element: {self.tostring(title_element)}")
+				continue
 			n_opts = len([c for c in title_element if c.tag == "opt"])
 			if n_opts:
 				for j in range(n_opts + 1):
