@@ -1,54 +1,32 @@
 # -*- coding: utf-8 -*-
 
-from formats_common import *
+from pyglossary.plugins.formats_common import *
 
 enable = True
+lname = "zim"
 format = "Zim"
-description = "Zim (Kiwix)"
+description = "Zim (.zim, for Kiwix)"
 extensions = (".zim",)
+extensionCreate = ".zim"
 singleFile = True
+kind = "binary"
+wiki = "https://en.wikipedia.org/wiki/ZIM_(file_format)"
+website = (
+	"https://wiki.openzim.org/wiki/OpenZIM",
+	"OpenZIM",
+)
 optionsProp = {
-	"skip_duplicate_words": BoolOption(),
+	"skip_duplicate_words": BoolOption(
+		comment="Detect and skip duplicate words",
+	),
 }
 
 # https://wiki.kiwix.org/wiki/Software
-tools = [
-	{
-		"name": "Kiwix Desktop",
-		"web": "https://github.com/kiwix/kiwix-desktop",
-		"platforms": ["Linux", "Windows"],
-		"license": "GPL",
-	},
-	{
-		"name": "Kiwix JS",
-		"web": "https://github.com/kiwix/kiwix-js",
-		"platforms": ["Windows"],
-		"license": "GPL",
-	},
-	{
-		"name": "Kiwix Serve",
-		"web": "https://github.com/kiwix/kiwix-tools",
-		"platforms": ["Linux", "Windows"],
-		"license": "GPL",
-	},
-	{
-		"name": "Kiwix for Apple Mac OS X",
-		"web": "macos.kiwix.org",
-		"platforms": ["Mac"],
-		"license": "",
-	},
-	{
-		"name": "Kiwix for Android",
-		"web": "https://github.com/kiwix/kiwix-android",
-		"platforms": ["Android"],
-		"license": "GPL",
-	},
-]
 
 
 class Reader(object):
 	depends = {
-		"libzim": "libzim",
+		"libzim": "libzim==1.0",
 	}
 
 	_skip_duplicate_words = False
@@ -58,9 +36,19 @@ class Reader(object):
 		"image/jpeg",
 		"image/gif",
 		"image/svg+xml",
+		"image/webp",
+		"image/x-icon",
+
 		"text/css",
+		"text/javascript",
+
 		"application/javascript",
+		"application/json",
+		"application/octet-stream",
 		"application/octet-stream+xapian",
+		"application/x-chrome-extension",
+		"application/warc-headers",
+		"application/font-woff",
 	}
 
 	def __init__(self, glos: GlossaryType) -> None:
@@ -70,13 +58,13 @@ class Reader(object):
 
 	def open(self, filename: str) -> None:
 		try:
-			from libzim.reader import File
+			from libzim.reader import Archive
 		except ModuleNotFoundError as e:
 			e.msg += f", run `{pip} install libzim` to install"
 			raise e
 
 		self._filename = filename
-		self._zimfile = File(filename)
+		self._zimfile = Archive(filename)
 
 	def close(self) -> None:
 		self._filename = None
@@ -86,27 +74,31 @@ class Reader(object):
 		if self._zimfile is None:
 			log.error(f"len(reader) called before reader.open()")
 			return 0
-		return self._zimfile.article_count
+		return self._zimfile.entry_count
 
 	def __iter__(self):
 		glos = self._glos
 		zimfile = self._zimfile
 		emptyContentCount = 0
 		invalidMimeTypeCount = 0
-		articleCount = zimfile.article_count
+		entryCount = zimfile.entry_count
 
-		duplicateArticleCount = 0
+		duplicateEntryCount = 0
 		redirectCount = 0
 		skip_dup = self._skip_duplicate_words
 		hashSet = set()
 
-		for articleIndex in range(articleCount):
-			ar = zimfile.get_article_by_id(articleIndex)
-			word = ar.title
+		f_namemax = os.statvfs(cacheDir).f_namemax
 
-			if ar.is_redirect:
+		fileNameTooLong = []
+
+		for entryIndex in range(entryCount):
+			zEntry = zimfile._get_entry_by_id(entryIndex)
+			word = zEntry.title
+
+			if zEntry.is_redirect:
 				redirectCount += 1
-				targetWord = ar.get_redirect_article().title
+				targetWord = zEntry.get_redirect_entry().title
 				yield glos.newEntry(
 					word,
 					f'Redirect: <a href="bword://{targetWord}">{targetWord}</a>',
@@ -114,31 +106,37 @@ class Reader(object):
 				)
 				continue
 
-			b_content = ar.content.tobytes()
+			zItem = zEntry.get_item()
+			b_content = zItem.content.tobytes()
 
 			if skip_dup:
 				if word in hashSet:
-					duplicateArticleCount += 1
+					duplicateEntryCount += 1
 					yield None
 					continue
 				hashSet.add(word)
 
 			if not b_content:
 				emptyContentCount += 1
-				if ar.url == word:
-					yield None
-				else:
-					defi = f"URL: {ar.url}"
-					yield glos.newEntry(word, defi, defiFormat="m")
+				yield None
+				# TODO: test with more zim files
+				# Looks like: zItem.path == zEntry.path == "-" + word
+				# print(f"b_content empty, word={word!r}, zEntry.path={zEntry.path!r}, zItem.path={zItem.path}")
+				# if zEntry.path == "-" + word:
+				# 	yield None
+				# else:
+				# 	defi = f"Path: {zEntry.path}"
+				# 	yield glos.newEntry(word, defi, defiFormat="m")
 				continue
 
 			try:
-				mimetype = ar.mimetype
+				mimetype = zItem.mimetype
 			except RuntimeError:
 				invalidMimeTypeCount += 1
 				yield glos.newDataEntry(word, b_content)
 
-			if mimetype == "text/html":
+			if mimetype.startswith("text/html"):
+				# can be "text/html;raw=true"
 				defi = b_content.decode("utf-8")
 				defi = defi.replace(' src="../I/', ' src="./')
 				yield glos.newEntry(word, defi, defiFormat="h")
@@ -152,16 +150,24 @@ class Reader(object):
 				continue
 
 			if mimetype not in self.resourceMimeTypes:
-				log.warn(f"Unrecognized mimetype={mimetype!r}")
+				log.warning(f"Unrecognized mimetype={mimetype!r}")
+
+			if len(word) > f_namemax:
+				fileNameTooLong.append(word)
+				continue
 
 			if "|" in word:
 				log.error(f"resource title: {word}")
 
 			yield glos.newDataEntry(word, b_content)
 
-		log.info(f"Article Count: {articleCount}")
-		if duplicateArticleCount > 0:
-			log.info(f"Duplicate Title Count: {duplicateArticleCount}")
+		log.info(f"ZIM Entry Count: {entryCount}")
+
+		if len(fileNameTooLong) > 0:
+			log.error(f"Files with name too long: {len(fileNameTooLong)}")
+
+		if duplicateEntryCount > 0:
+			log.info(f"Duplicate Title Count: {duplicateEntryCount}")
 		if emptyContentCount > 0:
 			log.info(f"Empty Content Count: {emptyContentCount}")
 		if invalidMimeTypeCount > 0:
