@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2012-2020 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
+# Copyright © 2012-2022 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 # This file is part of PyGlossary project, https://github.com/ilius/pyglossary
 #
 # This program is a free software; you can redistribute it and/or modify
@@ -54,68 +54,119 @@ summary = "A tool for converting dictionary files aka glossaries with" \
 class UIBase(object):
 	configDefDict = OrderedDict([
 		("log_time", BoolOption(
-			cmd=True,
-			comment="show date and time in logs",
-			falseComment="do not show date and time in logs",
+			hasFlag=True,
+			comment="Show date and time in logs",
+			falseComment="Do not show date and time in logs",
 		)),
 		("cleanup", BoolOption(
-			cmd=True,
-			comment="cleanup cache or temporary files after convertion",
-			falseComment="do not cleanup cache or temporary files after convertion",
+			hasFlag=True,
+			comment="Cleanup cache or temporary files after conversion",
+			falseComment="Do not cleanup cache or temporary files after conversion",
+		)),
+
+		("auto_sqlite", BoolOption(
+			hasFlag=False,
+			comment=(
+				"Auto-enable --sqlite to limit RAM usage when direct\n"
+				"mode is not possible. Can override with --no-sqlite"
+			),
 		)),
 
 		("lower", BoolOption(
-			cmd=True,
-			comment="lowercase words before writing",
-			falseComment="do not lowercase words before writing",
+			hasFlag=True,
+			comment="Lowercase words before writing",
+			falseComment="Do not lowercase words before writing",
 		)),
 		("utf8_check", BoolOption(
-			cmd=True,
-			comment="ensure entries contain valid UTF-8 strings",
-			falseComment="do not ensure entries contain valid UTF-8 strings",
+			hasFlag=True,
+			comment="Ensure entries contain valid UTF-8 strings",
+			falseComment="Do not ensure entries contain valid UTF-8 strings",
 		)),
 		("enable_alts", BoolOption(
-			cmd=True,
-			cmdFlag="alts",
-			comment="",
-			falseComment="disable alternates",
+			hasFlag=True,
+			customFlag="alts",
+			comment="Enable alternates",
+			falseComment="Disable alternates",
 		)),
+		# FIXME: replace with "resources"
+		# 	comment="Use resources (images, audio, etc)"
 		("skip_resources", BoolOption(
-			cmd=True,
-			comment="skip resources (images, audio, etc)",
-			falseComment="",
+			hasFlag=True,
+			comment="Skip resources (images, audio, css, etc)",
 		)),
 
+		("rtl", BoolOption(
+			hasFlag=True,
+			comment=(
+				"Right-To-Left all (HTML) definitions"
+			),
+		)),
 		("remove_html", StrOption(
-			cmd=True,
-			comment="remove given html tags (comma-separated) from definitions",
+			hasFlag=True,
+			comment=(
+				"Remove given HTML tags (comma-separated)\n"
+				"from definitions"
+			),
 		)),
 		("remove_html_all", BoolOption(
-			cmd=True,
-			comment="remove all html tags from definitions",
-			falseComment="",
+			hasFlag=True,
+			comment="Remove all HTML tags from definitions",
 		)),
 		("normalize_html", BoolOption(
-			cmd=True,
-			comment="lowercase and normalize html tags in definitions",
+			hasFlag=True,
+			comment="Lowercase and normalize HTML tags in definitions",
 		)),
 		("save_info_json", BoolOption(
-			cmd=True,
-			cmdFlag="info",
-			comment="save glossary info as json file with .info extension",
-			falseComment="",
+			hasFlag=True,
+			customFlag="info",
+			comment="Save .info file alongside output file(s)",
 		)),
 
-		("ui_autoSetFormat", BoolOption(cmd=False)),
+		("color.enable.cmd.unix", BoolOption(
+			hasFlag=False,
+			comment="Enable colors in Linux/Unix command line"
+		)),
+		("color.enable.cmd.windows", BoolOption(
+			hasFlag=False,
+			comment="Enable colors in Windows command line"
+		)),
 
-		("reverse_matchWord", BoolOption(cmd=False)),
-		("reverse_showRel", StrOption(cmd=False)),
-		("reverse_saveStep", IntOption(cmd=False)),
-		("reverse_minRel", FloatOption(cmd=False)),
-		("reverse_maxNum", IntOption(cmd=False)),
-		("reverse_includeDefs", BoolOption(cmd=False)),
+		("color.cmd.critical", IntOption(
+			hasFlag=False,
+			comment="Color code for critical errors in command line",
+		)),
+		("color.cmd.error", IntOption(
+			hasFlag=False,
+			comment="Color code for errors in command line",
+		)),
+		("color.cmd.warning", IntOption(
+			hasFlag=False,
+			comment="Color code for warnings in command line",
+		)),
+
+
+		# interactive command line interface
+		("cmdi.prompt.indent.str", StrOption(hasFlag=False)),
+		("cmdi.prompt.indent.color", IntOption(hasFlag=False)),
+		("cmdi.prompt.msg.color", IntOption(hasFlag=False)),
+		("cmdi.msg.color", IntOption(hasFlag=False)),
+
+
+		("ui_autoSetFormat", BoolOption(hasFlag=False)),
+
+		("reverse_matchWord", BoolOption(hasFlag=False)),
+		("reverse_showRel", StrOption(hasFlag=False)),
+		("reverse_saveStep", IntOption(hasFlag=False)),
+		("reverse_minRel", FloatOption(hasFlag=False)),
+		("reverse_maxNum", IntOption(hasFlag=False)),
+		("reverse_includeDefs", BoolOption(hasFlag=False)),
 	])
 
+	conflictingParams = [
+		("sqlite", "direct"),
+		("remove_html", "remove_html_all"),
+	]
+
 	def __init__(self, **kwargs):
 		self.config = {}
 
@@ -128,10 +179,14 @@ class UIBase(object):
 	def progressEnd(self):
 		self.progress(1.0)
 
-	def loadConfig(self, **options):
+	def loadConfig(
+		self,
+		user: bool = True,
+		**options
+	):
 		from pyglossary.json_utils import jsonToData
 		data = jsonToData(fread(rootConfJsonFile))
-		if isfile(confJsonFile):
+		if user and isfile(confJsonFile):
 			try:
 				userData = jsonToData(fread(confJsonFile))
 			except Exception:
@@ -158,7 +213,7 @@ class UIBase(object):
 
 		log.setTimeEnable(self.config["log_time"])
 
-		log.debug("loaded config")
+		log.debug(f"loaded config: {self.config}")
 		return True
 
 	def saveConfig(self):
