@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # main.py
 #
-# Copyright © 2008-2020 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
+# Copyright © 2008-2022 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 # This file is part of PyGlossary project, https://github.com/ilius/pyglossary
 #
 # This program is a free software; you can redistribute it and/or modify
@@ -28,6 +28,8 @@ import logging
 from pyglossary import core  # essential
 from pyglossary.entry import Entry
 from pyglossary.ui.base import UIBase
+from pyglossary.langs import langDict
+from pyglossary.sort_keys import namedSortKeyList, namedSortKeyByName
 
 # the first thing to do is to set up logger.
 # other modules also using logger "root", so it is essential to set it up prior
@@ -52,6 +54,8 @@ from pyglossary.ui.base import UIBase
 # -v (verbose or version?)
 # -r (reverse or read-options)
 
+log = None
+
 
 def canRunGUI():
 	if core.sysName == "linux":
@@ -66,10 +70,52 @@ def canRunGUI():
 	return True
 
 
-def registerOption(parser, key: str, option: "Option"):
-	if not option.cmd:
+class StoreConstAction(argparse.Action):
+	def __init__(
+		self,
+		option_strings,
+		same_dest="",
+		const_value=None,
+		nargs=0,
+		**kwargs
+	):
+		if isinstance(option_strings, str):
+			option_strings = [option_strings]
+		argparse.Action.__init__(
+			self,
+			option_strings=option_strings,
+			nargs=nargs,
+			**kwargs
+		)
+		self.same_dest = same_dest
+		self.const_value = const_value
+
+	def __call__(
+		self,
+		parser=None,
+		namespace=None,
+		values=None,
+		option_strings=None,
+		required=False,
+		dest=None,
+	):
+		if not parser:
+			return self
+		dest = self.dest
+		if getattr(namespace, dest) is not None:
+			flag = self.option_strings[0]
+			if getattr(namespace, dest) == self.const_value:
+				parser.error(f"multiple {flag} options")
+			else:
+				parser.error(f"conflicting options: {self.same_dest} and {flag}")
+		setattr(namespace, dest, self.const_value)
+		return self
+
+
+def registerConfigOption(parser, key: str, option: "Option"):
+	if not option.hasFlag:
 		return
-	flag = option.cmdFlag
+	flag = option.customFlag
 	if not flag:
 		flag = key.replace('_', '-')
 
@@ -82,7 +128,11 @@ def registerOption(parser, key: str, option: "Option"):
 		)
 		return
 
-	if option.comment:
+	if not option.comment:
+		print(f"registerConfigOption: option has no comment: {option}")
+		return
+
+	if not option.falseComment:
 		parser.add_argument(
 			f"--{flag}",
 			dest=key,
@@ -90,19 +140,124 @@ def registerOption(parser, key: str, option: "Option"):
 			default=None,
 			help=option.comment,
 		)
+		return
 
-	if option.falseComment:
-		parser.add_argument(
+	parser.add_argument(
+		dest=key,
+		action=StoreConstAction(
+			f"--{flag}",
+			same_dest=f"--no-{flag}",
+			const_value=True,
+			dest=key,
+			default=None,
+			help=option.comment,
+		),
+	)
+	parser.add_argument(
+		dest=key,
+		action=StoreConstAction(
 			f"--no-{flag}",
+			same_dest=f"--{flag}",
+			const_value=False,
 			dest=key,
-			action="store_false",
 			default=None,
 			help=option.falseComment,
-		)
+		),
+	)
+
+
+def base_ui_run(
+	inputFilename: str = "",
+	outputFilename: str = "",
+	inputFormat: str = "",
+	outputFormat: str = "",
+	reverse: bool = False,
+	config: "Optional[Dict]" = None,
+	readOptions: "Optional[Dict]" = None,
+	writeOptions: "Optional[Dict]" = None,
+	convertOptions: "Optional[Dict]" = None,
+	glossarySetAttrs: "Optional[Dict]" = None,
+):
+	from pyglossary.glossary import Glossary
+	if reverse:
+		log.error("--reverse does not work with --ui=none")
+		return False
+	ui = UIBase()
+	ui.loadConfig(**config)
+	glos = Glossary(ui=ui)
+	glos.config = ui.config
+	if glossarySetAttrs:
+		for attr, value in glossarySetAttrs.items():
+			setattr(glos, attr, value)
+	glos.convert(
+		inputFilename=inputFilename,
+		outputFilename=outputFilename,
+		inputFormat=inputFormat,
+		outputFormat=outputFormat,
+		readOptions=readOptions,
+		writeOptions=writeOptions,
+		**convertOptions
+	)
+	return True
+
+
+def getGitVersion(gitDir):
+	import subprocess
+	try:
+		outputB, error = subprocess.Popen(
+			[
+				"git",
+				"--git-dir", gitDir,
+				"describe",
+				"--always",
+			],
+			stdout=subprocess.PIPE,
+		).communicate()
+	except Exception as e:
+		sys.stderr.write(str(e) + "\n")
+		return ""
+	# if error is None:
+	return outputB.decode("utf-8").strip()
+
+
+def getVersion():
+	from pyglossary.core import rootDir
+	gitDir = os.path.join(rootDir, ".git")
+	if os.path.isdir(gitDir):
+		version = getGitVersion(gitDir)
+		if version:
+			return version
+	return core.VERSION
+
+
+def validateLangStr(st) -> "Optional[str]":
+	lang = langDict[st]
+	if lang:
+		return lang.name
+	lang = langDict[st.lower()]
+	if lang:
+		return lang.name
+	log.error(f"unknown language {st!r}")
+	return
 
 
 def main():
-	parser = argparse.ArgumentParser(add_help=False)
+	global log
+
+	uiBase = UIBase()
+	uiBase.loadConfig()
+	config = uiBase.config
+	defaultHasColor = config.get(
+		"color.enable.cmd.windows" if os.sep == "\\"
+		else "color.enable.cmd.unix",
+		True,
+	)
+
+	parser = argparse.ArgumentParser(
+		prog=sys.argv[0],
+		add_help=False,
+		# allow_abbrev=False,
+	)
 
 	parser.add_argument(
 		"-v",
@@ -117,8 +272,7 @@ def main():
 
 	parser.add_argument(
 		"--version",
-		action="version",
-		version=f"PyGlossary {core.VERSION}",
+		action="store_true",
 	)
 	parser.add_argument(
 		"-h",
@@ -164,12 +318,24 @@ def main():
 		default=None,
 		help="use Tkinter-based user interface",
 	)
+	parser.add_argument(
+		"--interactive",
+		"--inter",
+		dest="interactive",
+		action="store_true",
+		default=None,
+		help="switch to interactive command line interface",
+	)
 	parser.add_argument(
 		"--no-interactive",
+		"--no-inter",
 		dest="no_interactive",
 		action="store_true",
 		default=None,
-		help="do not automatically switch to interactive command line interface, for scripts",
+		help=(
+			"do not automatically switch to interactive command line"
+			" interface, for scripts"
+		),
 	)
 
 	parser.add_argument(
@@ -223,6 +389,23 @@ def main():
 			", this is default"
 		),
 	)
+	parser.add_argument(
+		"--sqlite",
+		dest="sqlite",
+		action="store_true",
+		default=None,
+		help=(
+			"use SQLite as middle storage instead of RAM in direct mode,"
+			"for very large glossaries"
+		),
+	)
+	parser.add_argument(
+		"--no-sqlite",
+		dest="sqlite",
+		action="store_false",
+		default=None,
+		help="do not use SQLite mode",
+	)
 
 	parser.add_argument(
 		"--no-progress-bar",
@@ -234,7 +417,7 @@ def main():
 		"--no-color",
 		dest="noColor",
 		action="store_true",
-		default=(os.sep != "/"),
+		default=not defaultHasColor,
 	)
 
 	parser.add_argument(
@@ -250,10 +433,42 @@ def main():
 		default=None,
 	)
 	parser.add_argument(
-		"--sort-cache-size",
-		dest="sortCacheSize",
-		type=int,
+		"--sort-key",
+		action="store",
+		dest="sortKeyName",
 		default=None,
+		help="name of sort key",
+	)
+	parser.add_argument(
+		"--sort-encoding",
+		action="store",
+		dest="sortEncoding",
+		default=None,
+		help="encoding of sort (default utf-8)",
+	)
+
+	# _______________________________
+
+	parser.add_argument(
+		"--source-lang",
+		action="store",
+		dest="sourceLang",
+		default=None,
+		help="source/query language",
+	)
+	parser.add_argument(
+		"--target-lang",
+		action="store",
+		dest="targetLang",
+		default=None,
+		help="target/definition language",
+	)
+	parser.add_argument(
+		"--name",
+		action="store",
+		dest="name",
+		default=None,
+		help="glossary name/title",
 	)
 
 	# _______________________________
@@ -277,32 +492,84 @@ def main():
 		nargs="?",
 	)
 
+	def shouldUseCMD(args):
+		if not canRunGUI():
+			return True
+		if args.interactive:
+			return True
+		if args.inputFilename and args.outputFilename:
+			return True
+		return False
+
 	# _______________________________
 
 	for key, option in UIBase.configDefDict.items():
-		registerOption(parser, key, option)
+		registerConfigOption(parser, key, option)
 
 	# _______________________________
 
 	args = parser.parse_args()
 
+	# parser.conflict_handler == "error"
+
+	if args.version:
+		print(f"PyGlossary {getVersion()}")
+		sys.exit(0)
+
 	log = logging.getLogger("pyglossary")
 
-	log.setVerbosity(args.verbosity)
-	log.addHandler(
-		core.StdLogHandler(noColor=args.noColor),
+	ui_type = args.ui_type
+	if ui_type == "none":
+		args.noColor = True
+
+	core.noColor = args.noColor
+	logHanlder = core.StdLogHandler(
+		noColor=args.noColor
 	)
+	log.setVerbosity(args.verbosity)
+	log.addHandler(logHanlder)
 	# with the logger setted up, we can import other pyglossary modules, so they
-	# can do some loggging in right way.
+	# can do some logging in right way.
+
+	for param1, param2 in UIBase.conflictingParams:
+		if getattr(args, param1) and getattr(args, param2):
+			log.critical(
+				"Conflicting flags: "
+				f"--{param1.replace('_', '-')} and "
+				f"--{param2.replace('_', '-')}"
+			)
+			sys.exit(1)
+
+	if args.sqlite:
+		# args.direct is None by default which means automatic
+		args.direct = False
+
+	if not args.sort:
+		if args.sortKeyName:
+			log.critical("Passed --sort-key without --sort")
+			sys.exit(1)
+		if args.sortEncoding:
+			log.critical("Passed --sort-encoding without --sort")
+			sys.exit(1)
+
+	if args.sortKeyName:
+		if args.sortKeyName not in namedSortKeyByName:
+			_valuesStr = ", ".join([_sk.name for _sk in namedSortKeyList])
+			log.critical(
+				f"Invalid sortKeyName={args.sortKeyName!r}"
+				f". Supported values:\n{_valuesStr}"
+			)
+			sys.exit(1)
 
 	core.checkCreateConfDir()
 
 	if sys.getdefaultencoding() != "utf-8":
-		log.warn(f"System encoding is not utf-8, it's {sys.getdefaultencoding()!r}")
+		log.warning(f"System encoding is not utf-8, it's {sys.getdefaultencoding()!r}")
 
 	##############################
 
-	from pyglossary.glossary import Glossary, langDict
+	from pyglossary.glossary import Glossary
+	from pyglossary.langs import langDict
 	from pyglossary.ui.ui_cmd import help, parseFormatOptionsStr
 
 	Glossary.init()
@@ -359,9 +626,9 @@ def main():
 		--read-options 'testOption=stringValue; enableFoo=True; fooList=[1, 2, 3]'
 		--read-options 'testOption=stringValue;enableFoo=True;fooList=[1,2,3]'
 
-		if a desired value contains ";", you can use --json-read-options or --json-write-options
-		flags instead, with json object as value, quoted for command line
-		for example:
+		if a desired value contains ";", you can use --json-read-options
+		or --json-write-options flags instead, with json object as value,
+		quoted for command line. for example:
 			'--json-write-options={"delimiter": ";"}'
 
 	"""
@@ -370,31 +637,47 @@ def main():
 		"direct",
 		"progressbar",
 		"sort",
-		"sortCacheSize",
-		# "sortKey",  # TODO
+		"sortKeyName",
+		"sortEncoding",
+		"sqlite",
+	)
+	infoOverrideSpec = (
+		("sourceLang", validateLangStr),
+		("targetLang", validateLangStr),
+		("name", str),
 	)
 
-	config = {}
-	for key, option in UIBase.configDefDict.items():
-		if not option.cmd:
+	for key, option in uiBase.configDefDict.items():
+		if not option.hasFlag:
 			continue
 		value = getattr(args, key, None)
 		if value is None:
 			continue
-		option = UIBase.configDefDict[key]
+		log.debug(f"config: {key} = {value}")
 		if not option.validate(value):
-			log.error("invalid config value: {key} = {value!r}")
+			log.error(f"invalid config value: {key} = {value!r}")
 			continue
 		config[key] = value
 
+	logHanlder.config = config
+
 	convertOptions = {}
 	for key in convertOptionsKeys:
 		value = getattr(args, key, None)
 		if value is not None:
 			convertOptions[key] = value
 
-	if convertOptions.get("sort", False):
-		convertOptions["defaultSortKey"] = Entry.defaultSortKey
+	infoOverride = {}
+	for key, validate in infoOverrideSpec:
+		value = getattr(args, key, None)
+		if value is None:
+			continue
+		value = validate(value)
+		if value is None:
+			continue
+		infoOverride[key] = value
+	if infoOverride:
+		convertOptions["infoOverride"] = infoOverride
 
 	if args.inputFilename and readOptions:
 		inputArgs = Glossary.detectInputFormat(
@@ -423,16 +706,14 @@ def main():
 			readOptions[optName] = optValueNew
 
 	if args.outputFilename and writeOptions:
-		_, outputFormat, _ = Glossary.detectOutputFormat(
+		outputArgs = Glossary.detectOutputFormat(
 			filename=args.outputFilename,
 			format=args.outputFormat,
 			inputFilename=args.inputFilename,
 		)
-		if not outputFormat:
-			log.error(
-				f"Could not detect format for output file {args.outputFilename}"
-			)
+		if outputArgs is None:
 			sys.exit(1)
+		_, outputFormat, _ = outputArgs
 		writeOptionsProp = Glossary.plugins[outputFormat].optionsProp
 		for optName, optValue in writeOptions.items():
 			if optName not in Glossary.formatsWriteOptions[outputFormat]:
@@ -448,52 +729,35 @@ def main():
 				sys.exit(1)
 			writeOptions[optName] = optValueNew
 
-	if config:
-		log.debug(f"config = {config}")
 	if convertOptions:
 		log.debug(f"convertOptions = {convertOptions}")
 
-	ui_type = args.ui_type
-
-	if ui_type == "auto":
-		if not canRunGUI() or (args.inputFilename and args.outputFilename):
-			ui_type = "cmd"
-
-	if ui_type == "none":
-		if args.reverse:
-			log.error("--reverse does not work with --ui=none")
-			sys.exit(1)
-		ui = UIBase()
-		ui.loadConfig(**config)
-		glos = Glossary(ui=ui)
-		glos.convert(
-			args.inputFilename,
-			inputFormat=args.inputFormat,
-			outputFilename=args.outputFilename,
-			outputFormat=args.outputFormat,
-			readOptions=readOptions,
-			writeOptions=writeOptions,
-			**convertOptions
-		)
-		sys.exit(0)
-
 	runKeywordArgs = dict(
 		inputFilename=args.inputFilename,
 		outputFilename=args.outputFilename,
 		inputFormat=args.inputFormat,
 		outputFormat=args.outputFormat,
 		reverse=args.reverse,
-		configOptions=config,
+		config=config,
 		readOptions=readOptions,
 		writeOptions=writeOptions,
 		convertOptions=convertOptions,
+		glossarySetAttrs=None,
 	)
 
+	if ui_type == "none":
+		sys.exit(0 if base_ui_run(**runKeywordArgs) else 1)
+
+	if ui_type == "auto" and shouldUseCMD(args):
+		ui_type = "cmd"
+
 	if ui_type == "cmd":
-		if args.inputFilename and args.outputFilename:
-			from ui.ui_cmd import UI
+		if args.interactive:
+			from pyglossary.ui.ui_cmd_interactive import UI
+		elif args.inputFilename and args.outputFilename:
+			from pyglossary.ui.ui_cmd import UI
 		elif not args.no_interactive:
-			from ui.ui_cmd_interactive import UI
+			from pyglossary.ui.ui_cmd_interactive import UI
 		else:
 			log.error("no input file given, try --help")
 			sys.exit(1)
@@ -503,9 +767,9 @@ def main():
 		ui_module = None
 		for ui_type2 in ui_list:
 			try:
-				ui_module = getattr(
-					__import__(f"ui.ui_{ui_type2}"),
-					f"ui_{ui_type2}",
+				ui_module = __import__(
+					f"pyglossary.ui.ui_{ui_type2}",
+					fromlist=f"ui_{ui_type2}",
 				)
 			except ImportError:
 				log.exception("error while importing UI module:")
@@ -518,9 +782,9 @@ def main():
 			)
 			sys.exit(1)
 	else:
-		ui_module = getattr(
-			__import__(f"ui.ui_{ui_type}"),
-			f"ui_{ui_type}",
+		ui_module = __import__(
+			f"pyglossary.ui.ui_{ui_type}",
+			fromlist=f"ui_{ui_type}",
 		)
 
 	sys.exit(0 if ui_module.UI().run(**runKeywordArgs) else 1)
