@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ui_cmd.py
 #
-# Copyright © 2008-2019 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
+# Copyright © 2008-2021 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 # This file is part of PyGlossary project, https://github.com/ilius/pyglossary
 #
 # This program is a free software; you can redistribute it and/or modify
@@ -32,12 +32,10 @@ def wc_ljust(text, length, padding=' '):
 
 
 if os.sep == "\\":  # Operating system is Windows
-	startRed = ""
 	startBold = ""
 	startUnderline = ""
 	endFormat = ""
 else:
-	startRed = "\x1b[31m"
 	startBold = "\x1b[1m"  # Start Bold # len=4
 	startUnderline = "\x1b[4m"  # Start Underline # len=4
 	endFormat = "\x1b[0;0;0m"  # End Format # len=8
@@ -98,8 +96,7 @@ def getFormatsTable(names, header):
 
 def help():
 	import string
-	with open(join(dataDir, "help")) as fp:
-		text = fp.read()
+	text = fread(join(dataDir, "help"))
 	text = text.replace("<b>", startBold)\
 		.replace("<u>", startUnderline)\
 		.replace("</b>", endFormat)\
@@ -262,21 +259,24 @@ class UI(UIBase):
 		inputFormat: str = "",
 		outputFormat: str = "",
 		reverse: bool = False,
-		configOptions: "Optional[Dict]" = None,
+		config: "Optional[Dict]" = None,
 		readOptions: "Optional[Dict]" = None,
 		writeOptions: "Optional[Dict]" = None,
 		convertOptions: "Optional[Dict]" = None,
+		glossarySetAttrs: "Optional[Dict]" = None,
 	):
-		if configOptions is None:
-			configOptions = {}
+		if config is None:
+			config = {}
 		if readOptions is None:
 			readOptions = {}
 		if writeOptions is None:
 			writeOptions = {}
 		if convertOptions is None:
 			convertOptions = {}
+		if glossarySetAttrs is None:
+			glossarySetAttrs = {}
 
-		self.loadConfig(**configOptions)
+		self.config = config
 
 		if inputFormat:
 			# inputFormat = inputFormat.capitalize()
@@ -306,6 +306,11 @@ class UI(UIBase):
 				return 1
 
 		glos = self.glos = Glossary(ui=self)
+		self.glos.config = self.config
+
+		for attr, value in glossarySetAttrs.items():
+			setattr(glos, attr, value)
+
 		if reverse:
 			import signal
 			signal.signal(signal.SIGINT, self.onSigInt)  # good place? FIXME
