@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # ui_cmd_interactive.py
 #
-# Copyright © 2008-2020 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
+# Copyright © 2008-2022 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 # This file is part of PyGlossary project, https://github.com/ilius/pyglossary
 #
 # This program is a free software; you can redistribute it and/or modify
@@ -20,36 +20,67 @@
 # If not, see <http://www.gnu.org/licenses/gpl.txt>.
 
 """
-sudo pip3 install prompt_toolkit
+To use this user interface:
+	sudo pip3 install prompt_toolkit
 """
 
+# GitHub repo for prompt_toolkit
+# https://github.com/prompt-toolkit/python-prompt-toolkit
+
+# The code for Python's cmd.Cmd was very ugly and hard to understand last I
+# cheched. But we don't use cmd module here, and nor does prompt_toolkit.
+
+# Completion func for Python's readline, silently (and stupidly) hides any
+# exception, and only shows the print if it's in the first line of function.
+# very awkward!
+# We also don't use readline module, and nor does prompt_toolkit.
+# Looks like prompt_toolkit works directly with sys.stdin, sys.stdout
+# and sys.stderr.
+
+# prompt_toolkit also supports ncurses-like dialogs with buttons and widgets,
+# but I prefer this kind of UI with auto-completion and history
+
 import sys
 import os
-from os.path import dirname, join, abspath, isdir, isabs
+from os.path import (
+	dirname,
+	join,
+	abspath,
+	relpath,
+	isdir,
+	isabs,
+)
 import logging
 from collections import OrderedDict
+import argparse
+import shlex
 
 import json
 
-# the code for cmd.Cmd is very ugly and hard to understan
-
-# readline's complete func silently (and stupidly) hides any exception
-# and only shows the print if it's in the first line of function. very awkward!
-
+from pyglossary import core
 from pyglossary.core import confDir
 from pyglossary.glossary import Glossary
 from pyglossary.ui import ui_cmd
+from pyglossary.sort_keys import namedSortKeyList, namedSortKeyByName
 
 
 from prompt_toolkit import prompt as promptLow
 from prompt_toolkit.history import FileHistory
 from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
-from prompt_toolkit.completion.word_completer import WordCompleter
+from prompt_toolkit.completion import (
+	WordCompleter,
+	PathCompleter,
+	Completion,
+)
+from prompt_toolkit import ANSI
 from prompt_toolkit.shortcuts import confirm, PromptSession
 from prompt_toolkit.key_binding import KeyBindings
 from prompt_toolkit.keys import Keys
 
 
+endFormat = "\x1b[0;0;0m"
+
+
 class MiniCheckBoxPrompt(object):
 	def __init__(
 		self,
@@ -62,10 +93,12 @@ class MiniCheckBoxPrompt(object):
 		self.value = value
 
 	def formatMessage(self):
-		return self.fmt.format(
+		msg = self.fmt.format(
 			check="[x]" if self.value else "[ ]",
 			message=self.message,
 		)
+		# msg = ANSI(msg)  # NOT SUPPORTED
+		return msg
 
 	def __pt_formatted_text__(self):
 		return [("", self.formatMessage())]
@@ -130,6 +163,18 @@ writeFormatDescList = [
 	for _format in Glossary.writeFormats
 ]
 
+convertOptionsFlags = {
+	"direct": ("indirect", "direct"),
+	"sqlite": ("", "sqlite"),
+	"progressbar": ("no-progress-bar", ""),
+	"sort": ("no-sort", "sort"),
+}
+infoOverrideFlags = {
+	"sourceLang": "source-lang",
+	"targetLang": "target-lang",
+	"name": "name",
+}
+
 
 def dataToPrettyJson(data, ensure_ascii=False, sort_keys=False):
 	return json.dumps(
@@ -161,23 +206,102 @@ def prompt(
 back = "back"
 
 
+class MyPathCompleter(PathCompleter):
+	def __init__(
+		self,
+		reading: bool,
+		fs_action_names=None,
+		**kwargs
+	):
+		PathCompleter.__init__(
+			self,
+			file_filter=self.file_filter,
+			**kwargs
+		)
+		if fs_action_names is None:
+			fs_action_names = []
+		self.fs_action_names = fs_action_names
+
+	def file_filter(self, filename: str) -> bool:
+		# filename is full/absoule file path
+		return True
+
+	# def get_completions_exception(document, complete_event, e):
+	# 	log.error(f"Execption in get_completions: {e}")
+
+	def get_completions(
+		self,
+		document: "Document",
+		complete_event: "CompleteEvent",
+	) -> "Iterable[Completion]":
+		text = document.text_before_cursor
+
+		for action in self.fs_action_names:
+			if action.startswith(text):
+				yield Completion(
+					text=action,
+					start_position=-len(text),
+					display=action,
+				)
+
+		yield from PathCompleter.get_completions(
+			self,
+			document=document,
+			complete_event=complete_event,
+		)
+
+
+class AbsolutePathHistory(FileHistory):
+	def load_history_strings(self) -> "Iterable[str]":
+		# pwd = os.getcwd()
+		pathList = FileHistory.load_history_strings(self)
+		return [
+			relpath(p)
+			for p in pathList
+		]
+
+	def store_string(self, string: str) -> None:
+		FileHistory.store_string(self, abspath(string))
+
+
 class UI(ui_cmd.UI):
 	def __init__(self):
 		self._inputFilename = ""
 		self._outputFilename = ""
 		self._inputFormat = ""
 		self._outputFormat = ""
-		self._configOptions = None
+		self.config = None
 		self._readOptions = None
 		self._writeOptions = None
 		self._convertOptions = None
 		ui_cmd.UI.__init__(self)
 
+		self.ls_parser = argparse.ArgumentParser(add_help=False)
+		self.ls_parser.add_argument(
+			"-l",
+			"--long",
+			action="store_true",
+			dest="long",
+			help="use a long listing format",
+		)
+		self.ls_parser.add_argument(
+			"--help",
+			action="store_true",
+			dest="help",
+			help="display help",
+		)
+		self.ls_usage = (
+			'Usage: !ls [--help] [-l] [FILE/DIRECTORY]...\n\n'
+			'optional arguments:\n'
+			'    --help      show this help message and exit\n'
+			'    -l, --long  use a long listing format\n'
+		)
+
 		self._fsActions = OrderedDict([
-			("!pwd", self.fs_pwd),
-			("!ls", self.fs_ls),
-			("!..", self.fs_cd_parent),
-			("!cd", self.fs_cd),
+			("!pwd", (self.fs_pwd, "")),
+			("!ls", (self.fs_ls, self.ls_usage)),
+			("!..", (self.fs_cd_parent, "")),
+			("!cd", (self.fs_cd, "")),
 		])
 		self._finalActions = OrderedDict([
 			("formats", self.askFormats),
@@ -187,8 +311,10 @@ class UI(ui_cmd.UI):
 			("reset-write-options", self.resetWriteOptions),
 			("config", self.askConfig),
 			("indirect", self.setIndirect),
+			("sqlite", self.setSQLite),
 			("no-progressbar", self.setNoProgressbar),
 			("sort", self.setSort),
+			("sort-key", self.setSortKey),
 			("show-options", self.showOptions),
 			("back", None),
 		])
@@ -196,42 +322,83 @@ class UI(ui_cmd.UI):
 	def fs_pwd(self, args: "List[str]"):
 		print(os.getcwd())
 
-	def get_ls_l(self, arg: str) -> str:
+	def get_ls_l(
+		self,
+		arg: str,
+		st: "Optional[os.stat_result]" = None,
+		parentDir: str = "",
+		sizeWidth: int = 0,
+	) -> str:
 		import stat
 		import pwd
 		import grp
 		import time
-		st = os.lstat(arg)
+		argPath = arg
+		if parentDir:
+			argPath = join(parentDir, arg)
+		if st is None:
+			st = os.lstat(argPath)
 		# os.lstat does not follow sym links, like "ls" command
 		details = [
 			stat.filemode(st.st_mode),
 			pwd.getpwuid(st.st_uid).pw_name,
 			grp.getgrgid(st.st_gid).gr_name,
-			str(st.st_size),
+			str(st.st_size).rjust(sizeWidth),
 			time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(st.st_mtime)),
 			arg,
 		]
 		if stat.S_ISLNK(st.st_mode):
-			details.append(f"-> {os.readlink(arg)}")
+			details.append(f"-> {os.readlink(argPath)}")
 		return "  ".join(details)
 
 	def fs_ls(self, args: "List[str]"):
+		opts, args = self.ls_parser.parse_known_args(args=args)
+
+		if opts.help:
+			print(self.ls_usage)
+			return
+
 		if not args:
 			args = [os.getcwd()]
+
 		showTitle = len(args) > 1
 		# Note: isdir and isfile funcs follow sym links, so no worry about links
+
 		for i, arg in enumerate(args):
 			if i > 0:
 				print()
+
 			if not isdir(arg):
 				print(self.get_ls_l(arg))
 				continue
+
 			if showTitle:
-				print(f"> List of directory {arg}:")
-			for _path in os.listdir(arg):
-				if isdir(_path):
-					_path += "/"
-				print(f"{_path}")
+				print(f"> List of directory {arg!r}:")
+
+			if not opts.long:
+				for _path in os.listdir(arg):
+					if isdir(_path):
+						_path += "/"
+					print(f"{_path}")
+				continue
+
+			contents = os.listdir(arg)
+			statList = [
+				os.lstat(join(arg, _path))
+				for _path in contents
+			]
+			maxFileSize = max([
+				st.st_size
+				for st in statList
+			])
+			sizeWidth = len(str(maxFileSize))
+			for i, _path in enumerate(contents):
+				print(self.get_ls_l(
+					_path,
+					parentDir=arg,
+					st=statList[i],
+					sizeWidth=sizeWidth,
+				))
 
 	def fs_cd_parent(self, args: "List[str]"):
 		if args:
@@ -251,26 +418,45 @@ class UI(ui_cmd.UI):
 		os.chdir(newDir)
 		print(f"Changed current directory to: {newDir}")
 
+	def formatPromptMsg(self, level, msg, colon=":"):
+		indent = self.promptIndentStr * level
+
+		if core.noColor:
+			return f"{indent} {msg}{colon} ", False
+
+		if self.promptIndentColor >= 0:
+			indent = f"\x1b[38;5;{self.promptIndentColor}m{indent}{endFormat}"
+
+		if self.promptMsgColor >= 0:
+			msg = f"\x1b[38;5;{self.promptMsgColor}m{msg}{endFormat}"
+
+		return f"{indent} {msg}{colon} ", True
+
+	def prompt(self, level, msg, colon=":", **kwargs):
+		msg, colored = self.formatPromptMsg(level, msg, colon)
+		if colored:
+			msg = ANSI(msg)
+		return prompt(msg, **kwargs)
+
+	def checkbox_prompt(self, level, msg, colon=":", **kwargs):
+		# FIXME: colors are not working, they are being escaped
+		msg = f"{self.promptIndentStr * level} {msg}{colon} "
+		# msg, colored = self.formatPromptMsg(level, msg, colon)
+		return checkbox_prompt(msg, **kwargs)
+
 	def askFile(self, kind: str, histName: str, varName: str, reading: bool):
 		from shlex import split as shlex_split
-		history = FileHistory(join(histDir, histName))
+		history = AbsolutePathHistory(join(histDir, histName))
 		auto_suggest = AutoSuggestFromHistory()
-		completer_keys = list(self._fsActions.keys())
 		# Note: isdir and isfile funcs follow sym links, so no worry about links
-		for _path in os.listdir(os.getcwd()):
-			if isdir(_path):
-				continue
-			completer_keys.append(_path)
-		completer = WordCompleter(
-			completer_keys,
-			ignore_case=False,
-			match_middle=False,
-			sentence=True,
+		completer = MyPathCompleter(
+			reading=reading,
+			fs_action_names=list(self._fsActions.keys()),
 		)
 		default = getattr(self, varName)
 		while True:
-			filename = prompt(
-				f"> {kind}: ",
+			filename = self.prompt(
+				1, kind,
 				history=history,
 				auto_suggest=auto_suggest,
 				completer=completer,
@@ -280,11 +466,13 @@ class UI(ui_cmd.UI):
 				continue
 			parts = shlex_split(filename)
 			if parts[0] in self._fsActions:
-				actionFunc = self._fsActions[parts[0]]
+				actionFunc, usage = self._fsActions[parts[0]]
 				try:
 					actionFunc(parts[1:])
 				except Exception as e:
 					log.exception("")
+					if usage:
+						print("\n" + usage)
 				continue
 			setattr(self, varName, filename)
 			return filename
@@ -326,8 +514,8 @@ class UI(ui_cmd.UI):
 			sentence=True,
 		)
 		while True:
-			value = prompt(
-				"> Input format: ",
+			value = self.prompt(
+				1, "Input format",
 				history=history,
 				auto_suggest=auto_suggest,
 				completer=completer,
@@ -350,8 +538,8 @@ class UI(ui_cmd.UI):
 			sentence=True,
 		)
 		while True:
-			value = prompt(
-				"> Output format: ",
+			value = self.prompt(
+				1, "Output format",
 				history=history,
 				auto_suggest=auto_suggest,
 				completer=completer,
@@ -404,8 +592,8 @@ class UI(ui_cmd.UI):
 		)
 		while True:
 			try:
-				optName = prompt(
-					">> ReadOption: Name (ENTER if done): ",
+				optName = self.prompt(
+					2, "ReadOption: Name (ENTER if done)",
 					history=history,
 					auto_suggest=auto_suggest,
 					completer=completer,
@@ -419,11 +607,12 @@ class UI(ui_cmd.UI):
 			default = self._readOptions.get(optName)
 			if default is None:
 				default = options[optName]
+			print(f"Comment: {option.longComment}")
 			while True:
 				if option.typ == "bool":
 					try:
-						valueNew = checkbox_prompt(
-							f">>> ReadOption: {optName}",
+						valueNew = self.checkbox_prompt(
+							3, f"ReadOption: {optName}",
 							default=default,
 						)
 					except (KeyboardInterrupt, EOFError):
@@ -432,8 +621,9 @@ class UI(ui_cmd.UI):
 					self._readOptions[optName] = valueNew
 					break
 				try:
-					value = prompt(
-						f">>> ReadOption: {optName} = ",
+					value = self.prompt(
+						3, f"ReadOption: {optName}",
+						colon=" =",
 						history=FileHistory(join(histDir, f"option-value-{optName}")),
 						auto_suggest=AutoSuggestFromHistory(),
 						default=str(default),
@@ -475,8 +665,8 @@ class UI(ui_cmd.UI):
 		)
 		while True:
 			try:
-				optName = prompt(
-					">> WriteOption: Name (ENTER if done): ",
+				optName = self.prompt(
+					2, "WriteOption: Name (ENTER if done)",
 					history=history,
 					auto_suggest=auto_suggest,
 					completer=completer,
@@ -486,6 +676,7 @@ class UI(ui_cmd.UI):
 			if not optName:
 				return
 			option = optionsProp[optName]
+			print(f"Comment: {option.longComment}")
 			valueCompleter = self.getOptionValueCompleter(option)
 			default = self._writeOptions.get(optName)
 			if default is None:
@@ -493,8 +684,8 @@ class UI(ui_cmd.UI):
 			while True:
 				if option.typ == "bool":
 					try:
-						valueNew = checkbox_prompt(
-							f">>> WriteOption: {optName}",
+						valueNew = self.checkbox_prompt(
+							3, f"WriteOption: {optName}",
 							default=default,
 						)
 					except (KeyboardInterrupt, EOFError):
@@ -503,8 +694,9 @@ class UI(ui_cmd.UI):
 					self._writeOptions[optName] = valueNew
 					break
 				try:
-					value = prompt(
-						f">>> WriteOption: {optName} = ",
+					value = self.prompt(
+						3, f"WriteOption: {optName}",
+						colon=" =",
 						history=FileHistory(join(histDir, f"option-value-{optName}")),
 						auto_suggest=AutoSuggestFromHistory(),
 						default=str(default),
@@ -538,12 +730,13 @@ class UI(ui_cmd.UI):
 	def askConfigValue(self, configKey, option):
 		default = self.config.get(configKey, "")
 		if option.typ == "bool":
-			return str(checkbox_prompt(
-				message=f">>> Config: {configKey}",
+			return str(self.checkbox_prompt(
+				3, f"Config: {configKey}",
 				default=bool(default),
 			))
-		return prompt(
-			f">>> Config: {configKey} = ",
+		return self.prompt(
+			3, f"Config: {configKey}",
+			colon=" =",
 			history=FileHistory(join(histDir, f"config-value-{configKey}")),
 			auto_suggest=AutoSuggestFromHistory(),
 			default=str(default),
@@ -563,8 +756,8 @@ class UI(ui_cmd.UI):
 
 		while True:
 			try:
-				configKey = prompt(
-					">> Config: Key (ENTER if done): ",
+				configKey = self.prompt(
+					2, "Config: Key (ENTER if done)",
 					history=history,
 					auto_suggest=auto_suggest,
 					completer=completer,
@@ -580,9 +773,9 @@ class UI(ui_cmd.UI):
 				except (KeyboardInterrupt, EOFError):
 					break
 				if value == "":
-					if configKey in self._configOptions:
+					if configKey in self.config:
 						print(f"Unset config {configKey!r}")
-						del self._configOptions[configKey]
+						del self.config[configKey]
 					# FIXME: set empty value?
 					break
 				valueNew, ok = option.evaluate(value)
@@ -592,7 +785,7 @@ class UI(ui_cmd.UI):
 					)
 					continue
 				print(f"Set config: {configKey} = {valueNew!r}")
-				self._configOptions[configKey] = valueNew
+				self.config[configKey] = valueNew
 				self.config[configKey] = valueNew
 				break
 
@@ -600,13 +793,19 @@ class UI(ui_cmd.UI):
 		print(f"readOptions = {self._readOptions}")
 		print(f"writeOptions = {self._writeOptions}")
 		print(f"convertOptions = {self._convertOptions}")
-		print(f"configOptions = {self._configOptions}")
+		print(f"config = {self.config}")
 		print()
 
 	def setIndirect(self):
 		self._convertOptions["direct"] = False
+		self._convertOptions["sqlite"] = None
 		print("Switched to indirect mode")
 
+	def setSQLite(self):
+		self._convertOptions["direct"] = None
+		self._convertOptions["sqlite"] = True
+		print("Switched to SQLite mode")
+
 	def setNoProgressbar(self):
 		self._convertOptions["progressbar"] = False
 		print("Disabled progress bar")
@@ -614,15 +813,42 @@ class UI(ui_cmd.UI):
 	def setSort(self):
 		from pyglossary.entry import Entry
 		try:
-			value = checkbox_prompt(
-				message=f">> Enable Sort",
+			value = self.checkbox_prompt(
+				2, f"Enable Sort",
 				default=self._convertOptions.get("sort", False),
 			)
 		except (KeyboardInterrupt, EOFError):
 			return
 		self._convertOptions["sort"] = value
-		if value:
-			self._convertOptions["defaultSortKey"] = Entry.defaultSortKey
+
+	def setSortKey(self):
+		completer = WordCompleter(
+			[_sk.name for _sk in namedSortKeyList],
+			ignore_case=False,
+			match_middle=True,
+			sentence=True,
+		)
+		default = self._convertOptions.get("sortKeyName", "")
+		sortKeyName = self.prompt(
+			2, "SortKey",
+			history=FileHistory(join(histDir, f"sort-key")),
+			auto_suggest=AutoSuggestFromHistory(),
+			default=default,
+			completer=completer,
+		)
+		if not sortKeyName:
+			if "sortKeyName" in self._convertOptions:
+				del self._convertOptions["sortKeyName"]
+			return
+
+		if sortKeyName not in namedSortKeyByName:
+			log.error(f"invalid sortKeyName = {sortKeyName!r}")
+			return
+
+		self._convertOptions["sortKeyName"] = sortKeyName
+
+		if not self._convertOptions.get("sort"):
+			self.setSort()
 
 	def askFinalAction(self) -> "Optional[str]":
 		history = FileHistory(join(histDir, "action"))
@@ -634,8 +860,8 @@ class UI(ui_cmd.UI):
 			sentence=True,
 		)
 		while True:
-			action = prompt(
-				"> Select action (ENTER to convert): ",
+			action = self.prompt(
+				1, "Select action (ENTER to convert)",
 				history=history,
 				auto_suggest=auto_suggest,
 				completer=completer,
@@ -673,10 +899,11 @@ class UI(ui_cmd.UI):
 			outputFilename=self._outputFilename,
 			inputFormat=self._inputFormat,
 			outputFormat=self._outputFormat,
-			configOptions=self._configOptions,
+			config=self.config,
 			readOptions=self._readOptions,
 			writeOptions=self._writeOptions,
 			convertOptions=self._convertOptions,
+			glossarySetAttrs=self._glossarySetAttrs,
 		)
 
 	def checkInputFormat(self, forceAsk: bool = False):
@@ -728,16 +955,18 @@ class UI(ui_cmd.UI):
 			optionsJson = json.dumps(self._writeOptions, ensure_ascii=True)
 			cmd.append(quote(f"--json-write-options={optionsJson}"))
 
-		if self._configOptions:
-			for key, value in self._configOptions.items():
+		if self.config:
+			for key, value in self.config.items():
 				if value is None:
 					continue
+				if value == self.savedConfig.get(key):
+					continue
 				option = self.configDefDict.get(key)
 				if option is None:
 					log.error(f"config key {key} was not found")
-				if not option.cmd:
+				if not option.hasFlag:
 					log.error(f"config key {key} has no command line flag")
-				flag = option.cmdFlag
+				flag = option.customFlag
 				if not flag:
 					flag = key.replace('_', '-')
 				if option.typ == "bool":
@@ -748,13 +977,19 @@ class UI(ui_cmd.UI):
 					cmd.append(quote(f"--{flag}={value}"))
 
 		if self._convertOptions:
-			convertOptionsFlags = {
-				"direct": ("indirect", "direct"),
-				"progressbar": ("no-progress-bar", ""),
-				"sort": ("no-sort", "sort"),
-				"sortCacheSize": ("sort-cache-size", ""),
-				"defaultSortKey": None,
-			}
+			if "infoOverride" in self._convertOptions:
+				infoOverride = self._convertOptions.pop("infoOverride")
+				for key, value in infoOverride.items():
+					flag = infoOverrideFlags.get(key)
+					if not flag:
+						log.error(f"unknow key {key} in infoOverride")
+						continue
+					cmd.append(f"--{flag}={value}")
+
+			if "sortKeyName" in self._convertOptions:
+				value = self._convertOptions.pop("sortKeyName")
+				cmd.append(f"--sort-key={value}")
+
 			for key, value in self._convertOptions.items():
 				if value is None:
 					continue
@@ -766,7 +1001,8 @@ class UI(ui_cmd.UI):
 					continue
 				if isinstance(value, bool):
 					flag = ftup[int(value)]
-					cmd.append(f"--{flag}")
+					if flag:
+						cmd.append(f"--{flag}")
 				else:
 					flag = ftup[0]
 					cmd.append(f"--{flag}={value}")
@@ -776,7 +1012,14 @@ class UI(ui_cmd.UI):
 			"If you want to repeat this conversion later, "
 			"you can use this command:"
 		)
-		print(" ".join(cmd))
+		print(shlex.join(cmd))
+
+	def setConfigAttrs(self):
+		config = self.config
+		self.promptIndentStr = config.get("cmdi.prompt.indent.str", ">")
+		self.promptIndentColor = config.get("cmdi.prompt.indent.color", 2)
+		self.promptMsgColor = config.get("cmdi.prompt.msg.color", -1)
+		self.msgColor = config.get("cmdi.msg.color", -1)
 
 	def run(
 		self,
@@ -785,33 +1028,41 @@ class UI(ui_cmd.UI):
 		inputFormat: str = "",
 		outputFormat: str = "",
 		reverse: bool = False,
-		configOptions: "Optional[Dict]" = None,
+		config: "Optional[Dict]" = None,
 		readOptions: "Optional[Dict]" = None,
 		writeOptions: "Optional[Dict]" = None,
 		convertOptions: "Optional[Dict]" = None,
+		glossarySetAttrs: "Optional[Dict]" = None,
 	):
-		if configOptions is None:
-			configOptions = {}
+		if config is None:
+			config = {}
 		if readOptions is None:
 			readOptions = {}
 		if writeOptions is None:
 			writeOptions = {}
 		if convertOptions is None:
 			convertOptions = {}
+		if glossarySetAttrs is None:
+			glossarySetAttrs = {}
 
 		self._inputFilename = inputFilename
 		self._outputFilename = outputFilename
 		self._inputFormat = inputFormat
 		self._outputFormat = outputFormat
-		self._configOptions = configOptions
 		self._readOptions = readOptions
 		self._writeOptions = writeOptions
 		self._convertOptions = convertOptions
+		self._glossarySetAttrs = glossarySetAttrs
+
+
+		self.loadConfig()
+		self.savedConfig = dict(self.config)
+		self.config = config
 
 		del inputFilename, outputFilename, inputFormat, outputFormat
-		del configOptions, readOptions, writeOptions, convertOptions
+		del config, readOptions, writeOptions, convertOptions
 
-		self.loadConfig(**self._configOptions)
+		self.setConfigAttrs()
 
 		if not self._inputFilename:
 			try:
@@ -848,7 +1099,7 @@ class UI(ui_cmd.UI):
 			else:
 				self.printNonInteractiveCommand()
 				if succeed:
-					if self._configOptions and confirm("Save Config?"):
+					if self.config != self.savedConfig and confirm("Save Config?"):
 						self.saveConfig()
 					return succeed
 			print("Press Control + C to exit")
