@@ -1,9 +1,7 @@
 # -*- coding: utf-8 -*-
 # ui_gtk.py
 #
-# Copyright © 2008-2020 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
-# Thanks to Pier Carteri <m3tr0@dei.unipd.it> for program Py_Shell.py
-# Thanks to Milad Rastian for program pySQLiteGUI
+# Copyright © 2008-2022 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 #
 # This program is a free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,14 +22,16 @@ import os
 from os.path import join, isfile, isabs, splitext, abspath
 import logging
 import traceback
+from collections import OrderedDict
 
 from pyglossary.text_utils import urlToPath
 from pyglossary.os_utils import click_website
 
 from pyglossary.glossary import (
 	Glossary,
-	homePage,
+	defaultSortKeyName,
 )
+from pyglossary.sort_keys import namedSortKeyList, namedSortKeyByName
 
 from .base import (
 	UIBase,
@@ -61,6 +61,33 @@ gtk.Window.set_default_icon_from_file(logo)
 
 _ = str  # later replace with translator function
 
+pluginByDesc = {
+	plugin.description: plugin
+	for plugin in Glossary.plugins.values()
+}
+readDesc = [
+	plugin.description
+	for plugin in Glossary.plugins.values()
+	if plugin.canRead
+]
+writeDesc = [
+	plugin.description
+	for plugin in Glossary.plugins.values()
+	if plugin.canWrite
+]
+
+
+def getScreenSize():
+	rootWindow = gdk.get_default_root_window()
+	return rootWindow.get_width(), rootWindow.get_height()
+
+
+def getWorkAreaSize():
+	display = gdk.Display.get_default()
+	monitor = display.get_primary_monitor()
+	rect = monitor.get_workarea()
+	return rect.width, rect.height
+
 
 def buffer_get_text(b):
 	return b.get_text(
@@ -70,51 +97,248 @@ def buffer_get_text(b):
 	)
 
 
+class FormatDialog(gtk.Dialog):
+	def __init__(self, descList: "List[str]", parent=None, **kwargs):
+		gtk.Dialog.__init__(self, parent=parent, **kwargs)
+		self.descList = descList
+		self.items = descList
+		self.activeDesc = ""
+		##
+		self.connect("response", lambda w, e: self.hide())
+		dialog_add_button(
+			self,
+			"gtk-cancel",
+			"_Cancel",
+			gtk.ResponseType.CANCEL,
+		)
+		dialog_add_button(
+			self,
+			"gtk-ok",
+			"_OK",
+			gtk.ResponseType.OK,
+		)
+		###
+		treev = gtk.TreeView()
+		treeModel = gtk.ListStore(str)
+		treev.set_headers_visible(False)
+		treev.set_model(treeModel)
+		treev.connect("row-activated", self.rowActivated)
+		# treev.connect("response", self.onResponse)
+		###
+		self.treev = treev
+		#############
+		cell = gtk.CellRendererText(editable=False)
+		col = gtk.TreeViewColumn(
+			title="Descriptin",
+			cell_renderer=cell,
+			text=0,
+		)
+		col.set_property("expand", True)
+		col.set_resizable(True)
+		treev.append_column(col)
+		self.descCol = col
+		############
+		hbox = HBox(spacing=15)
+		hbox.set_border_width(10)
+		pack(hbox, gtk.Label("Search:"))
+		entry = self.entry = gtk.Entry()
+		pack(hbox, entry, 1, 1)
+		pack(self.vbox, hbox)
+		###
+		entry.connect("changed", self.onEntryChange)
+		############
+		self.swin = swin = gtk.ScrolledWindow()
+		swin.add(treev)
+		swin.set_policy(gtk.PolicyType.NEVER, gtk.PolicyType.AUTOMATIC)
+		pack(self.vbox, swin, 1, 1)
+		self.vbox.show_all()
+		##
+		treev.set_can_focus(True)  # no need, just to be safe
+		treev.set_can_default(True)
+		treev.set_receives_default(True)
+		# print("can_focus:", treev.get_can_focus())
+		# print("can_default:", treev.get_can_default())
+		# print("receives_default:", treev.get_receives_default())
+		####
+		self.updateTree()
+		self.resize(400, 400)
+		self.connect("realize", self.onRealize)
+
+	def onRealize(self, widget=None):
+		if self.activeDesc:
+			self.treev.grab_focus()
+		else:
+			self.entry.grab_focus()
+
+	def onEntryChange(self, entry):
+		text = entry.get_text().strip()
+		if not text:
+			self.items = self.descList
+			self.updateTree()
+			return
+
+		text = text.lower()
+		descList = self.descList
+
+		items1 = []
+		items2 = []
+		for desc in descList:
+			if desc.lower().startswith(text):
+				items1.append(desc)
+			elif text in desc.lower():
+				items2.append(desc)
+
+		self.items = items1 + items2
+		self.updateTree()
+
+	def setCursor(self, desc: str):
+		model = self.treev.get_model()
+		_iter = model.iter_children(None)
+		while _iter is not None:
+			if model.get_value(_iter, 0) == desc:
+				path = model.get_path(_iter)
+				self.treev.set_cursor(path, self.descCol, False)
+				self.treev.scroll_to_cell(path)
+				return
+			_iter = model.iter_next(_iter)
+
+	def updateTree(self):
+		model = self.treev.get_model()
+		model.clear()
+		for desc in self.items:
+			model.append([desc])
+
+		if self.activeDesc:
+			self.setCursor(self.activeDesc)
+
+	def getActive(self) -> "Optional[PluginProp]":
+		_iter = self.treev.get_selection().get_selected()[1]
+		if _iter is None:
+			return
+		model = self.treev.get_model()
+		desc = model.get_value(_iter, 0)
+		return pluginByDesc[desc]
+
+	def setActive(self, plugin):
+		if plugin is None:
+			self.activeDesc = ""
+			return
+		desc = plugin.description
+		self.activeDesc = desc
+		self.setCursor(desc)
+
+	def rowActivated(self, treev, path, col):
+		model = treev.get_model()
+		_iter = model.get_iter(path)
+		desc = model.get_value(_iter, 0)
+		self.activeDesc = desc
+		self.response(gtk.ResponseType.OK)
+
+	# def onResponse
+
+
+class FormatButton(gtk.Button):
+	noneLabel = "[Select Format]"
+	dialogTitle = "Select Format"
+
+	def __init__(self, descList: "List[str]", parent=None):
+		gtk.Button.__init__(self)
+		self.set_label(self.noneLabel)
+		###
+		self.descList = descList
+		self._parent = parent
+		self.activePlugin = None
+		###
+		self.connect("clicked", self.onClick)
+
+	def onChanged(self, obj=None):
+		pass
+
+	def onClick(self, button=None):
+		dialog = FormatDialog(
+			descList=self.descList,
+			parent=self._parent,
+			title=self.dialogTitle,
+		)
+		dialog.setActive(self.activePlugin)
+		if dialog.run() != gtk.ResponseType.OK:
+			return
+
+		plugin = dialog.getActive()
+		self.activePlugin = plugin
+		if plugin:
+			self.set_label(plugin.description)
+		else:
+			self.set_label(self.noneLabel)
+		self.onChanged()
+
+	def getActive(self):
+		if self.activePlugin is None:
+			return ""
+		return self.activePlugin.name
+
+	def setActive(self, _format):
+		plugin = Glossary.plugins[_format]
+		self.activePlugin = plugin
+		self.set_label(plugin.description)
+		self.onChanged()
+
+
 class FormatOptionsDialog(gtk.Dialog):
-	def __init__(self, formatName: str, options: "List[str]", optionsValues: "Dict[str, Any]", parent=None):
+	def __init__(
+		self,
+		formatName: str,
+		options: "List[str]",
+		optionsValues: "Dict[str, Any]",
+		parent=None,
+	):
 		gtk.Dialog.__init__(self, parent=parent)
 		optionsProp = Glossary.plugins[formatName].optionsProp
 		self.optionsProp = optionsProp
 		##
 		self.connect("response", lambda w, e: self.hide())
-		dialog_add_button(
-			self,
-			"gtk-ok",
-			"_OK",
-			gtk.ResponseType.OK,
-		)
 		dialog_add_button(
 			self,
 			"gtk-cancel",
 			"_Cancel",
 			gtk.ResponseType.CANCEL,
 		)
+		dialog_add_button(
+			self,
+			"gtk-ok",
+			"_OK",
+			gtk.ResponseType.OK,
+		)
 		###
 		treev = gtk.TreeView()
-		trees = gtk.ListStore(
-			bool, # enable
-			str, # name
-			str, # comment
-			str, # value
+		treeModel = gtk.ListStore(
+			bool,  # enable
+			str,  # name
+			str,  # comment
+			str,  # value
 		)
 		treev.set_headers_clickable(True)
-		treev.set_model(trees)
+		treev.set_model(treeModel)
 		treev.connect("row-activated", self.rowActivated)
 		treev.connect("button-press-event", self.treeviewButtonPress)
 		###
 		self.treev = treev
 		#############
 		cell = gtk.CellRendererToggle()
-		#cell.set_property("activatable", True)
+		# cell.set_property("activatable", True)
 		cell.connect("toggled", self.enableToggled)
 		col = gtk.TreeViewColumn(title="Enable", cell_renderer=cell)
 		col.add_attribute(cell, "active", 0)
-		#cell.set_active(False)
+		# cell.set_active(False)
 		col.set_property("expand", False)
 		col.set_resizable(True)
 		treev.append_column(col)
 		###
-		col = gtk.TreeViewColumn(title="Name", cell_renderer=gtk.CellRendererText(), text=1)
+		col = gtk.TreeViewColumn(
+			title="Name",
+			cell_renderer=gtk.CellRendererText(),
+			text=1,
+		)
 		col.set_property("expand", False)
 		col.set_resizable(True)
 		treev.append_column(col)
@@ -123,29 +347,35 @@ class FormatOptionsDialog(gtk.Dialog):
 		self.valueCell = cell
 		self.valueCol = 3
 		cell.connect("edited", self.valueEdited)
-		col = gtk.TreeViewColumn(title="Value", cell_renderer=cell, text=self.valueCol)
+		col = gtk.TreeViewColumn(
+			title="Value",
+			cell_renderer=cell,
+			text=self.valueCol,
+		)
 		col.set_property("expand", True)
 		col.set_resizable(True)
 		col.set_min_width(200)
 		treev.append_column(col)
 		###
-		col = gtk.TreeViewColumn(title="Comment", cell_renderer=gtk.CellRendererText(), text=2)
+		col = gtk.TreeViewColumn(
+			title="Comment",
+			cell_renderer=gtk.CellRendererText(),
+			text=2,
+		)
 		col.set_property("expand", False)
 		col.set_resizable(False)
 		treev.append_column(col)
 		#############
 		for name in options:
 			prop = optionsProp[name]
-			comment = prop.typ
-			if prop.comment:
-				comment += ", " + prop.comment
-			if prop.typ == "str" and not prop.values:
-				comment += ", double-click to edit"
-			trees.append([
-				name in optionsValues, # enable
-				name, # name
-				comment, # comment
-				str(optionsValues.get(name, "")), # value
+			comment = prop.longComment
+			if prop.typ != "bool" and not prop.values:
+				comment += " (double-click to edit)"
+			treeModel.append([
+				name in optionsValues,  # enable
+				name,  # name
+				comment,  # comment
+				str(optionsValues.get(name, "")),  # value
 			])
 		############
 		pack(self.vbox, treev, 1, 1)
@@ -199,7 +429,7 @@ class FormatOptionsDialog(gtk.Dialog):
 		value = item.get_label()
 		model = self.treev.get_model()
 		model.set_value(itr, self.valueCol, value)
-		model.set_value(itr, 0, True) # enable it
+		model.set_value(itr, 0, True)  # enable it
 
 	def valueCustomOpenDialog(self, itr: gtk.TreeIter, optName: str):
 		model = self.treev.get_model()
@@ -211,18 +441,18 @@ class FormatOptionsDialog(gtk.Dialog):
 		label = gtk.Label(label=f"Value for {optDesc}")
 		dialog = gtk.Dialog(parent=self, title="Option Value")
 		dialog.connect("response", lambda w, e: dialog.hide())
-		dialog_add_button(
-			dialog,
-			"gtk-ok",
-			"_OK",
-			gtk.ResponseType.OK,
-		)
 		dialog_add_button(
 			dialog,
 			"gtk-cancel",
 			"_Cancel",
 			gtk.ResponseType.CANCEL,
 		)
+		dialog_add_button(
+			dialog,
+			"gtk-ok",
+			"_OK",
+			gtk.ResponseType.OK,
+		)
 		pack(dialog.vbox, label, 0, 0)
 		entry = gtk.Entry()
 		entry.set_text(currentValue)
@@ -233,7 +463,7 @@ class FormatOptionsDialog(gtk.Dialog):
 			return
 		value = entry.get_text()
 		model.set_value(itr, self.valueCol, value)
-		model.set_value(itr, 0, True) # enable it
+		model.set_value(itr, 0, True)  # enable it
 
 	def valueItemCustomActivate(self, item: gtk.MenuItem, itr: gtk.TreeIter):
 		model = self.treev.get_model()
@@ -242,7 +472,8 @@ class FormatOptionsDialog(gtk.Dialog):
 
 	def valueCellClicked(self, path, forceMenu=False) -> bool:
 		"""
-		returns True if event is handled, False if not handled (need to enter edit mode)
+		returns True if event is handled, False if not handled
+		(need to enter edit mode)
 		"""
 		model = self.treev.get_model()
 		itr = model.get_iter(path)
@@ -258,7 +489,7 @@ class FormatOptionsDialog(gtk.Dialog):
 					log.error(f"invalid {optName} = {rawValue!r}")
 					value = False
 			model.set_value(itr, self.valueCol, str(not value))
-			model.set_value(itr, 0, True) # enable it
+			model.set_value(itr, 0, True)  # enable it
 			return True
 		propValues = prop.values
 		if not propValues:
@@ -305,7 +536,7 @@ class FormatOptionsDialog(gtk.Dialog):
 		model = self.treev.get_model()
 		optionsValues = {}
 		for row in model:
-			if not row[0]: # not enable
+			if not row[0]:  # not enable
 				continue
 			optName = row[1]
 			rawValue = row[3]
@@ -318,42 +549,23 @@ class FormatOptionsDialog(gtk.Dialog):
 		return optionsValues
 
 
-
-class FormatComboBox(gtk.ComboBox):
-	def __init__(self, parent=None):
-		gtk.ComboBox.__init__(self)
-		self._parent = parent
-		self.model = gtk.ListStore(
-			str,  # format name, hidden
-			# GdkPixbuf.Pixbuf,# icon
-			str,  # format description, shown
-		)
-		self.set_model(self.model)
-
-		cell = gtk.CellRendererText()
-		cell.set_visible(False)
-		pack(self, cell)
-		self.add_attribute(cell, "text", 0)
-
-		# cell = gtk.CellRendererPixbuf()
-		# pack(self, cell, False)
-		# self.add_attribute(cell, "pixbuf", 1)
-
-		cell = gtk.CellRendererText()
-		pack(self, cell, True)
-		self.add_attribute(cell, "text", 1)
+class FormatBox(FormatButton):
+	def __init__(self, descList: "List[str]", parent=None):
+		FormatButton.__init__(self, descList, parent=parent)
 
 		self.optionsValues = {}
 
 		self.optionsButton = gtk.Button(label="Options")
+		self.optionsButton.set_image(gtk.Image.new_from_icon_name(
+			"gtk-preferences",
+			gtk.IconSize.BUTTON,
+		))
 		self.optionsButton.connect("clicked", self.optionsButtonClicked)
 
 		self.dependsButton = gtk.Button(label="Install dependencies")
 		self.dependsButton.pkgNames = []
 		self.dependsButton.connect("clicked", self.dependsButtonClicked)
 
-		self.connect("changed", self.onChanged)
-
 	def setOptionsValues(self, optionsValues: "Dict[str, Any]"):
 		self.optionsValues = optionsValues
 
@@ -364,25 +576,6 @@ class FormatComboBox(gtk.ComboBox):
 	def getActiveOptions(self):
 		raise NotImplementedError
 
-	def addFormat(self, _format):
-		self.get_model().append((
-			_format,
-			# icon,
-			Glossary.plugins[_format].description,
-		))
-
-	def getActive(self):
-		index = gtk.ComboBox.get_active(self)
-		if index is None or index < 0:
-			return ""
-		return self.get_model()[index][0]
-
-	def setActive(self, _format):
-		for i, row in enumerate(self.get_model()):
-			if row[0] == _format:
-				gtk.ComboBox.set_active(self, i)
-				return
-
 	def optionsButtonClicked(self, button):
 		formatName = self.getActive()
 		options = self.getActiveOptions()
@@ -418,8 +611,11 @@ class FormatComboBox(gtk.ComboBox):
 		)
 		self.onChanged(self)
 
-	def onChanged(self, combo):
+	def onChanged(self, obj=None):
 		name = self.getActive()
+		if not name:
+			self.optionsButton.set_visible(False)
+			return
 		self.optionsValues.clear()
 
 		options = self.getActiveOptions()
@@ -428,36 +624,39 @@ class FormatComboBox(gtk.ComboBox):
 		kind = self.kind()
 		plugin = Glossary.plugins[name]
 		if kind == "r":
-			cls = plugin.readerClass
+			depends = plugin.readDepends
 		elif kind == "w":
-			cls = plugin.writerClass
+			depends = plugin.writeDepends
 		else:
 			raise RuntimeError(f"invalid kind={kind}")
-		uninstalled = checkDepends(cls.depends)
+		uninstalled = checkDepends(depends)
 
 		self.dependsButton.pkgNames = uninstalled
 		self.dependsButton.set_visible(bool(uninstalled))
 
 
-class InputFormatComboBox(FormatComboBox):
+class InputFormatBox(FormatBox):
+	dialogTitle = "Select Input Format"
+
 	def __init__(self, **kwargs):
-		FormatComboBox.__init__(self, **kwargs)
-		for _format in Glossary.readFormats:
-			self.addFormat(_format)
+		FormatBox.__init__(self, readDesc, **kwargs)
 
 	def kind(self):
 		"returns 'r' or 'w'"
 		return "r"
 
 	def getActiveOptions(self):
-		return list(Glossary.formatsReadOptions[self.getActive()].keys())
+		formatName = self.getActive()
+		if not formatName:
+			return
+		return list(Glossary.formatsReadOptions[formatName].keys())
 
 
-class OutputFormatComboBox(FormatComboBox):
+class OutputFormatBox(FormatBox):
+	dialogTitle = "Select Output Format"
+
 	def __init__(self, **kwargs):
-		FormatComboBox.__init__(self, **kwargs)
-		for _format in Glossary.writeFormats:
-			self.addFormat(_format)
+		FormatBox.__init__(self, writeDesc, **kwargs)
 
 	def kind(self):
 		"returns 'r' or 'w'"
@@ -472,20 +671,15 @@ class GtkTextviewLogHandler(logging.Handler):
 		logging.Handler.__init__(self)
 
 		self.buffers = {}
-		for levelname in (
-			"CRITICAL",
-			"ERROR",
-			"WARNING",
-			"INFO",
-			"DEBUG",
-		):
-			textview = treeview_dict[levelname]
+		for levelNameCap in log.levelNamesCap[:-1]:
+			levelName = levelNameCap.upper()
+			textview = treeview_dict[levelName]
 
 			buff = textview.get_buffer()
-			tag = gtk.TextTag.new(levelname)
+			tag = gtk.TextTag.new(levelName)
 			buff.get_tag_table().add(tag)
 
-			self.buffers[levelname] = buff
+			self.buffers[levelName] = buff
 
 	def getTag(self, levelname):
 		return self.buffers[levelname].get_tag_table().lookup(levelname)
@@ -526,6 +720,7 @@ class GtkSingleTextviewLogHandler(GtkTextviewLogHandler):
 			"WARNING": textview,
 			"INFO": textview,
 			"DEBUG": textview,
+			"TRACE": textview,
 		})
 
 
@@ -570,13 +765,192 @@ class BrowseButton(gtk.Button):
 		fcd.destroy()
 
 
+sortKeyNameByDesc = {
+	_sk.desc: _sk.name
+	for _sk in namedSortKeyList
+}
+sortKeyNames = [
+	_sk.name for _sk in namedSortKeyList
+]
+
+
+class SortOptionsBox(gtk.Box):
+	def __init__(self, ui):
+		gtk.Box.__init__(self, orientation=gtk.Orientation.VERTICAL)
+		self.ui = ui
+		###
+		hbox = gtk.HBox()
+		sortCheck = gtk.CheckButton("Sort entries by")
+		sortKeyCombo = gtk.ComboBoxText()
+		for _sk in namedSortKeyList:
+			sortKeyCombo.append_text(_sk.desc)
+		sortKeyCombo.set_active(sortKeyNames.index(defaultSortKeyName))
+		sortKeyCombo.set_border_width(0)
+		sortKeyCombo.set_sensitive(False)
+		# sortKeyCombo.connect("changed", self.sortKeyComboChanged)
+		self.sortCheck = sortCheck
+		self.sortKeyCombo = sortKeyCombo
+		sortCheck.connect("clicked", self.onSortCheckClicked)
+		pack(hbox, sortCheck, 0, 0, padding=5)
+		pack(hbox, sortKeyCombo, 0, 0, padding=5)
+		pack(self, hbox, 0, 0, padding=5)
+		###
+		hbox = self.encodingHBox = gtk.HBox()
+		label = gtk.Label(label="Sort Encoding")
+		encodingEntry = self.encodingEntry = gtk.Entry()
+		encodingEntry.set_text("utf-8")
+		encodingEntry.set_width_chars(15)
+		pack(hbox, label, 0, 0, padding=10)
+		pack(hbox, encodingEntry, 0, 0, padding=5)
+		pack(self, hbox, 0, 0, padding=5)
+		###
+		self.show_all()
+
+	def onSortCheckClicked(self, check):
+		sort = check.get_active()
+		self.sortKeyCombo.set_sensitive(sort)
+		self.encodingHBox.set_sensitive(sort)
+
+	def updateWidgets(self):
+		convertOptions = self.ui.convertOptions
+		sort = convertOptions.get("sort")
+		self.sortCheck.set_active(sort)
+		self.sortKeyCombo.set_sensitive(sort)
+		self.encodingHBox.set_sensitive(sort)
+		sortKeyName = convertOptions.get("sortKeyName")
+		if sortKeyName:
+			self.sortKeyCombo.set_active(sortKeyNames.index(sortKeyName))
+		sortEncoding = convertOptions.get("sortEncoding", "utf-8")
+		self.encodingEntry.set_text(sortEncoding)
+
+	def applyChanges(self):
+		convertOptions = self.ui.convertOptions
+		sort = self.sortCheck.get_active()
+		if not sort:
+			for param in ("sort", "sortKeyName", "sortEncoding"):
+				if param in convertOptions:
+					del convertOptions[param]
+			return
+
+		sortKeyDesc = self.sortKeyCombo.get_active_text()
+		convertOptions["sort"] = sort
+		convertOptions["sortKeyName"] = sortKeyNameByDesc[sortKeyDesc]
+		convertOptions["sortEncoding"] = self.encodingEntry.get_text()
+
+
+class GeneralOptionsDialog(gtk.Dialog):
+	def onDeleteEvent(self, widget, event):
+		self.hide()
+		return True
+
+	def onResponse(self, widget, event):
+		self.applyChanges()
+		self.hide()
+		return True
+
+	def __init__(self, ui, **kwargs):
+		gtk.Dialog.__init__(
+			self,
+			transient_for=ui,
+			**kwargs,
+		)
+		self.set_title("General Options")
+		self.ui = ui
+		##
+		self.resize(600, 500)
+		self.connect("delete-event", self.onDeleteEvent)
+		##
+		self.connect("response", self.onResponse)
+		dialog_add_button(
+			self,
+			"gtk-ok",
+			"_OK",
+			gtk.ResponseType.OK,
+		)
+		##
+		hpad = 10
+		vpad = 5
+		##
+		self.sortOptionsBox = SortOptionsBox(ui)
+		pack(self.vbox, self.sortOptionsBox, 0, 0, padding=vpad)
+		##
+		hbox = gtk.HBox()
+		self.sqliteCheck = gtk.CheckButton(label="SQLite mode")
+		pack(hbox, self.sqliteCheck, 0, 0, padding=hpad)
+		pack(self.vbox, hbox, 0, 0, padding=vpad)
+		##
+		self.configParams = OrderedDict([
+			("save_info_json", False),
+			("lower", False),
+			("skip_resources", False),
+			("rtl", False),
+			("enable_alts", True),
+			("cleanup", True),
+			("remove_html_all", True),
+		])
+		self.configCheckButtons = {}
+		configDefDict = UIBase.configDefDict
+		for param, default in self.configParams.items():
+			hbox = gtk.HBox()
+			comment = configDefDict[param].comment
+			checkButton = gtk.CheckButton(
+				label=comment.split("\n")[0]
+			)
+			self.configCheckButtons[param] = checkButton
+			pack(hbox, checkButton, 0, 0, padding=hpad)
+			pack(self.vbox, hbox, 0, 0, padding=vpad)
+		##
+		self.updateWidgets()
+		self.vbox.show_all()
+
+	def getSQLite(self) -> bool:
+		convertOptions = self.ui.convertOptions
+		sqlite = convertOptions.get("sqlite")
+		if sqlite is not None:
+			return sqlite
+		return self.ui.config.get("auto_sqlite", True)
+
+	def updateWidgets(self):
+		config = self.ui.config
+		self.sortOptionsBox.updateWidgets()
+		self.sqliteCheck.set_active(self.getSQLite())
+		for param, check in self.configCheckButtons.items():
+			default = self.configParams[param]
+			check.set_active(config.get(param, default))
+
+	def applyChanges(self):
+		# print("applyChanges")
+		self.sortOptionsBox.applyChanges()
+
+		convertOptions = self.ui.convertOptions
+		config = self.ui.config
+
+		convertOptions["sqlite"] = self.sqliteCheck.get_active()
+
+		for param, check in self.configCheckButtons.items():
+			config[param] = check.get_active()
+
+
+class GeneralOptionsButton(gtk.Button):
+	def __init__(self, ui):
+		gtk.Button.__init__(self, label="General Options")
+		self.ui = ui
+		self.connect("clicked", self.onClick)
+		self.dialog = None
+
+	def onClick(self, widget):
+		if self.dialog is None:
+			self.dialog = GeneralOptionsDialog(self.ui)
+		self.dialog.present()
+
+
 class UI(gtk.Dialog, MyDialog, UIBase):
 	def status(self, msg):
 		# try:
-		#	_id = self.statusMsgDict[msg]
+		# 	_id = self.statusMsgDict[msg]
 		# except KeyError:
-		#	_id = self.statusMsgDict[msg] = self.statusNewId
-		#	self.statusNewId += 1
+		# 	_id = self.statusMsgDict[msg] = self.statusNewId
+		# 	self.statusNewId += 1
 		_id = self.statusBar.get_context_id(msg)
 		self.statusBar.push(_id, msg)
 
@@ -584,19 +958,33 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		gtk.Dialog.__init__(self)
 		UIBase.__init__(self)
 		self.set_title("PyGlossary (Gtk3)")
-		self.resize(800, 800)
+		#####
+		screenW, screenH = getWorkAreaSize()
+		winSize = min(800, screenW - 50, screenH - 50)
+		self.resize(winSize, winSize)
+		# print(f"{screenW}x{screenH}, {'%sx%s' % getScreenSize()}")
+		#####
 		self.connect("delete-event", self.onDeleteEvent)
 		self.pages = []
 		# self.statusNewId = 0
 		# self.statusMsgDict = {}## message -> id
 		#####
-		self._convertOptions = {}
+		self.convertOptions = {}
+		#####
+		self.styleProvider = gtk.CssProvider()
+		gtk.StyleContext.add_provider_for_screen(
+			gdk.Screen.get_default(),
+			self.styleProvider,
+			gtk.STYLE_PROVIDER_PRIORITY_APPLICATION,
+		)
+		css = "check {min-width: 1.25em; min-height: 1.25em;}\n"
+		self.styleProvider.load_from_data(css.encode("utf-8"))
 		#####
 		self.assert_quit = False
 		self.path = ""
-		self.glos = Glossary(ui=self)
 		# ____________________ Tab 1 - Convert ____________________ #
-		sizeGroup = gtk.SizeGroup(mode=gtk.SizeGroupMode.HORIZONTAL)
+		labelSizeGroup = gtk.SizeGroup(mode=gtk.SizeGroupMode.HORIZONTAL)
+		buttonSizeGroup = gtk.SizeGroup(mode=gtk.SizeGroupMode.HORIZONTAL)
 		####
 		vbox = VBox()
 		vbox.label = _("Convert")
@@ -604,21 +992,9 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		self.pages.append(vbox)
 		######
 		hbox = HBox(spacing=3)
-		hbox.label = gtk.Label(label=_("Input Format")+":")
+		hbox.label = gtk.Label(label=_("Input File:"))
 		pack(hbox, hbox.label)
-		sizeGroup.add_widget(hbox.label)
-		hbox.label.set_property("xalign", 0)
-		self.convertInputFormatCombo = InputFormatComboBox(parent=self)
-		pack(hbox, self.convertInputFormatCombo)
-		pack(hbox, gtk.Label(), 1, 1)
-		pack(hbox, self.convertInputFormatCombo.dependsButton)
-		pack(hbox, self.convertInputFormatCombo.optionsButton)
-		pack(vbox, hbox)
-		###
-		hbox = HBox(spacing=3)
-		hbox.label = gtk.Label(label=_("Input File")+":")
-		pack(hbox, hbox.label)
-		sizeGroup.add_widget(hbox.label)
+		labelSizeGroup.add_widget(hbox.label)
 		hbox.label.set_property("xalign", 0)
 		self.convertInputEntry = gtk.Entry()
 		pack(hbox, self.convertInputEntry, 1, 1)
@@ -629,33 +1005,35 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 			title="Select Input File",
 		)
 		pack(hbox, button)
+		buttonSizeGroup.add_widget(button)
 		pack(vbox, hbox)
 		##
 		self.convertInputEntry.connect(
 			"changed",
 			self.convertInputEntryChanged,
 		)
+		###
+		hbox = HBox(spacing=3)
+		hbox.label = gtk.Label(label=_("Input Format:"))
+		pack(hbox, hbox.label)
+		labelSizeGroup.add_widget(hbox.label)
+		hbox.label.set_property("xalign", 0)
+		self.convertInputFormatCombo = InputFormatBox(parent=self)
+		buttonSizeGroup.add_widget(self.convertInputFormatCombo.optionsButton)
+		pack(hbox, self.convertInputFormatCombo)
+		pack(hbox, gtk.Label(), 1, 1)
+		pack(hbox, self.convertInputFormatCombo.dependsButton)
+		pack(hbox, self.convertInputFormatCombo.optionsButton)
+		pack(vbox, hbox)
 		#####
 		vbox.sep1 = gtk.Label(label="")
 		vbox.sep1.show()
 		pack(vbox, vbox.sep1)
 		#####
 		hbox = HBox(spacing=3)
-		hbox.label = gtk.Label(label=_("Output Format")+":")
+		hbox.label = gtk.Label(label=_("Output File:"))
 		pack(hbox, hbox.label)
-		sizeGroup.add_widget(hbox.label)
-		hbox.label.set_property("xalign", 0)
-		self.convertOutputFormatCombo = OutputFormatComboBox(parent=self)
-		pack(hbox, self.convertOutputFormatCombo)
-		pack(hbox, gtk.Label(), 1, 1)
-		pack(hbox, self.convertOutputFormatCombo.dependsButton)
-		pack(hbox, self.convertOutputFormatCombo.optionsButton)
-		pack(vbox, hbox)
-		###
-		hbox = HBox(spacing=3)
-		hbox.label = gtk.Label(label=_("Output File")+":")
-		pack(hbox, hbox.label)
-		sizeGroup.add_widget(hbox.label)
+		labelSizeGroup.add_widget(hbox.label)
 		hbox.label.set_property("xalign", 0)
 		self.convertOutputEntry = gtk.Entry()
 		pack(hbox, self.convertOutputEntry, 1, 1)
@@ -666,22 +1044,43 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 			title="Select Output File",
 		)
 		pack(hbox, button)
+		buttonSizeGroup.add_widget(button)
 		pack(vbox, hbox)
 		##
 		self.convertOutputEntry.connect(
 			"changed",
 			self.convertOutputEntryChanged,
 		)
+		###
+		hbox = HBox(spacing=3)
+		hbox.label = gtk.Label(label=_("Output Format:"))
+		pack(hbox, hbox.label)
+		labelSizeGroup.add_widget(hbox.label)
+		hbox.label.set_property("xalign", 0)
+		self.convertOutputFormatCombo = OutputFormatBox(parent=self)
+		buttonSizeGroup.add_widget(self.convertOutputFormatCombo.optionsButton)
+		pack(hbox, self.convertOutputFormatCombo)
+		pack(hbox, gtk.Label(), 1, 1)
+		pack(hbox, self.convertOutputFormatCombo.dependsButton)
+		pack(hbox, self.convertOutputFormatCombo.optionsButton)
+		pack(vbox, hbox)
 		#####
 		hbox = HBox(spacing=10)
 		label = gtk.Label(label="")
-		pack(hbox, label, 1, 1, 10)
+		pack(hbox, label, 1, 1, 5)
+		##
+		button = GeneralOptionsButton(self)
+		button.set_size_request(300, 40)
+		pack(hbox, button, 0, 0, 0)
+		##
 		self.convertButton = gtk.Button()
 		self.convertButton.set_label("Convert")
 		self.convertButton.connect("clicked", self.convertClicked)
-		pack(hbox, self.convertButton, 1, 1, 10)
+		self.convertButton.set_size_request(300, 40)
+		pack(hbox, self.convertButton, 0, 0, 10)
+		##
 		pack(vbox, hbox, 0, 0, 15)
-		####
+		#####
 		self.convertConsoleTextview = textview = gtk.TextView()
 		swin = gtk.ScrolledWindow()
 		swin.set_policy(gtk.PolicyType.AUTOMATIC, gtk.PolicyType.AUTOMATIC)
@@ -691,7 +1090,7 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		# ____________________ Tab 2 - Reverse ____________________ #
 		self.reverseStatus = ""
 		####
-		sizeGroup = gtk.SizeGroup(mode=gtk.SizeGroupMode.HORIZONTAL)
+		labelSizeGroup = gtk.SizeGroup(mode=gtk.SizeGroupMode.HORIZONTAL)
 		####
 		vbox = VBox()
 		vbox.label = _("Reverse")
@@ -699,18 +1098,18 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		# self.pages.append(vbox)
 		######
 		hbox = HBox(spacing=3)
-		hbox.label = gtk.Label(label=_("Input Format")+":")
+		hbox.label = gtk.Label(label=_("Input Format:"))
 		pack(hbox, hbox.label)
-		sizeGroup.add_widget(hbox.label)
+		labelSizeGroup.add_widget(hbox.label)
 		hbox.label.set_property("xalign", 0)
-		self.reverseInputFormatCombo = InputFormatComboBox()
+		self.reverseInputFormatCombo = InputFormatBox()
 		pack(hbox, self.reverseInputFormatCombo)
 		pack(vbox, hbox)
 		###
 		hbox = HBox(spacing=3)
-		hbox.label = gtk.Label(label=_("Input File")+":")
+		hbox.label = gtk.Label(label=_("Input File:"))
 		pack(hbox, hbox.label)
-		sizeGroup.add_widget(hbox.label)
+		labelSizeGroup.add_widget(hbox.label)
 		hbox.label.set_property("xalign", 0)
 		self.reverseInputEntry = gtk.Entry()
 		pack(hbox, self.reverseInputEntry, 1, 1)
@@ -733,9 +1132,9 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		pack(vbox, vbox.sep1)
 		#####
 		hbox = HBox(spacing=3)
-		hbox.label = gtk.Label(label=_("Output Tabfile")+":")
+		hbox.label = gtk.Label(label=_("Output Tabfile:"))
 		pack(hbox, hbox.label)
-		sizeGroup.add_widget(hbox.label)
+		labelSizeGroup.add_widget(hbox.label)
 		hbox.label.set_property("xalign", 0)
 		self.reverseOutputEntry = gtk.Entry()
 		pack(hbox, self.reverseOutputEntry, 1, 1)
@@ -786,7 +1185,7 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 			logo=logo,
 			header=f"PyGlossary\nVersion {core.VERSION}",
 			# about=summary,
-			about=f'{aboutText}\n<a href="{homePage}">{homePage}</a>',
+			about=f'{aboutText}\n<a href="{core.homePage}">{core.homePage}</a>',
 			authors="\n".join(authors),
 			license=licenseText,
 		)
@@ -799,13 +1198,13 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		self.notebook = notebook
 		#########
 		for vbox in self.pages:
-			l = gtk.Label(label=vbox.label)
-			l.set_use_underline(True)
+			label = gtk.Label(label=vbox.label)
+			label.set_use_underline(True)
 			vb = VBox(spacing=3)
 			if vbox.icon:
 				vbox.image = imageFromFile(vbox.icon)
 				pack(vb, vbox.image)
-			pack(vb, l)
+			pack(vb, label)
 			vb.show_all()
 			notebook.append_page(vbox, vb)
 			try:
@@ -815,11 +1214,11 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		#######################
 		pack(self.vbox, notebook, 1, 1)
 		# for i in ui.pagesOrder:
-		#	try:
-		#		j = pagesOrder[i]
-		#	except IndexError:
-		#		continue
-		#	notebook.reorder_child(self.pages[i], j)
+		# 	try:
+		# 		j = pagesOrder[i]
+		# 	except IndexError:
+		# 		continue
+		# 	notebook.reorder_child(self.pages[i], j)
 		# ____________________________________________________________ #
 		handler = GtkSingleTextviewLogHandler(textview)
 		log.addHandler(handler)
@@ -834,6 +1233,7 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		handler.setColor("WARNING", rgba_parse("yellow"))
 		handler.setColor("INFO", rgba_parse("white"))
 		handler.setColor("DEBUG", rgba_parse("white"))
+		handler.setColor("TRACE", rgba_parse("white"))
 		###
 		textview.get_buffer().set_text("Output & Error Console:\n")
 		textview.set_editable(False)
@@ -864,7 +1264,7 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		# hbox.sepLabel1 = gtk.Label(label="")
 		# pack(hbox, hbox.sepLabel1, 1, 1)
 		######
-		hbox.verbosityLabel = gtk.Label(label=_("Verbosity")+":")
+		hbox.verbosityLabel = gtk.Label(label=_("Verbosity:"))
 		pack(hbox, hbox.verbosityLabel, 0, 0)
 		##
 		self.verbosityCombo = combo = gtk.ComboBoxText()
@@ -902,12 +1302,16 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		inputFormat: str = "",
 		outputFormat: str = "",
 		reverse: bool = False,
-		configOptions: "Optional[Dict]" = None,
+		config: "Optional[Dict]" = None,
 		readOptions: "Optional[Dict]" = None,
 		writeOptions: "Optional[Dict]" = None,
 		convertOptions: "Optional[Dict]" = None,
+		glossarySetAttrs: "Optional[Dict]" = None,
 	):
-		self.loadConfig(**configOptions)
+		if glossarySetAttrs is None:
+			glossarySetAttrs = {}
+
+		self.config = config
 
 		if inputFilename:
 			self.convertInputEntry.set_text(abspath(inputFilename))
@@ -927,9 +1331,11 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		if writeOptions:
 			self.convertOutputFormatCombo.setOptionsValues(writeOptions)
 
-		self._convertOptions = convertOptions
+		self.convertOptions = convertOptions
 		if convertOptions:
-			log.info(f"Using convertOptions={convertOptions}")
+			log.debug(f"Using convertOptions={convertOptions}")
+
+		self._glossarySetAttrs = glossarySetAttrs
 
 		gtk.Dialog.present(self)
 		gtk.main()
@@ -939,7 +1345,7 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		# gtk.main_quit()
 		# if callled while converting, main_quit does not exit program,
 		# it keeps printing warnings,
-		# and makes you close the terminal or force kill the proccess
+		# and makes you close the terminal or force kill the process
 		sys.exit(0)
 
 	def consoleClearButtonClicked(self, widget=None):
@@ -982,22 +1388,27 @@ class UI(gtk.Dialog, MyDialog, UIBase):
 		self.progressTitle = "Converting"
 		readOptions = self.convertInputFormatCombo.optionsValues
 		writeOptions = self.convertOutputFormatCombo.optionsValues
+
+		glos = Glossary(ui=self)
+		glos.config = self.config
+
+		for attr, value in self._glossarySetAttrs.items():
+			setattr(glos, attr, value)
+
+		log.debug(f"readOptions: {readOptions}")
+		log.debug(f"writeOptions: {writeOptions}")
+		log.debug(f"convertOptions: {self.convertOptions}")
+		log.debug(f"config: {self.config}")
+
 		try:
-			# if inFormat=="Omnidic":
-			#	dicIndex = self.xml.get_widget("spinbutton_omnidic_i")\
-			#		.get_value_as_int()
-			#	ex = self.glos.readOmnidic(inPath, dicIndex=dicIndex)
-			# else:
-			log.debug(f"readOptions: {readOptions}")
-			log.debug(f"writeOptions: {writeOptions}")
-			finalOutputFile = self.glos.convert(
+			finalOutputFile = glos.convert(
 				inPath,
 				inputFormat=inFormat,
 				outputFilename=outPath,
 				outputFormat=outFormat,
 				readOptions=readOptions,
 				writeOptions=writeOptions,
-				**self._convertOptions,
+				**self.convertOptions,
 			)
 			if finalOutputFile:
 				self.status("Convert finished")
