@@ -24,44 +24,9 @@ from os.path import join
 from PyQt4 import QtGui as qt
 from PyQt4 import QtCore as qc
 
-stderr_saved = sys.stderr
-stdout_saved = sys.stdout
-
-# startBold = '\x1b[1m'  # Start Bold # len=4
-# startUnderline = '\x1b[4m' # Start Underline # len=4
-endFormat = '\x1b[0;0;0m'  # End Format # len=8
-# redOnGray = '\x1b[0;1;31;47m'
-startRed = '\x1b[31m'
-
 noneItem = 'Not Selected'
 
 
-class QVirtualFile(object):
-	def __init__(self, qtext, mode):
-		self.qtext = qtext
-		self.mode = mode
-
-	def write(self, text):
-		self.qtext.insertPlainText(text)
-		if self.mode == 'stdout':
-			stdout_saved.write(text)
-		elif self.mode == 'stderr':
-			stderr_saved.write(startRed+text+endFormat)
-
-	def writelines(self, lines):
-		for line in lines:
-			self.write(line)
-
-	def flush(self):
-		pass
-
-	def isatty(self):
-		return 1
-
-	def fileno(self):
-		pass
-
-
 class UI(qt.QWidget, UIBase):
 	def __init__(self):
 		qt.QWidget.__init__(self)
@@ -71,6 +36,7 @@ class UI(qt.QWidget, UIBase):
 		######################
 		self.running = False
 		self.glos = Glossary(ui=self)
+		self.glos.config = self.config
 		self.pathI = ''
 		self.pathO = ''
 		self.fcd_dir = join(homeDir, 'Desktop')
