@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ui_tk.py
 #
-# Copyright © 2009-2020 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
+# Copyright © 2009-2021 Saeed Rasooli <saeed.gnu@gmail.com> (ilius)
 #
 # This program is a free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,11 +17,10 @@
 # GNU General Public License for more details.
 
 from pyglossary import core
-from pyglossary.core import homeDir, confDir, dataDir
+from pyglossary.core import homeDir, confDir
 
 from pyglossary.glossary import (
 	Glossary,
-	homePage,
 )
 
 from .base import (
@@ -34,7 +33,7 @@ from .base import (
 
 from pyglossary.text_utils import urlToPath
 import os
-from os.path import join, isfile, abspath
+from os.path import join, isfile, abspath, splitext
 import logging
 import traceback
 
@@ -46,14 +45,6 @@ from tkinter import font as tkFont
 
 log = logging.getLogger("pyglossary")
 
-# startBold = "\x1b[1m"  # Start Bold #len=4
-# startUnderline = "\x1b[4m"  # Start Underline #len=4
-endFormat = "\x1b[0;0;0m"  # End Format #len=8
-# redOnGray = "\x1b[0;1;31;47m"
-startRed = "\x1b[31m"
-
-resDir = join(dataDir, "res")
-
 pluginByDesc = {
 	plugin.description: plugin
 	for plugin in Glossary.plugins.values()
@@ -113,6 +104,28 @@ def centerWindow(win):
 	win.deiconify()
 
 
+def newButton(*args, **kwargs):
+	button = tk.Button(*args, **kwargs)
+
+	def onEnter(event=None):
+		button.invoke()
+
+	button.bind("<Return>", onEnter)
+	button.bind("<KP_Enter>", onEnter)
+	return button
+
+
+def newTTKButton(*args, **kwargs):
+	button = ttk.Button(*args, **kwargs)
+
+	def onEnter(event=None):
+		button.invoke()
+
+	button.bind("<Return>", onEnter)
+	button.bind("<KP_Enter>", onEnter)
+	return button
+
+
 def newLabelWithImage(parent, file=""):
 	image = tk.PhotoImage(file=file)
 	label = ttk.Label(parent, image=image)
@@ -127,24 +140,27 @@ def newReadOnlyText(
 	font=None,
 ):
 	height = len(text.strip().split("\n"))
-	w = tk.Text(
+	widget = tk.Text(
 		parent,
 		height=height,
 		borderwidth=borderwidth,
 		font=font,
 	)
-	w.insert(1.0, text)
-	w.pack()
-	w.configure(state="disabled")
+	widget.insert(1.0, text)
+	widget.pack()
+
+	# widget.bind("<Key>", lambda e: "break")
+	widget.configure(state="disabled")
+
 	# if tkinter is 8.5 or above you'll want the selection background
 	# to appear like it does when the widget is activated
 	# comment this out for older versions of Tkinter
-	w.configure(
-		inactiveselectbackground=w.cget("selectbackground"),
+	widget.configure(
+		inactiveselectbackground=widget.cget("selectbackground"),
 		bg=parent.cget('bg'),
 		relief="flat",
 	)
-	return w
+	return widget
 
 
 class TkTextLogHandler(logging.Handler):
@@ -156,6 +172,7 @@ class TkTextLogHandler(logging.Handler):
 		tktext.tag_config("WARNING", foreground="#ffff00")
 		tktext.tag_config("INFO", foreground="#00ff00")
 		tktext.tag_config("DEBUG", foreground="#ffffff")
+		tktext.tag_config("TRACE", foreground="#ffffff")
 		###
 		self.tktext = tktext
 
@@ -260,10 +277,10 @@ class ProgressBar(tix.Frame):
 		self.bind("<Configure>", self.update)
 		self.canvas.pack(side="top", fill="x", expand="no")
 
-	def updateProgress(self, newVal, newMax=None, text=""):
-		if newMax:
-			self.max = newMax
-		self.value = newVal
+	def updateProgress(self, value, _max=None, text=""):
+		if _max:
+			self.max = _max
+		self.value = value
 		self.update(None, text)
 
 	def update(self, event=None, labelText=""):
@@ -306,112 +323,285 @@ class ProgressBar(tix.Frame):
 		self.canvas.update_idletasks()
 
 
-class FormatOptionsButton(tk.Button):
+class FormatDialog(tix.Toplevel):
 	def __init__(
 		self,
-		kind: "Literal['Read', 'Write']",
-		values: "Dict",
-		formatVar: tk.StringVar,
-		master=None,
+		descList: "List[str]",
+		title: str,
+		onOk: "Callable",
+		button: "FormatButton",
+		activeDesc: str = "",
 	):
-		tk.Button.__init__(
-			self,
-			master=master,
-			text="Options",
-			command=self.buttonClicked,
-			# bg="#f0f000",
-			# activebackground="#f6f622",
-			borderwidth=3,
-		)
-		self.kind = kind
-		self.kindFormatsOptions = {
-			"Read": Glossary.formatsReadOptions,
-			"Write": Glossary.formatsWriteOptions,
-		}
-		self.values = values
-		self.formatVar = formatVar
-		self.menu = None
+		tix.Toplevel.__init__(self)
+		# bg="#0f0" does not work
+		self.descList = descList
+		self.items = self.descList
+		self.onOk = onOk
+		self.activeDesc = activeDesc
+		self.lastSearch = None
+		self.resizable(width=True, height=True)
+		if title:
+			self.title(title)
+		set_window_icon(self)
+		self.bind('<Escape>', lambda e: self.destroy())
 
-	def setOptionsValues(self, values):
-		self.values = values
-
-	def valueMenuItemCustomSelected(self, treev, format, optName, menu=None):
-		if menu:
-			menu.destroy()
-			self.menu = None
-
-		value = treev.set(optName, self.valueCol)
-
-		dialog = tix.Toplevel(master=treev)  # bg="#0f0" does not work
-		dialog.resizable(width=True, height=True)
-		dialog.title(optName)
-		set_window_icon(dialog)
-		dialog.bind('<Escape>', lambda e: dialog.destroy())
-
-		px, py, pw, ph = decodeGeometry(treev.winfo_toplevel().geometry())
-		w = 300
-		h = 100
-		dialog.geometry(encodeGeometry(
-			px + pw // 2 - w // 2,
-			py + ph // 2 - h // 2,
-			w,
-			h,
+		px, py, pw, ph = decodeGeometry(button.winfo_toplevel().geometry())
+		width = 400
+		height = 400
+		self.geometry(encodeGeometry(
+			px + pw // 2 - width // 2,
+			py + ph // 2 - height // 2,
+			width,
+			height,
 		))
 
-		frame = tix.Frame(master=dialog)
+		entryBox = tk.Frame(master=self)
+		label = ttk.Label(master=entryBox, text="Search: ")
+		label.pack(side="left")
+		entry = self.entry = ttk.Entry(master=entryBox)
+		entry.pack(fill="x", expand=True, side="left")
+		entryBox.pack(fill="x", padx=5, pady=5)
 
-		label = ttk.Label(master=frame, text="Value for " + optName)
-		label.pack()
+		entry.bind("<KeyRelease>", self.onEntryKeyRelease)
+		entry.focus()
 
-		entry = ttk.Entry(master=frame)
-		entry.insert(0, value)
-		entry.pack(fill="x")
+		treevBox = tk.Frame(master=self)
 
-		prop = Glossary.plugins[format].optionsProp[optName]
+		treev = self.treev = ttk.Treeview(
+			master=treevBox,
+			columns=["Description"],
+			show="",
+		)
+		treev.bind("<Double-1>", self.onTreeDoubleClick)
+		treev.pack(
+			side="left",
+			fill="both",
+			expand=True,
+		)
 
-		def okClicked(event=None):
-			rawValue = entry.get()
-			if not prop.validateRaw(rawValue):
-				log.error(f"invalid {prop.typ} value: {optName} = {rawValue!r}")
-				return
-			treev.set(optName, self.valueCol, rawValue)
-			treev.set(optName, "#1", "1")  # enable it
-			col_w = tkFont.Font().measure(rawValue)
-			if treev.column("Value", width=None) < col_w:
-				treev.column("Value", width=col_w)
-			dialog.destroy()
+		vsb = ttk.Scrollbar(
+			master=treevBox,
+			orient="vertical",
+			command=treev.yview,
+		)
+		vsb.pack(side="right", fill="y")
 
-		entry.bind("<Return>", okClicked)
+		treevBox.pack(
+			fill="both",
+			expand=True,
+			padx=5,
+			pady=5,
+		)
 
-		label = ttk.Label(master=frame)
-		label.pack(fill="x")
+		treev.configure(yscrollcommand=vsb.set)
 
-		button = ttk.Button(
-			frame,
-			text="OK",
-			command=okClicked,
+		self.updateTree()
+
+		buttonBox = tix.Frame(master=self)
+
+		cancelButton = newTTKButton(
+			buttonBox,
+			text="Cancel",
+			command=self.cancelClicked,
+		)
+		cancelButton.pack(side="right")
+
+		okButton = newTTKButton(
+			buttonBox,
+			text="  OK  ",
+			command=self.okClicked,
 			# bg="#ff0000",
 			# activebackground="#ff5050",
 		)
-		button.pack(side="right")
-		###
-		frame.pack(fill="x")
+		okButton.pack(side="right")
+
+		buttonBox.pack(fill="x")
+
+		self.bind("<Return>", self.onReturnPress)
+		self.bind("<KP_Enter>", self.onReturnPress)
+
+		self.bind("<Down>", self.onDownPress)
+		self.bind("<Up>", self.onUpPress)
+
+		# self.bind("<KeyPress>", self.onKeyPress)
+
+	def setActiveRow(self, desc):
+		self.treev.selection_set(desc)
+		self.treev.see(desc)
+
+	def updateTree(self):
+		treev = self.treev
+		current = treev.get_children()
+		if current:
+			treev.delete(*current)
+		for desc in self.items:
+			treev.insert("", "end", values=[desc], iid=desc)
+			# iid should be rowId
+		if self.activeDesc in self.items:
+			self.setActiveRow(self.activeDesc)
+
+	def onEntryKeyRelease(self, event):
+		text = self.entry.get().strip()
+		if text == self.lastSearch:
+			return
+
+		if not text:
+			self.items = self.descList
+			self.updateTree()
+			self.lastSearch = text
+			return
+
+		text = text.lower()
+		descList = self.descList
+
+		items1 = []
+		items2 = []
+		for desc in descList:
+			if desc.lower().startswith(text):
+				items1.append(desc)
+			elif text in desc.lower():
+				items2.append(desc)
+
+		self.items = items1 + items2
+		self.updateTree()
+		self.lastSearch = text
+
+	def onTreeDoubleClick(self, event):
+		self.okClicked()
+
+	def cancelClicked(self):
+		self.destroy()
+
+	def onReturnPress(self, event):
+		self.okClicked()
+
+	def onDownPress(self, event):
+		treev = self.treev
+		selection = treev.selection()
+		if selection:
+			nextDesc = treev.next(selection[0])
+			if nextDesc:
+				self.setActiveRow(nextDesc)
+		else:
+			if self.items:
+				self.setActiveRow(self.items[0])
+		treev.focus()
+
+	def onUpPress(self, event):
+		treev = self.treev
+		treev.focus()
+		selection = treev.selection()
+		if not selection:
+			if self.items:
+				self.setActiveRow(self.items[0])
+			return
+		nextDesc = treev.prev(selection[0])
+		if nextDesc:
+			self.setActiveRow(nextDesc)
+
+	def onKeyPress(self, event):
+		print(f"FormatDialog: onKeyPress: {event}")
+
+	def okClicked(self):
+		treev = self.treev
+		selectedList = treev.selection()
+		if selectedList:
+			desc = selectedList[0]
+		else:
+			desc = ""
+		self.onOk(desc)
+		self.destroy()
+
+
+class FormatButton(ttk.Button):
+	noneLabel = "[Select Format]"
+
+	def __init__(
+		self,
+		descList: "List[str]",
+		dialogTitle: str,
+		onChange: "Callable",
+		master=None,
+	):
+		self.var = tk.StringVar()
+		self.var.set(self.noneLabel)
+		ttk.Button.__init__(
+			self,
+			master=master,
+			textvariable=self.var,
+			command=self.onClick,
+		)
+		self.descList = descList
+		self.dialogTitle = dialogTitle
+		self._onChange = onChange
+		self.activeDesc = ""
+		self.bind("<Return>", self.onEnter)
+		self.bind("<KP_Enter>", self.onEnter)
+
+	def onEnter(self, event=None):
+		self.invoke()
+
+	def onChange(self, desc):
+		self.set(desc)
+		self._onChange(desc)
+
+	def get(self):
+		return self.activeDesc
+
+	def set(self, desc):
+		if desc:
+			self.var.set(desc)
+		else:
+			self.var.set(self.noneLabel)
+		self.activeDesc = desc
+
+	def onClick(self):
+		dialog = FormatDialog(
+			descList=self.descList,
+			title=self.dialogTitle,
+			onOk=self.onChange,
+			button=self,
+			activeDesc=self.activeDesc,
+		)
 		dialog.focus()
 
-	def buttonClicked(self):
-		formatD = self.formatVar.get()
-		if not formatD:
-			return
-		format = pluginByDesc[formatD].name
-		options = list(self.kindFormatsOptions[self.kind][format].keys())
-		optionsProp = Glossary.plugins[format].optionsProp
 
-		dialog = tix.Toplevel()  # bg="#0f0" does not work
-		dialog.resizable(width=True, height=True)
-		dialog.title(self.kind + " Options")
-		set_window_icon(dialog)
-		dialog.bind('<Escape>', lambda e: dialog.destroy())
-		###
+class FormatOptionsDialog(tix.Toplevel):
+	kindFormatsOptions = {
+		"Read": Glossary.formatsReadOptions,
+		"Write": Glossary.formatsWriteOptions,
+	}
+
+	def __init__(self, format, kind, values, master=None):
+		tix.Toplevel.__init__(self)
+		# bg="#0f0" does not work
+		self.resizable(width=True, height=True)
+		self.title(kind + " Options")
+		set_window_icon(self)
+		self.bind('<Escape>', lambda e: self.destroy())
+
+		self.menu = None
+		self.format = format
+		self.kind = kind
+		self.values = values
+		self.options = list(self.kindFormatsOptions[kind][format].keys())
+		self.optionsProp = Glossary.plugins[format].optionsProp
+
+		self.createOptionsList()
+
+		buttonBox = tix.Frame(self)
+		okButton = newTTKButton(
+			buttonBox,
+			text="  OK  ",
+			command=self.okClicked,
+			# bg="#ff0000",
+			# activebackground="#ff5050",
+		)
+		okButton.pack(side="right")
+		buttonBox.pack(fill="x")
+
+	def createOptionsList(self):
+		values = self.values
 		self.valueCol = "#3"
 		cols = [
 			"Enable",  # bool
@@ -419,8 +609,8 @@ class FormatOptionsButton(tk.Button):
 			"Value",  # str
 			"Comment",  # str
 		]
-		treev = ttk.Treeview(
-			master=dialog,
+		treev = self.treev = ttk.Treeview(
+			master=self,
 			columns=cols,
 			show="headings",
 		)
@@ -436,142 +626,20 @@ class FormatOptionsButton(tk.Button):
 				width=tkFont.Font().measure(col.title()),
 			)
 		###
-
-		def valueMenuItemSelected(optName, menu, value):
-			treev.set(optName, self.valueCol, value)
-			treev.set(optName, "#1", "1")  # enable it
-			col_w = tkFont.Font().measure(value)
-			if treev.column("Value", width=None) < col_w:
-				treev.column("Value", width=col_w)
-			menu.destroy()
-			self.menu = None
-
-		def valueCellClicked(event, optName):
-			if not optName:
-				return
-			prop = optionsProp[optName]
-			propValues = prop.values
-			if not propValues:
-				if prop.customValue:
-					self.valueMenuItemCustomSelected(treev, format, optName, None)
-				else:
-					log.error(
-						f"invalid option {optName}, values={propValues}"
-						f", customValue={prop.customValue}"
-					)
-				return
-			if prop.typ == "bool":
-				rawValue = treev.set(optName, self.valueCol)
-				if rawValue == "":
-					value = False
-				else:
-					value, isValid = prop.evaluate(rawValue)
-					if not isValid:
-						log.error(f"invalid {optName} = {rawValue!r}")
-						value = False
-				treev.set(optName, self.valueCol, str(not value))
-				treev.set(optName, "#1", "1")  # enable it
-				return
-			menu = tk.Menu(
-				master=treev,
-				title=optName,
-				tearoff=False,
-			)
-			self.menu = menu  # to destroy it later
-			if prop.customValue:
-				menu.add_command(
-					label="[Custom Value]",
-					command=lambda: self.valueMenuItemCustomSelected(
-						treev,
-						format,
-						optName,
-						menu,
-					),
-				)
-			groupedValues = None
-			if len(propValues) > 10:
-				groupedValues = prop.groupValues()
-			maxItemW = 0
-			if groupedValues:
-				for groupName, subValues in groupedValues.items():
-					if subValues is None:
-						menu.add_command(
-							label=str(value),
-							command=lambda value=value: valueMenuItemSelected(optName, menu, value),
-						)
-						maxItemW = max(maxItemW, tkFont.Font().measure(str(value)))
-					else:
-						subMenu = tk.Menu(tearoff=False)
-						for subValue in subValues:
-							subMenu.add_command(
-								label=str(subValue),
-								command=lambda value=subValue: valueMenuItemSelected(
-									optName,
-									menu,
-									value,
-								),
-							)
-						menu.add_cascade(label=groupName, menu=subMenu)
-						maxItemW = max(maxItemW, tkFont.Font().measure(groupName))
-			else:
-				for value in propValues:
-					value = str(value)
-					menu.add_command(
-						label=value,
-						command=lambda value=value: valueMenuItemSelected(optName, menu, value),
-					)
-
-			def close():
-				menu.destroy()
-				self.menu = None
-
-			menu.add_command(
-				label="[Close]",
-				command=close,
-			)
-			try:
-				menu.tk_popup(
-					event.x_root,
-					event.y_root,
-				)
-				# do not pass the third argument (entry), so that the menu
-				# apears where the pointer is on its top-left corner
-			finally:
-				# make sure to release the grab (Tk 8.0a1 only)
-				menu.grab_release()
-
-		def treeClicked(event):
-			if self.menu:
-				self.menu.destroy()
-				self.menu = None
-				return
-			optName = treev.identify_row(event.y)  # optName is rowId
-			if not optName:
-				return
-			col = treev.identify_column(event.x)  # "#1" to self.valueCol
-			if col == "#1":
-				value = treev.set(optName, col)
-				treev.set(optName, col, 1 - int(value))
-				return
-			if col == self.valueCol:
-				valueCellClicked(event, optName)
-
 		treev.bind(
 			"<Button-1>",
 			# "<<TreeviewSelect>>", # event.x and event.y are zero
-			treeClicked,
+			self.treeClicked,
 		)
 		treev.pack(fill="x", expand=True)
 		###
-		for optName in options:
-			prop = optionsProp[optName]
-			comment = prop.typ
-			if prop.comment:
-				comment += ", " + prop.comment
+		for optName in self.options:
+			prop = self.optionsProp[optName]
+			comment = prop.longComment
 			row = [
-				int(optName in self.values),
+				int(optName in values),
 				optName,
-				str(self.values.get(optName, "")),
+				str(values.get(optName, "")),
 				comment,
 			]
 			treev.insert("", "end", values=row, iid=optName)  # iid should be rowId
@@ -584,37 +652,244 @@ class FormatOptionsButton(tk.Button):
 				col_w = tkFont.Font().measure(value)
 				if treev.column(cols[col_i], width=None) < col_w:
 					treev.column(cols[col_i], width=col_w)
-		###########
-		frame = tix.Frame(dialog)
-		###
 
-		def okClicked():
-			for optName in options:
-				enable = bool(int(treev.set(optName, "#1")))
-				if not enable:
-					if optName in self.values:
-						del self.values[optName]
-					continue
-				rawValue = treev.set(optName, self.valueCol)
-				prop = optionsProp[optName]
-				value, isValid = prop.evaluate(rawValue)
-				if not isValid:
-					log.error(f"invalid option value {optName} = {rawValue}")
-					continue
-				self.values[optName] = value
+	def valueMenuItemCustomSelected(self, treev, format, optName, menu=None):
+		if menu:
+			menu.destroy()
+			self.menu = None
+
+		value = treev.set(optName, self.valueCol)
+
+		dialog = tix.Toplevel(master=treev)  # bg="#0f0" does not work
+		dialog.resizable(width=True, height=True)
+		dialog.title(optName)
+		set_window_icon(dialog)
+		dialog.bind('<Escape>', lambda e: dialog.destroy())
+
+		px, py, pw, ph = decodeGeometry(treev.winfo_toplevel().geometry())
+		width = 300
+		height = 100
+		dialog.geometry(encodeGeometry(
+			px + pw // 2 - width // 2,
+			py + ph // 2 - height // 2,
+			width,
+			height,
+		))
+
+		frame = tix.Frame(master=dialog)
+
+		label = ttk.Label(master=frame, text="Value for " + optName)
+		label.pack()
+
+		entry = ttk.Entry(master=frame)
+		entry.insert(0, value)
+		entry.pack(fill="x")
+
+		prop = Glossary.plugins[format].optionsProp[optName]
+
+		def customOkClicked(event=None):
+			rawValue = entry.get()
+			if not prop.validateRaw(rawValue):
+				log.error(f"invalid {prop.typ} value: {optName} = {rawValue!r}")
+				return
+			treev.set(optName, self.valueCol, rawValue)
+			treev.set(optName, "#1", "1")  # enable it
+			col_w = tkFont.Font().measure(rawValue)
+			if treev.column("Value", width=None) < col_w:
+				treev.column("Value", width=col_w)
 			dialog.destroy()
 
-		button = ttk.Button(
+		entry.bind("<Return>", customOkClicked)
+
+		label = ttk.Label(master=frame)
+		label.pack(fill="x")
+
+		customOkbutton = newTTKButton(
 			frame,
-			text="OK",
-			command=okClicked,
+			text="  OK  ",
+			command=customOkClicked,
 			# bg="#ff0000",
 			# activebackground="#ff5050",
 		)
-		button.pack(side="right")
+		customOkbutton.pack(side="right")
 		###
 		frame.pack(fill="x")
-		###
+		dialog.focus()
+
+	def valueMenuItemSelected(self, optName, menu, value):
+		treev = self.treev
+		treev.set(optName, self.valueCol, value)
+		treev.set(optName, "#1", "1")  # enable it
+		col_w = tkFont.Font().measure(value)
+		if treev.column("Value", width=None) < col_w:
+			treev.column("Value", width=col_w)
+		menu.destroy()
+		self.menu = None
+
+	def valueCellClicked(self, event, optName):
+		if not optName:
+			return
+		treev = self.treev
+		prop = self.optionsProp[optName]
+		propValues = prop.values
+		if not propValues:
+			if prop.customValue:
+				self.valueMenuItemCustomSelected(treev, self.format, optName, None)
+			else:
+				log.error(
+					f"invalid option {optName}, values={propValues}"
+					f", customValue={prop.customValue}"
+				)
+			return
+		if prop.typ == "bool":
+			rawValue = treev.set(optName, self.valueCol)
+			if rawValue == "":
+				value = False
+			else:
+				value, isValid = prop.evaluate(rawValue)
+				if not isValid:
+					log.error(f"invalid {optName} = {rawValue!r}")
+					value = False
+			treev.set(optName, self.valueCol, str(not value))
+			treev.set(optName, "#1", "1")  # enable it
+			return
+		menu = tk.Menu(
+			master=treev,
+			title=optName,
+			tearoff=False,
+		)
+		self.menu = menu  # to destroy it later
+		if prop.customValue:
+			menu.add_command(
+				label="[Custom Value]",
+				command=lambda: self.valueMenuItemCustomSelected(
+					treev,
+					self.format,
+					optName,
+					menu,
+				),
+			)
+		groupedValues = None
+		if len(propValues) > 10:
+			groupedValues = prop.groupValues()
+		maxItemW = 0
+
+		def valueMenuItemSelectedCommand(value):
+			def callback():
+				self.valueMenuItemSelected(optName, menu, value)
+			return callback
+
+		if groupedValues:
+			for groupName, subValues in groupedValues.items():
+				if subValues is None:
+					menu.add_command(
+						label=str(value),
+						command=valueMenuItemSelectedCommand(value),
+					)
+					maxItemW = max(maxItemW, tkFont.Font().measure(str(value)))
+				else:
+					subMenu = tk.Menu(tearoff=False)
+					for subValue in subValues:
+						subMenu.add_command(
+							label=str(subValue),
+							command=valueMenuItemSelectedCommand(subValue),
+						)
+					menu.add_cascade(label=groupName, menu=subMenu)
+					maxItemW = max(maxItemW, tkFont.Font().measure(groupName))
+		else:
+			for value in propValues:
+				value = str(value)
+				menu.add_command(
+					label=value,
+					command=valueMenuItemSelectedCommand(value),
+				)
+
+		def close():
+			menu.destroy()
+			self.menu = None
+
+		menu.add_command(
+			label="[Close]",
+			command=close,
+		)
+		try:
+			menu.tk_popup(
+				event.x_root,
+				event.y_root,
+			)
+			# do not pass the third argument (entry), so that the menu
+			# apears where the pointer is on its top-left corner
+		finally:
+			# make sure to release the grab (Tk 8.0a1 only)
+			menu.grab_release()
+
+	def treeClicked(self, event):
+		treev = self.treev
+		if self.menu:
+			self.menu.destroy()
+			self.menu = None
+			return
+		optName = treev.identify_row(event.y)  # optName is rowId
+		if not optName:
+			return
+		col = treev.identify_column(event.x)  # "#1" to self.valueCol
+		if col == "#1":
+			value = treev.set(optName, col)
+			treev.set(optName, col, 1 - int(value))
+			return
+		if col == self.valueCol:
+			self.valueCellClicked(event, optName)
+
+	def okClicked(self):
+		treev = self.treev
+		for optName in self.options:
+			enable = bool(int(treev.set(optName, "#1")))
+			if not enable:
+				if optName in self.values:
+					del self.values[optName]
+				continue
+			rawValue = treev.set(optName, self.valueCol)
+			prop = self.optionsProp[optName]
+			value, isValid = prop.evaluate(rawValue)
+			if not isValid:
+				log.error(f"invalid option value {optName} = {rawValue}")
+				continue
+			self.values[optName] = value
+		self.destroy()
+
+
+class FormatOptionsButton(tk.Button):
+	def __init__(
+		self,
+		kind: "Literal['Read', 'Write']",
+		values: "Dict",
+		formatInput: "FormatButton",
+		master=None,
+	):
+		tk.Button.__init__(
+			self,
+			master=master,
+			text="Options",
+			command=self.buttonClicked,
+			# bg="#f0f000",
+			# activebackground="#f6f622",
+			borderwidth=3,
+		)
+		self.kind = kind
+		self.values = values
+		self.formatInput = formatInput
+
+	def setOptionsValues(self, values):
+		self.values = values
+
+	def buttonClicked(self):
+		formatD = self.formatInput.get()
+		if not formatD:
+			return
+		format = pluginByDesc[formatD].name
+
+		dialog = FormatOptionsDialog(format, self.kind, self.values, master=self)
+
 		# x, y, w, h = decodeGeometry(dialog.geometry())
 		w, h = 380, 250
 		# w and h are rough estimated width and height of `dialog`
@@ -644,6 +919,12 @@ class UI(tix.Frame, UIBase):
 		########
 		set_window_icon(rootWin)
 		rootWin.bind('<Escape>', lambda e: rootWin.quit())
+		#########
+		# Linux: ('clam', 'alt', 'default', 'classic')
+		# Windows: ('winnative', 'clam', 'alt', 'default', 'classic', 'vista',
+		#           'xpnative')
+		# style = ttk.Style()
+		# style.theme_use("default")
 		########
 		self.pack(fill="x")
 		# rootWin.bind("<Configure>", self.resized)
@@ -653,9 +934,10 @@ class UI(tix.Frame, UIBase):
 			defaultFont.configure(size=int(defaultFont.cget("size") * 1.4))
 		####
 		self.biggerFont = defaultFont.copy()
-		self.biggerFont.configure(size=int(defaultFont.cget("size") * 1.5))
+		self.biggerFont.configure(size=int(defaultFont.cget("size") * 1.8))
 		######################
 		self.glos = Glossary(ui=self)
+		self.glos.config = self.config
 		self._convertOptions = {}
 		self.pathI = ""
 		self.pathO = ""
@@ -674,43 +956,28 @@ class UI(tix.Frame, UIBase):
 		notebook.add("tabAbout", label="About", underline=0)
 		convertFrame = tix.Frame(notebook.tabConvert)
 		aboutFrame = tix.Frame(notebook.tabAbout)
-		######################
-		row = 0
-		label = ttk.Label(convertFrame, text="Input Format: ")
-		label.grid(row=row, column=0, sticky=tk.W)
-		##
-		comboVar = tk.StringVar()
-		combo = ttk.OptionMenu(
-			convertFrame,
-			comboVar,
-			None,  # default
-			*readDesc,
-		)
-		combo.grid(row=row, column=1, sticky=tk.W)
-		comboVar.trace("w", self.inputComboChanged)
-		self.formatVarInputConvert = comboVar
-		##
-		self.readOptions = {}  # type: Dict[str, Any]
-		self.writeOptions = {}  # type: Dict[str, Any]
-		##
-		self.readOptionsButton = FormatOptionsButton(
-			"Read",
-			self.readOptions,
-			self.formatVarInputConvert,
-			master=convertFrame,
-		)
-		self.inputFormatRow = row
 		###################
-		row += 1
+		row = 0
 		label = ttk.Label(convertFrame, text="Input File: ")
-		label.grid(row=row, column=0, sticky=tk.W)
+		label.grid(
+			row=row,
+			column=0,
+			sticky=tk.W,
+			padx=5,
+		)
 		##
 		entry = tix.Entry(convertFrame)
-		entry.grid(row=row, column=1, columnspan=2, sticky=tk.W + tk.E)
+		entry.grid(
+			row=row,
+			column=1,
+			columnspan=2,
+			sticky=tk.W + tk.E,
+			padx=0,
+		)
 		entry.bind_all("<KeyPress>", self.anyEntryChanged)
 		self.entryInputConvert = entry
 		##
-		button = tk.Button(
+		button = newButton(
 			convertFrame,
 			text="Browse",
 			command=self.browseInputConvert,
@@ -718,45 +985,107 @@ class UI(tix.Frame, UIBase):
 			# activebackground="#f6f622",
 			borderwidth=3,
 		)
-		button.grid(row=row, column=3, sticky=tk.W + tk.E)
+		button.grid(
+			row=row,
+			column=3,
+			sticky=tk.W + tk.E,
+			padx=5,
+		)
+		######################
+		row += 1
+		label = ttk.Label(convertFrame, text="Input Format: ")
+		label.grid(
+			row=row,
+			column=0,
+			sticky=tk.W,
+			padx=5,
+		)
+		##
+		self.formatButtonInputConvert = FormatButton(
+			master=convertFrame,
+			descList=readDesc,
+			dialogTitle="Select Input Format",
+			onChange=self.inputFormatChanged,
+		)
+		self.formatButtonInputConvert.grid(
+			row=row,
+			column=1,
+			columnspan=2,
+			sticky=tk.W,
+			padx=0,
+		)
+		##
+		self.readOptions = {}  # type: Dict[str, Any]
+		self.writeOptions = {}  # type: Dict[str, Any]
+		##
+		self.readOptionsButton = FormatOptionsButton(
+			"Read",
+			self.readOptions,
+			self.formatButtonInputConvert,
+			master=convertFrame,
+		)
+		self.inputFormatRow = row
 		######################
 		row += 1
 		label = ttk.Label(convertFrame)
-		label.grid(row=row, column=0, sticky=tk.W)
+		label.grid(
+			row=row,
+			column=0,
+			sticky=tk.W,
+		)
 		######################
 		row += 1
 		label = ttk.Label(convertFrame, text="Output Format: ")
-		label.grid(row=row, column=0, sticky=tk.W)
+		label.grid(
+			row=row,
+			column=0,
+			sticky=tk.W,
+			padx=5,
+		)
 		##
-		comboVar = tk.StringVar()
-		combo = ttk.OptionMenu(
-			convertFrame,
-			comboVar,
-			None,  # default
-			*writeDesc,
+		self.formatButtonOutputConvert = FormatButton(
+			master=convertFrame,
+			descList=writeDesc,
+			dialogTitle="Select Output Format",
+			onChange=self.outputFormatChanged,
+		)
+		self.formatButtonOutputConvert.grid(
+			row=row,
+			column=1,
+			columnspan=2,
+			sticky=tk.W,
+			padx=0,
 		)
-		combo.grid(row=row, column=1, columnspan=2, sticky=tk.W)
-		comboVar.trace("w", self.outputComboChanged)
-		self.formatVarOutputConvert = comboVar
 		##
 		self.writeOptionsButton = FormatOptionsButton(
 			"Write",
 			self.writeOptions,
-			self.formatVarOutputConvert,
+			self.formatButtonOutputConvert,
 			master=convertFrame,
 		)
 		self.outputFormatRow = row
 		###################
 		row += 1
 		label = ttk.Label(convertFrame, text="Output File: ")
-		label.grid(row=row, column=0, sticky=tk.W)
+		label.grid(
+			row=row,
+			column=0,
+			sticky=tk.W,
+			padx=5,
+		)
 		##
 		entry = tix.Entry(convertFrame)
-		entry.grid(row=row, column=1, columnspan=2, sticky=tk.W + tk.E)
+		entry.grid(
+			row=row,
+			column=1,
+			columnspan=2,
+			sticky=tk.W + tk.E,
+			padx=0,
+		)
 		entry.bind_all("<KeyPress>", self.anyEntryChanged)
 		self.entryOutputConvert = entry
 		##
-		button = tk.Button(
+		button = newButton(
 			convertFrame,
 			text="Browse",
 			command=self.browseOutputConvert,
@@ -764,28 +1093,30 @@ class UI(tix.Frame, UIBase):
 			# activebackground="#f6f622",
 			borderwidth=3,
 		)
-		button.grid(row=row, column=3, sticky=tk.W + tk.E)
+		button.grid(
+			row=row,
+			column=3,
+			sticky=tk.W + tk.E,
+			padx=5,
+		)
 		###################
 		row += 1
-		label = ttk.Label(convertFrame)
-		label.grid(row=row, column=0, sticky=tk.W)
-		###################
-		row += 1
-		button = tk.Button(
+		button = newButton(
 			convertFrame,
 			text="Convert",
 			command=self.convert,
-			background="#00e000",
-			activebackground="#22f022",
-			borderwidth=5,
+			# background="#00e000",
+			# activebackground="#22f022",
+			borderwidth=7,
 			font=self.biggerFont,
-			# pady=10,
+			padx=5, pady=5,
 		)
 		button.grid(
 			row=row,
 			column=2,
 			columnspan=3,
 			sticky=tk.W + tk.E + tk.S,
+			padx=5, pady=5,
 		)
 		# print(f"row number for Convert button: {row}")
 		######
@@ -793,7 +1124,13 @@ class UI(tix.Frame, UIBase):
 		# convertFrame.grid(sticky=tk.W + tk.E + tk.N + tk.S)
 		#################
 		row += 1
-		console = tix.Text(convertFrame, height=15, background="#000", foreground="#fff")
+		console = tix.Text(
+			convertFrame,
+			height=15,
+			background="#000",
+			foreground="#fff",
+		)
+		console.bind("<Key>", self.consoleKeyPress)
 		# self.consoleH = 15
 		# sbar = Tix.Scrollbar(
 		# 	convertFrame,
@@ -807,13 +1144,13 @@ class UI(tix.Frame, UIBase):
 			column=0,
 			columnspan=4,
 			sticky=tk.W + tk.E,
+			padx=5, pady=0,
 		)
 		log.addHandler(
 			TkTextLogHandler(console),
 		)
 		console.insert("end", "Console:\n")
 		####
-		console.grid()
 		self.console = console
 		##################
 		aboutFrame2 = tix.Frame(aboutFrame)
@@ -837,8 +1174,8 @@ class UI(tix.Frame, UIBase):
 		authorsFrame = tk.Frame(aboutNotebook)
 		licenseFrame = tk.Frame(aboutNotebook)
 
-		# tabImg = tk.PhotoImage(file=join(resDir, "dialog-information-22.png"))
-		# tabImg = tk.PhotoImage(file=join(resDir, "author-22.png"))
+		# tabImg = tk.PhotoImage(file=join(dataDir, "res", "dialog-information-22.png"))
+		# tabImg = tk.PhotoImage(file=join(dataDir, "res", "author-22.png"))
 
 		aboutNotebook.add(
 			aboutFrame3,
@@ -858,7 +1195,7 @@ class UI(tix.Frame, UIBase):
 
 		label = newReadOnlyText(
 			aboutFrame3,
-			text=f"{aboutText}\nHome page: {homePage}",
+			text=f"{aboutText}\nHome page: {core.homePage}",
 			font=("DejaVu Sans", 11, ""),
 		)
 		label.pack(fill="x")
@@ -903,7 +1240,7 @@ class UI(tix.Frame, UIBase):
 		# _________________________________________________________________ #
 
 		statusBarframe = tk.Frame(self, borderwidth=3)
-		clearB = tk.Button(
+		clearB = newButton(
 			statusBarframe,
 			text="Clear",
 			command=self.console_clear,
@@ -944,6 +1281,23 @@ class UI(tix.Frame, UIBase):
 		else:  # Linux
 			rootWin.deiconify()
 
+	def textSelectAll(self, tktext):
+		tktext.tag_add(tk.SEL, "1.0", tk.END)
+		tktext.mark_set(tk.INSERT, "1.0")
+		tktext.see(tk.INSERT)
+
+	def consoleKeyPress(self, e):
+		# print(e.state, e.keysym)
+		if e.state > 0:
+			if e.keysym == "c":
+				return
+			if e.keysym == "a":
+				self.textSelectAll(self.console)
+				return "break"
+		if e.keysym == "Escape":
+			return
+		return "break"
+
 	def verbosityChanged(self, index, value, op):
 		log.setVerbosity(
 			int(self.verbosityCombo.get())
@@ -961,8 +1315,8 @@ class UI(tix.Frame, UIBase):
 		# 	if "info" in x:
 		# 		log.debug(x)
 
-	def inputComboChanged(self, *args):
-		formatDesc = self.formatVarInputConvert.get()
+	def inputFormatChanged(self, *args):
+		formatDesc = self.formatButtonInputConvert.get()
 		if not formatDesc:
 			return
 		self.readOptions.clear()  # reset the options, DO NOT re-assign
@@ -972,25 +1326,42 @@ class UI(tix.Frame, UIBase):
 				row=self.inputFormatRow,
 				column=3,
 				sticky=tk.W + tk.E,
+				padx=5, pady=0,
 			)
 		else:
 			self.readOptionsButton.grid_forget()
 
-	def outputComboChanged(self, *args):
-		# log.debug(self.formatVarOutputConvert.get())
-		formatDesc = self.formatVarOutputConvert.get()
+	def outputFormatChanged(self, *args):
+		formatDesc = self.formatButtonOutputConvert.get()
 		if not formatDesc:
 			return
-		self.writeOptions.clear()  # reset the options, DO NOT re-assign
+
 		format = pluginByDesc[formatDesc].name
+		plugin = Glossary.plugins.get(format)
+		if not plugin:
+			log.error(f"plugin {format} not found")
+			return
+
+		self.writeOptions.clear()  # reset the options, DO NOT re-assign
 		if Glossary.formatsWriteOptions[format]:
 			self.writeOptionsButton.grid(
 				row=self.outputFormatRow,
-				column=3, sticky=tk.W + tk.E,
+				column=3,
+				sticky=tk.W + tk.E,
+				padx=5, pady=0,
 			)
 		else:
 			self.writeOptionsButton.grid_forget()
 
+		pathI = self.entryInputConvert.get()
+		if pathI and not self.entryOutputConvert.get():
+			if self.formatButtonInputConvert.get() and plugin.extensionCreate:
+				pathNoExt, ext = splitext(pathI)
+				self.entryOutputConvert.insert(
+					0,
+					pathNoExt + plugin.extensionCreate,
+				)
+
 	def anyEntryChanged(self, event=None):
 		self.inputEntryChanged()
 		self.outputEntryChanged()
@@ -998,43 +1369,49 @@ class UI(tix.Frame, UIBase):
 	def inputEntryChanged(self, event=None):
 		# char = event.keysym
 		pathI = self.entryInputConvert.get()
-		if self.pathI != pathI:
-			if pathI.startswith("file://"):
-				pathI = urlToPath(pathI)
-				self.entryInputConvert.delete(0, "end")
-				self.entryInputConvert.insert(0, pathI)
-			if self.config["ui_autoSetFormat"]:
-				formatDesc = self.formatVarInputConvert.get()
-				if not formatDesc:
-					inputArgs = Glossary.detectInputFormat(pathI, quiet=True)
-					if inputArgs:
-						format = inputArgs[1]
-						plugin = Glossary.plugins.get(format)
-						if plugin:
-							self.formatVarInputConvert.set(plugin.description)
-			self.pathI = pathI
+		if self.pathI == pathI:
+			return
+
+		if pathI.startswith("file://"):
+			pathI = urlToPath(pathI)
+			self.entryInputConvert.delete(0, "end")
+			self.entryInputConvert.insert(0, pathI)
+		if self.config["ui_autoSetFormat"]:
+			formatDesc = self.formatButtonInputConvert.get()
+			if not formatDesc:
+				inputArgs = Glossary.detectInputFormat(pathI, quiet=True)
+				if inputArgs:
+					format = inputArgs[1]
+					plugin = Glossary.plugins.get(format)
+					if plugin:
+						self.formatButtonInputConvert.set(plugin.description)
+						self.inputFormatChanged()
+		self.pathI = pathI
 
 	def outputEntryChanged(self, event=None):
 		pathO = self.entryOutputConvert.get()
-		if self.pathO != pathO:
-			if pathO.startswith("file://"):
-				pathO = urlToPath(pathO)
-				self.entryOutputConvert.delete(0, "end")
-				self.entryOutputConvert.insert(0, pathO)
-			if self.config["ui_autoSetFormat"]:
-				formatDesc = self.formatVarOutputConvert.get()
-				if not formatDesc:
-					outputArgs = Glossary.detectOutputFormat(
-						filename=pathO,
-						inputFilename=self.entryInputConvert.get(),
-						quiet=True,
+		if self.pathO == pathO:
+			return
+
+		if pathO.startswith("file://"):
+			pathO = urlToPath(pathO)
+			self.entryOutputConvert.delete(0, "end")
+			self.entryOutputConvert.insert(0, pathO)
+		if self.config["ui_autoSetFormat"]:
+			formatDesc = self.formatButtonOutputConvert.get()
+			if not formatDesc:
+				outputArgs = Glossary.detectOutputFormat(
+					filename=pathO,
+					inputFilename=self.entryInputConvert.get(),
+					quiet=True,
+				)
+				if outputArgs:
+					outputFormat = outputArgs[1]
+					self.formatButtonOutputConvert.set(
+						Glossary.plugins[outputFormat].description
 					)
-					if outputArgs:
-						outputFormat = outputArgs[1]
-						self.formatVarOutputConvert.set(
-							Glossary.plugins[outputFormat].description
-						)
-			self.pathO = pathO
+					self.outputFormatChanged()
+		self.pathO = pathO
 
 	def save_fcd_dir(self):
 		if not self.fcd_dir:
@@ -1065,7 +1442,7 @@ class UI(tix.Frame, UIBase):
 		if not inPath:
 			log.critical("Input file path is empty!")
 			return
-		inFormatDesc = self.formatVarInputConvert.get()
+		inFormatDesc = self.formatButtonInputConvert.get()
 		if not inFormatDesc:
 			# log.critical("Input format is empty!");return
 			inFormat = ""
@@ -1076,12 +1453,15 @@ class UI(tix.Frame, UIBase):
 		if not outPath:
 			log.critical("Output file path is empty!")
 			return
-		outFormatDesc = self.formatVarOutputConvert.get()
+		outFormatDesc = self.formatButtonOutputConvert.get()
 		if not outFormatDesc:
 			log.critical("Output format is empty!")
 			return
 		outFormat = pluginByDesc[outFormatDesc].name
 
+		for attr, value in self._glossarySetAttrs.items():
+			setattr(self.glos, attr, value)
+
 		finalOutputFile = self.glos.convert(
 			inPath,
 			inputFormat=inFormat,
@@ -1104,26 +1484,38 @@ class UI(tix.Frame, UIBase):
 		inputFormat: str = "",
 		outputFormat: str = "",
 		reverse: bool = False,
-		configOptions: "Optional[Dict]" = None,
+		config: "Optional[Dict]" = None,
 		readOptions: "Optional[Dict]" = None,
 		writeOptions: "Optional[Dict]" = None,
 		convertOptions: "Optional[Dict]" = None,
+		glossarySetAttrs: "Optional[Dict]" = None,
 	):
+		if glossarySetAttrs is None:
+			glossarySetAttrs = {}
+
+		self.config = config
+
 		if inputFilename:
 			self.entryInputConvert.insert(0, abspath(inputFilename))
+			self.inputEntryChanged()
 		if outputFilename:
 			self.entryOutputConvert.insert(0, abspath(outputFilename))
+			self.outputEntryChanged()
 
 		if inputFormat:
-			self.formatVarInputConvert.set(Glossary.plugins[inputFormat].description)
+			self.formatButtonInputConvert.set(
+				Glossary.plugins[inputFormat].description,
+			)
+			self.inputFormatChanged()
 		if outputFormat:
-			self.formatVarOutputConvert.set(Glossary.plugins[outputFormat].description)
+			self.formatButtonOutputConvert.set(
+				Glossary.plugins[outputFormat].description,
+			)
+			self.outputFormatChanged()
 
 		if reverse:
 			log.error(f"Tkinter interface does not support Reverse feature")
 
-		self.loadConfig(**configOptions)
-
 		# must be before setting self.readOptions and self.writeOptions
 		self.anyEntryChanged()
 
@@ -1139,6 +1531,8 @@ class UI(tix.Frame, UIBase):
 		if convertOptions:
 			log.info(f"Using convertOptions={convertOptions}")
 
+		self._glossarySetAttrs = glossarySetAttrs
+
 		# inputFilename and readOptions are for DB Editor
 		# which is not implemented
 		self.mainloop()
